/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
package searchtool;

import searchtool.util.*;
import searchtool.*;
import searchtool.wrappers.*;
import java.awt.event.*;
import java.io.*;

/**
 * @author thorsten
 * 
 *  this class stores the settings of buttons that affect one list only.
 *
 */
public class ButtonTracker extends XMLStorable {

	// VALUES
	public static final int IP = SearchtoolTableModel.IP,
		PING = SearchtoolTableModel.PING,
		MAP = SearchtoolTableModel.MAP,
		CURRENTPLAYERS = SearchtoolTableModel.CURRENTPLAYERS,
		MAXPLAYERS = SearchtoolTableModel.MAXPLAYERS,
		SERVERNAME = SearchtoolTableModel.SERVERNAME;

	private boolean pingEnabled, sortEnabled, filterEnabled, reverseSortEnabled;

	// VALUES
	public static final String ENABLED = "ENABLED", DISABLED = "DISABLED";

	// KEYS
	public static final String SORT_BUTTON = "SORTING",
		SORTING_CRITERION = "SORTING_CRITERION",
		PING_BUTTON = "PINGING",
		LOCAL_FILTER_BUTTON = "LOCAL_FILTER",
		REVERSE_SORT = "REVERSE_SORT";

	public ButtonTracker(File SETTINGSFILE) {
		super(SETTINGSFILE);
		initValues();
		load();
	}

	public boolean pingEnabled() {
		return pingEnabled;
	}

	public boolean filterEnabled() {
		return filterEnabled;
	}

	public boolean sortEnabled() {
		return sortEnabled;
	}

	public boolean reverseSortEnabled() {
		return reverseSortEnabled;
	}

	public void setValue(String KEY, String VALUE) {
		super.setValue(KEY, VALUE);
		if (KEY.equals(this.LOCAL_FILTER_BUTTON))
			if (VALUE.equals(this.ENABLED))
				filterEnabled = true;
			else
				filterEnabled = false;

		if (KEY.equals(this.SORT_BUTTON))
			if (VALUE.equals(this.ENABLED))
				sortEnabled = true;
			else
				sortEnabled = false;

		if (KEY.equals(this.REVERSE_SORT))
			if (VALUE.equals(this.ENABLED))
				reverseSortEnabled = true;
			else
				reverseSortEnabled = false;

		if (KEY.equals(this.PING_BUTTON))
			if (VALUE.equals(this.ENABLED))
				pingEnabled = true;
			else
				pingEnabled = false;

	}

	protected void initValues() {
		setValue(PING_BUTTON, DISABLED);
		pingEnabled = false;
		setValue(SORT_BUTTON, DISABLED);
		sortEnabled = false;
		setValue(LOCAL_FILTER_BUTTON, DISABLED);
		filterEnabled = false;
		setValue(SORTING_CRITERION, IP);
		setValue(REVERSE_SORT, DISABLED);
		reverseSortEnabled = false;
	}
}
