/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package searchtool;

import java.util.*;
import searchtool.wrappers.*;
import searchtool.interfaces.*;
import searchtool.protocol.*;
import searchtool.util.*;
import searchtool.components.*;
// thirdparty
import gnu.trove.*;

/**
 * @author thorsten
 *
 */
public class DownloadServerlist {

	private ElementController elementController = null;
	private GameController gameController = null;
	private String downloadTaskName = null;
	private TaskObject taskObject = null;
	private FinalCall finalCall = null;
	private SearchtoolSettings settings = null;

	public DownloadServerlist(ElementController elementController, GameController gameController, FinalCall finalCall) {
		this.elementController = elementController;
		this.finalCall = finalCall;
		this.gameController = gameController;
		downloadTaskName = gameController.DOWNLOAD_SERVERLIST_TASK_NAME;
		settings = gameController.getSearchtoolSettings();
	}

	/**
	 *  selfexplaining
	 */
	public void startDownload(ServerRequest serverRequest) {
		// please inner class
		final ServerRequest finalRequest = serverRequest;
		StoppableThread thread = new StoppableThread() {
			boolean holded = false;
			GUI gui = Searchtool.getGui();
			int counter = 0;
			public boolean isHolded() {
				return holded;
			}
			public void hold() {
				holded = true;
			}
			public void run() {
				try {
					// get the current (main!!!) list
					TLongObjectHashMap oldMainList = elementController.getCompleteServerList();
					// clear the old list but register the copy at the ServerRequest object to
					// keep old data.
					elementController.clearList(elementController.MAIN_LIST);
					
					finalRequest.cleanUp();
					finalRequest.registerOldElementList(oldMainList);
					
					while (finalRequest.hasNext() && holded == false) {
						ServerElement[] result = null;
						try {
							if (settings.getValueAsInt(SearchtoolSettings.MASTERLEVEL_FILTER) == SearchtoolSettings.ENABLED) {
								result = finalRequest.getNext(true);
							} else {
								result = finalRequest.getNext(false);
							}
						} catch (Exception e) {
							new SearchtoolDialogs.ErrorDialog("exception while receiving serverlist:\n" + e.getMessage()).showErrorDialog();
							break;
						}
						for (int i = 0; i < result.length; i++) {
							counter++;
							elementController.addElement(result[i]);
							gui.giveMessage("Received " + counter + " server addresses.");
						}
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
				hold();
			}
		};
		TaskObject taskObject = new TaskObject(downloadTaskName, thread, finalCall);
		taskObject.startTask();
		Searchtool.getTaskManager().addTask(taskObject);
	}

	/**
	 *  selfexplaining
	 */
	public void stopDownload() {
		Searchtool.getTaskManager().stopAndRemoveTask(downloadTaskName);
	}

	/**
	 *  return the name of this task;
	 */
	public String getDownloadTaskName() {
		return downloadTaskName;
	}

}
