/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
 * Searchtool.java
 *
 * Created on December 16, 2001, 2:13 PM
 */

package searchtool;

import searchtool.halflife.components.*;
import searchtool.components.*;
import searchtool.interfaces.*;
import searchtool.halflife.*;
import searchtool.ut2003.*;
import searchtool.util.*;
import searchtool.wrappers.*;
import searchtool.halflife.wrappers.*;
import searchtool.unreal_tournament.*;
import searchtool.tribes2.*;
import searchtool.actions.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import java.awt.*;
import java.net.*;
import com.incors.plaf.kunststoff.*;
/**
 *
 * @author  zoidberg
 * @version
 */
public class Searchtool {

	private static TaskManager taskmanager = new TaskManager();
	private static GUI gui;
	private static SearchtoolSplash searchtoolSplash = null;
	private static JProgressBar splashProgressBar = new JProgressBar();
	private static GameController currentGameController = null;
	private static SearchtoolSettings globalSettings = null;

	public static void main(String[] args) {

		// install kunststoff look and feel
		UIManager.installLookAndFeel("Kunststoff", "com.incors.plaf.kunststoff.KunststoffLookAndFeel");
		System.setProperty("file.encoding","ISO-8859-1");

		// check install stuff
		String parameter = null;
		if (args.length > 0) {
			parameter = args[0];
			Constants.configDirectoryPath = parameter;
			Constants.configDirectory = parameter + File.separator + Constants.configDirectoryName;
			Constants.searchtoolConfigFile = new File(Constants.configDirectory + File.separator + "searchtool.conf");
			Constants.versionFile = new File(Constants.configDirectory + File.separator + "VERSION");
		}
		
		// test/update installation
		SearchtoolInstallation.testInstall(Constants.configDirectoryPath);

		// load global settings
		globalSettings = new SearchtoolSettings(Searchtool.Constants.searchtoolConfigFile);
		globalSettings.load();

		// setup splash-screen
		searchtoolSplash = new SearchtoolSplash();
		searchtoolSplash.setProgressBar(splashProgressBar);
		searchtoolSplash.setProgressBarText("Initializing GUI");
		searchtoolSplash.setVisible(true);
		gui = new GUI();

		// init last game controller
		String currentGame = globalSettings.getValue(SearchtoolSettings.CURRENT_GAME);
		openGameFrontend(currentGame, true);

		gui.setVisible(true);

		// remove splash-screen
		searchtoolSplash.dispose();
	}

	public static void openGameFrontend(String name, boolean blockingMode) {
		if (blockingMode) {
			if (currentGameController != null) {
				gui.removeGameFrontend(name);
				gui.setVisible(false);
				searchtoolSplash.clearProgressBar();
				searchtoolSplash.setVisible(true);
				searchtoolSplash.setProgressBarText("Saving " + currentGameController.gamePrefix + " data");
				currentGameController.safeExit();
				currentGameController = null;	
			}
		
			if (name.equals(Constants.UNREAL_TOURNAMENT_PREFIX)) {
				// UNREAL
				searchtoolSplash.setVisible(true);
				GameController unrealGameController = new UnrealTournamentGameController(Searchtool.Constants.UNREAL_TOURNAMENT_PREFIX);
				unrealGameController.initComponents();				
				unrealGameController.load(splashProgressBar);
				currentGameController = unrealGameController;
			}
			/*
			else if (gameFrontendName.equals(Constants.TRIBES2_PREFIX)) {
				// TRIBES2
				searchtoolSplash.setVisible(true);
				GameController tribes2GameController = new Tribes2GameController(Searchtool.Constants.TRIBES2_PREFIX);
				tribes2GameController.initComponents();				
				tribes2GameController.load(splashProgressBar);
				currentGameController = tribes2GameController;
			}
			*/
			else if (name.equals(Constants.UT2003_PREFIX)) {
				// UNREAL_TOURNAMENT_2003
				searchtoolSplash.setVisible(true);
				GameController ut2003GameController = new Ut2003GameController(Searchtool.Constants.UT2003_PREFIX);
				ut2003GameController.initComponents();				
				ut2003GameController.load(splashProgressBar);
				currentGameController = ut2003GameController;
			}
			else /* (gameFrontendName.equals(Constants.HALFLIFE_PREFIX))*/ {
				// HL
				
				// set name to default
				name = Searchtool.Constants.HALFLIFE_PREFIX;
				
				searchtoolSplash.setVisible(true);
				HlGameController hlGameController = new HlGameController(Searchtool.Constants.HALFLIFE_PREFIX);
				hlGameController.initComponents();
				hlGameController.load(splashProgressBar);
				currentGameController = hlGameController;
			}
			GameFrontend currentFrontend = gui.getGameFrontend(name);
			gui.displayFrontend(currentFrontend);
			globalSettings.setValue(SearchtoolSettings.CURRENT_GAME,name);
			searchtoolSplash.setVisible(false);
			gui.setVisible(true);
		
		} else {
			final String gameFrontendName = name;
			new Thread() {
				public void run() {
					openGameFrontend(gameFrontendName, true);
				}
			}
			.start();
		}
	}

	/**
	 * save all data for a secure exit and do exit.
	 */
	public static void safeExit() {
		// safe exit on game controller
		currentGameController.safeExit();
		// reset sleep value
		globalSettings.setValue(SearchtoolSettings.SLEEPING, SearchtoolSettings.DISABLED);
		// save global settings
		globalSettings.save();
		// bye
		System.exit(0);
	}

	/**
	 *  make searchtool sleep in order to use less resources
	 */
	public static void sleep(boolean value) {
		if (value) {
			//sleep
			globalSettings.setValue(SearchtoolSettings.SLEEPING, SearchtoolSettings.ENABLED);
		} else {
			// weakup
			globalSettings.setValue(SearchtoolSettings.SLEEPING, SearchtoolSettings.DISABLED);
		}
	}

	public static TaskManager getTaskManager() {
		return taskmanager;
	}

	public static GUI getGui() {
		return gui;
	}
	public static SearchtoolSettings getGlobalSettings() {
		return globalSettings;
	}

	/**
	 * inner class full of constants used all
	 * over this program.
	 */
	public static class Constants {
		public static final String HALFLIFE_PREFIX = "HL";
		public static final String UNREAL_TOURNAMENT_PREFIX = "UT";
		public static final String TRIBES2_PREFIX = "T2";
		public static final String UT2003_PREFIX= "UT2k3";
		public static final String[] PREFIXES = new String[] { HALFLIFE_PREFIX, UNREAL_TOURNAMENT_PREFIX/*,TRIBES2_PREFIX*/,UT2003_PREFIX };
		public static final String[][] frontendPrefixNameMap;
		static{
			frontendPrefixNameMap = new String[3][2];
			frontendPrefixNameMap[0][0] = HALFLIFE_PREFIX;	
			frontendPrefixNameMap[0][1] = "Half-Life";
			frontendPrefixNameMap[1][0] = UNREAL_TOURNAMENT_PREFIX;	
			frontendPrefixNameMap[1][1] = "Unreal-Tournament";
			//frontendPrefixNameMap[2][0] = TRIBES2_PREFIX;
			//frontendPrefixNameMap[2][1] = "Tribes2";
			frontendPrefixNameMap[2][0] = UT2003_PREFIX;
			frontendPrefixNameMap[2][1] = "Unreal-Tournament 2003";
		}
		public static final String configDirectoryName = ".searchtool";
		public static String configDirectoryPath;
		public static String configDirectory;
		public static final String version = "0.4.4";
		public final static String serverListTag = "SEARCHTOOL_SERVERLIST";
		public final static String versionTag = "VERSION";
		public final static String serverCountTag = "SERVERCOUNT";
		public final static String elementTypeTag = "ELEMENT_TYPE";
		public final static String serverTag = "SRV";
		public final static String elementType = "SERVER_ELEMENT";
		public final static String serverListVersion = "0.4";
		public final static Color backgroundColor = new Color(230, 230, 230);
		public final static String commentString = "searchtool serverlist - please do not edit by hand unless you know what you mess up";

		public final static String ipAtt = "IP",
			portAtt = "PRT",
			pingAtt = "PNG",
			nameAtt = "NME",
			mapAtt = "MAP",
			currentPlayersAtt = "CPL",
			maxPlayersAtt = "MPL",
			passwordAtt = "PWD",
			numberAtt = "NUMBER",
			dateAtt = "DTE",
			connectedAtt = "CON",
			favouriteAtt = "FVR",
			blacklistAtt = "OUT";

		public static File versionFile;
		public static File searchtoolConfigFile;

		static {
			// default
			Properties properties = System.getProperties();
			configDirectoryPath = properties.getProperty("user.home");
			configDirectory = configDirectoryPath + File.separator + configDirectoryName;
			searchtoolConfigFile = new File(configDirectory + File.separator + "searchtool.conf");
			versionFile = new File(configDirectory + File.separator + "VERSION");
		}
	}
}
