/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*
 *
 * Created on January 8, 2002, 7:43 PM
 */

package searchtool;

import java.util.*;
import searchtool.interfaces.*;
import searchtool.protocol.*;
import searchtool.wrappers.*;
import searchtool.halflife.protocol.*;

/**
 * this class gets all the nice info on a list of servers:
 * ping and details.
 *
 * @author  zoidberg
 * @version
 */

public class SearchtoolInfoCollector implements StoppableThread {

	private SearchtoolProtocol ping;
	private SearchtoolProtocol details;
	private SearchtoolProtocol rules;
	private Element[] queryUs;
	private int queryNumber;

	private volatile boolean holded = false;
	private volatile int counter = -1; // init
	private volatile int maximum;

	public SearchtoolInfoCollector(Element[] queryUs, SearchtoolProtocol ping, SearchtoolProtocol details, SearchtoolProtocol rules, int queryNumber) {
		this.ping = ping;
		this.details = details;
		this.rules = rules;
		this.queryUs = queryUs;
		this.queryNumber = queryNumber;
		maximum = queryUs.length - 1;
	}

	public boolean isHolded() {
		return holded;
	}

	public void hold() {
		holded = true;
	}

	private synchronized int getNextPosition() {
		counter++;
		return counter;
	}

	public void run() {
		for (int i = 0; i < queryNumber; i++) {
			try {
				SearchtoolProtocol newPingInstance = null;
				SearchtoolProtocol newDetailsInstance = null;
				SearchtoolProtocol newRulesInstance = null;
				if (ping != null)
					newPingInstance = (SearchtoolProtocol) ping.getClass().newInstance();
				if (details != null)
					newDetailsInstance = (SearchtoolProtocol) details.getClass().newInstance();
				if (rules != null)
					newRulesInstance = (SearchtoolProtocol) rules.getClass().newInstance();
				Thread thread = new SingleQueryThread(newPingInstance, newDetailsInstance, newRulesInstance);
				thread.start();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	private class SingleQueryThread extends Thread {

		private SearchtoolProtocol singlePing;
		private SearchtoolProtocol singleDetails;
		private SearchtoolProtocol singleRules;

		public SingleQueryThread(SearchtoolProtocol ping, SearchtoolProtocol details, SearchtoolProtocol rules) {
			singlePing = ping;
			singleDetails = details;
			singleRules = rules;
		}

		private boolean queryNext() {
			int queryMe = getNextPosition();
			if (queryMe > maximum || holded)
				return false;
			else {
				// prepare
				if (queryUs[queryMe] instanceof ServerElement) {
					if (singlePing != null)
						singlePing.setServerElement((ServerElement) queryUs[queryMe]);
					if (singleDetails != null)
						singleDetails.setServerElement((ServerElement) queryUs[queryMe]);
					if (singleRules != null)
						singleRules.setServerElement((ServerElement) queryUs[queryMe]);
				}
				if (queryUs[queryMe] instanceof SearchtoolPlayer) {
					if (singlePing != null)
						singlePing.setServerElement(((SearchtoolPlayer) queryUs[queryMe]).getServerElement());
					if (singleDetails != null)
						singleDetails.setServerElement(((SearchtoolPlayer) queryUs[queryMe]).getServerElement());
					if (singleRules != null)
						singleRules.setServerElement((ServerElement) queryUs[queryMe]);
				}

				// PING.  try ping twice and mark server as non-responsive if no ping comes back.
				// (no matter if any other packages come back)
				if (singlePing != null) {
					if (!singlePing.update()) {
						singlePing.getServerElement().setResponse(ServerElement.RESPONSE_NEGATIVE);
						if (singlePing.update()) {
							singlePing.getServerElement().setResponse(ServerElement.RESPONSE_POSITIVE);
						} else
							;
						//System.out.println("unresponsive server");
					} else {
						singlePing.getServerElement().setResponse(ServerElement.RESPONSE_POSITIVE);
					}
				}

				// DETAILS       
				if (singleDetails != null) {
					if (!singleDetails.update())
						singleDetails.update();
				}

				if (singleRules != null) {
					if (!singleRules.update())
						singleRules.update();
				}

				return true;
			}
		}

		public void run() {
			while (queryNext())
				this.yield();
			hold();
		}
	}
}
