/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * SearchtoolInstallation.java
 *
 * Created on March 12, 2002, 2:32 PM
 */

package searchtool;

import searchtool.interfaces.*;
import searchtool.actions.*;
import searchtool.halflife.*;
import searchtool.components.*;
import searchtool.wrappers.*;
import searchtool.halflife.wrappers.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import java.awt.*;
import java.net.*;

/**
 * this class holds static methods that handle everything concerning
 * the installation (procedure).
 *
 * @author  zoidberg
 * @version
 */
public class SearchtoolInstallation {

	protected static void checkThatEveryFileIsInPlace() {
		File versionFile = new File(Searchtool.Constants.configDirectory + File.separator + "VERSION");
		if (!versionFile.exists()) {
			// setup default version file
			try {
				versionFile.createNewFile();
				if (getInstalledVersionNumber() == null || !getInstalledVersionNumber().equals(Searchtool.Constants.version))
					writeToFile(versionFile, Searchtool.Constants.version);
			} catch (Exception e) {
				complainAbout("could not create VERSION file :\n" + e.getMessage());
			}
		}
		String installedVersion = getInstalledVersionNumber();
		if (!installedVersion.equals(Searchtool.Constants.version)) {
			writeToFile(versionFile, Searchtool.Constants.version);
		}
		File configFile = new File(Searchtool.Constants.configDirectory + File.separator + "searchtool.conf");
		if (!configFile.exists()) {
			// setup default config file
			try {
				configFile.createNewFile();
				writeToFile(configFile, "<init></init>");
			} catch (Exception e) {
				complainAbout("could not create searchtool.conf file :\n" + e.getMessage());
			}
		}
		// check if every supported game has its correct config directory
		String[] prefixes = Searchtool.Constants.PREFIXES;
		for (int i = 0; i < prefixes.length; i++) {
			File configdir = new File(Searchtool.Constants.configDirectory + File.separator + prefixes[i]);
			if (!configdir.exists()) {
				// if missing, create default one
				try {
					if (!configdir.mkdir())
						throw new Exception("");
				} catch (Exception e) {
					complainAbout("could not create config directory for " + prefixes[i] + ":\n" + e.getMessage());
				}
			}
			try {
				// check every file
				File testfile = new File(Searchtool.Constants.configDirectory + File.separator + prefixes[i] + File.separator + GameController.blacklistButtonFileName);
				if (!testfile.exists()) {
					if (!testfile.createNewFile())
						complainAbout("could not create file in " + prefixes[i]);
					writeToFile(testfile, "<init></init>");
				}
				testfile = new File(Searchtool.Constants.configDirectory + File.separator + prefixes[i] + File.separator + GameController.configFileName);
				if (!testfile.exists()) {
					if (!testfile.exists())
						if (!testfile.createNewFile())
							complainAbout("could not create file in " + prefixes[i]);
					writeToFile(testfile, "<init></init>");
				}
				testfile = new File(Searchtool.Constants.configDirectory + File.separator + prefixes[i] + File.separator + GameController.favouriteListButtonFileName);
				if (!testfile.exists()) {
					if (!testfile.exists())
						if (!testfile.createNewFile())
							complainAbout("could not create file in " + prefixes[i]);
					writeToFile(testfile, "<init></init>");
				}
				testfile = new File(Searchtool.Constants.configDirectory + File.separator + prefixes[i] + File.separator + GameController.lastConnectedListButtonFileName);
				if (!testfile.exists()) {
					if (!testfile.exists())
						if (!testfile.createNewFile())
							complainAbout("could not create file in " + prefixes[i]);
					writeToFile(testfile, "<init></init>");
				}
				testfile = new File(Searchtool.Constants.configDirectory + File.separator + prefixes[i] + File.separator + GameController.mainListButtonFileName);
				if (!testfile.exists()) {
					if (!testfile.exists())
						if (!testfile.createNewFile())
							complainAbout("could not create file in " + prefixes[i]);
					writeToFile(testfile, "<init></init>");
				}
				testfile = new File(Searchtool.Constants.configDirectory + File.separator + prefixes[i] + File.separator + GameController.resultListButtonFileName);
				if (!testfile.exists()) {
					if (!testfile.exists())
						if (!testfile.createNewFile())
							complainAbout("could not create file in " + prefixes[i]);
					writeToFile(testfile, "<init></init>");
				}
				testfile = new File(Searchtool.Constants.configDirectory + File.separator + prefixes[i] + File.separator + GameController.rulesConfigFileName);
				if (!testfile.exists()) {
					if (!testfile.exists())
						if (!testfile.createNewFile())
							complainAbout("could not create file in " + prefixes[i]);
					writeToFile(testfile, "<init></init>");
				}
				testfile = new File(Searchtool.Constants.configDirectory + File.separator + prefixes[i] + File.separator + GameController.serverFileName);
				if (!testfile.exists()) {
					if (!testfile.exists())
						if (!testfile.createNewFile())
							complainAbout("could not create file in " + prefixes[i]);
					writeToFile(testfile, "<init></init>");
				}
			} catch (Exception e) {
				complainAbout("could not create file in " + prefixes[i] + ":\n" + e.getMessage());
			}
		}
	}

	/** 
	 *  give error message and exit
	 */
	private static void complainAbout(String message) {
		new SearchtoolDialogs.ErrorDialog(message).showErrorDialog();
		System.exit(1);
	}

	/**
	 * if we do need a new installation, this method does the basic stuff.
	 * returns false if directory could not be created.
	 */
	protected static boolean testInstall(String INSTALL_PATH) {
		File config_dir = new File(INSTALL_PATH + File.separator + ".searchtool");
		// maybe new install
		if (!config_dir.exists()) {
			// request confirmation
			JOptionPane pane =
				new JOptionPane(
					"May I store my data files here ?\n\n"
						+ INSTALL_PATH
						+ File.separator
						+ Searchtool.Constants.configDirectoryName
						+ "\n\n( this will use up to 8MB of diskspace )",
					JOptionPane.QUESTION_MESSAGE,
					JOptionPane.CANCEL_OPTION);
			JDialog dialog = pane.createDialog(null, "INSTALL");
			dialog.show();
			Object value = pane.getValue();
			if (value == null)
				System.exit(0);
			if (value instanceof Integer) {
				if (((Integer) value).intValue() == pane.CANCEL_OPTION)
					System.exit(0);
				else
					; // fine, user accepted ---> keep going.
			}
			config_dir.mkdirs();
		}
		checkThatEveryFileIsInPlace();
		return true;
	}

	/**
	 *  write an initial and temporary root-tag to make the xml-parser happy.
	 */
	private static void writeToFile(File writeToMe, String content) {
		try {
			BufferedWriter waldorf = new BufferedWriter(new FileWriter(writeToMe));
			waldorf.write(content);
			waldorf.flush();
			waldorf.close();
		} catch (Exception e) {
			complainAbout("could not write to file: " + writeToMe.getAbsolutePath() + "\n" + e.getMessage());
		}
	}

	/**
	 *  return the version of the installation.
	 */
	private static String getInstalledVersionNumber() {
		String version = null;
		try {
			File versionFile = Searchtool.Constants.versionFile;
			BufferedReader buffy = new BufferedReader(new FileReader(versionFile));
			version = buffy.readLine();
			buffy.close();
		} catch (Exception e) {
			complainAbout("could not read version information\n" + e.getMessage());
		}
		return version;
	}
}
