/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package searchtool;

import java.util.*;
import javax.swing.*;
import searchtool.interfaces.*;
import searchtool.protocol.*;
import searchtool.wrappers.*;
import searchtool.halflife.protocol.*;

/**
 * @author thorsten
 *
 */
public class SearchtoolPlayerCollector implements StoppableThread {

	private SearchtoolProtocol players,details;
	private ServerElement[] queryUs;
	private SearchtoolResultTableModel model;
	private String[] searchForUs;
	private int queryNumber;
	private JProgressBar progressBar;

	private volatile boolean holded = false;
	private volatile int counter = -1; // init
	private volatile int maximum;

	public SearchtoolPlayerCollector(ServerElement[] queryUs, SearchtoolProtocol players, SearchtoolProtocol details, String[] searchForUs, SearchtoolResultTableModel model, int queryNumber) {
		this.players = players;
		this.details = details;
		this.queryUs = queryUs;
		this.model = model;
		this.searchForUs = searchForUs;
		this.queryNumber = queryNumber;
		maximum = queryUs.length - 1;
	}
	
	public void registerProgressBar(JProgressBar progressBar){
		this.progressBar = progressBar;
		this.progressBar.setMaximum(maximum);
	}
	
	private synchronized void addResult(Element player) {
		model.addElement(player);
	}

	public boolean isHolded() {
		return holded;
	}

	public void hold() {
		holded = true;
	}

	private synchronized int getNextPosition() {
		counter++;
		return counter;
	}

	public void run() {
		for (int i = 0; i < queryNumber; i++) {
			try {
				SearchtoolProtocol newPlayersInstance = (SearchtoolProtocol) players.getClass().newInstance();
				SearchtoolProtocol newDetailsInstance = (SearchtoolProtocol) details.getClass().newInstance();
				Thread thread = new SingleQueryThread(newPlayersInstance, newDetailsInstance);
				thread.start();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	private class SingleQueryThread extends Thread {

		private SearchtoolProtocol singlePlayers,singleDetails;

		public SingleQueryThread(SearchtoolProtocol players,SearchtoolProtocol details) {
			singlePlayers = players;
			singleDetails = details;
		}

		private boolean queryNext() {
			int queryMe = getNextPosition();
			if (queryMe >= maximum || holded)
				return false;// done
			else {
				
				// increase progress bar, if registered
				if( progressBar != null ){
					progressBar.setString( "Searched on " + (queryMe+1) + " of " + maximum + " servers.");	
					progressBar.setStringPainted(true);
					progressBar.setValue(queryMe+1);
				}
				
				// prepare
				singlePlayers.setServerElement(queryUs[queryMe]);
				singleDetails.setServerElement(queryUs[queryMe]);

				if ( ! singlePlayers.update() )
					singlePlayers.update();

				if (!singlePlayers.hasTimedout()) {
					String[] players = singlePlayers.getServerElement().getCurrentPlayerList();
					for (int i = 0; i < players.length; i++) {
						for (int j = 0; j < searchForUs.length; j++) {
							if (players[i].toLowerCase().indexOf(searchForUs[j].toLowerCase()) >= 0) {
								
								// update data - to ensure proper filter results
								if ( singleDetails.update() )
									singleDetails.getServerElement().setResponse(ServerElement.RESPONSE_POSITIVE);
								else if ( singleDetails.update() )
									singleDetails.getServerElement().setResponse(ServerElement.RESPONSE_POSITIVE);
								else
									singleDetails.getServerElement().setResponse(ServerElement.RESPONSE_NEGATIVE);

								// add result anyways
								addResult(new SearchtoolPlayer(players[i], singlePlayers.getServerElement()));
							}
						}
					}
				}
				return true;
			}
		}

		public void run() {
			while (queryNext())
				this.yield();
			hold();
		}
	}
}
