/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package searchtool;

import searchtool.*;
import searchtool.interfaces.*;
import searchtool.util.*;
import searchtool.wrappers.*;
import searchtool.actions.*;
import javax.swing.event.*;
import javax.swing.*;
import java.util.*;
import java.lang.*;
import java.io.*;
// thirdparty
import gnu.trove.*;

/**
 * @author thorsten
 *
 */
public class SearchtoolResultListManager extends ServerListManager {
	// the comparator used to sort lists
	private Comparator comp;

	public SearchtoolResultListManager(SearchtoolSettings SETTINGS, ButtonTracker BUTTONTRACKER) {
		super(SETTINGS, BUTTONTRACKER);
		comp = new SearchtoolResultListManager.ResultSettingsComparator(BUTTONTRACKER);
	}

	/*
	 * sort the serverlist with the the help of the Comparator 'comp'.
	 */
	public synchronized void sortElementList() {
		Element[] completeList = super.getCompleteList();
		Arrays.sort(completeList, comp);
		if (super.getButtonTracker().getValue(ButtonTracker.REVERSE_SORT).equals(ButtonTracker.ENABLED)) {
			Element[] reverseCopy = new Element[completeList.length];
			for (int i = 0; i < reverseCopy.length; i++)
				reverseCopy[i] = completeList[reverseCopy.length - i - 1];
			super.setCompleteList(reverseCopy);
			return;
		}
		super.setCompleteList(completeList);
	}

	/*
	 *  mark the given element that the filter does not filter it
	 */
	public void doNotFilterElement(Element doNotFilterMe) {
		((SearchtoolPlayer) doNotFilterMe).getServerElement().setResponse(ServerElement.SPECIAL);
	}

	/*
	 * filter out servers depending on the settings.
	 */
	public synchronized void filterElementList() {
		Element[] completeList = super.getCompleteList();
		if (completeList != null) {
			ArrayList tmp = new ArrayList();

			// init some stack variables for better performance
			String filteredMap = settings.getValue(SearchtoolSettings.FILTERED_MAP);
			String filteredMod = settings.getValue(SearchtoolSettings.FILTERED_MOD);

			boolean filterNotPingedServers = false;
			if (settings.getValueAsInt(SearchtoolSettings.FILTER_NOTPINGED_SERVERS) == SearchtoolSettings.ENABLED)
				filterNotPingedServers = true;

			boolean filterPublicServers = false;
			if (settings.getValueAsInt(SearchtoolSettings.FILTER_PUBLIC_SERVERS) == SearchtoolSettings.ENABLED)
				filterPublicServers = true;

			boolean filterPrivateServers = false;
			if (settings.getValueAsInt(SearchtoolSettings.FILTER_PRIVATE_SERVERS) == SearchtoolSettings.ENABLED)
				filterPrivateServers = true;

			boolean filterEmptyServers = false;
			if (settings.getValueAsInt(SearchtoolSettings.FILTER_EMPTY) == SearchtoolSettings.ENABLED)
				filterEmptyServers = true;

			boolean filterFullServers = false;
			if (settings.getValueAsInt(SearchtoolSettings.FILTER_FULL) == SearchtoolSettings.ENABLED)
				filterFullServers = true;

			int size = completeList.length;
			for (int i = 0; i < size; i++) {

				// SPECIAL (highest priority)
				if ((((SearchtoolPlayer)completeList[i]).getServerElement()).getResponse().byteValue() == ServerElement.SPECIAL.byteValue()) {
					tmp.add(((SearchtoolPlayer)completeList[i]).getServerElement());
					continue;
				}
				// AGE and RESPONSE_NEGATIVE
				if (!filterNotPingedServers
					&& /*(((ServerElement) completeList.get(i)).getResponse().byteValue() == ServerElement.OLD.byteValue()
																|| */
					 (((SearchtoolPlayer)completeList[i]).getServerElement()).getResponse().byteValue() == ServerElement.RESPONSE_NEGATIVE.byteValue()) {
					//System.out.println("response negative");
					continue;
				}
				// EMPTY SERVERS
				if (!filterEmptyServers && (((SearchtoolPlayer)completeList[i]).getServerElement()).getCurrentplayers().intValue() == 0) {
					//System.out.println("empty");
					continue;
				}
				// FULL SERVERS
				if (!filterFullServers && (((SearchtoolPlayer)completeList[i]).getServerElement()).getCurrentplayers().intValue() == (((SearchtoolPlayer)completeList[i]).getServerElement()).getMaxplayers().intValue()) {
					//System.out.println("full");
					continue;
				}
				// PUBLIC SERVERS
				if (!filterPublicServers && (((SearchtoolPlayer)completeList[i]).getServerElement()).getPassword() == ServerElement.NO_PASSWD) {
					//System.out.println("public");
					continue;
				}
				// PRIVATE SERVERS
				if (!filterPrivateServers && (((SearchtoolPlayer)completeList[i]).getServerElement()).getPassword() == ServerElement.PASSWD_REQUIRED) {
					//System.out.println("private");
					continue;
				}
				// MAP
				if (!(((SearchtoolPlayer)completeList[i]).getServerElement()).getMap().equalsIgnoreCase(filteredMap) && !filteredMap.equalsIgnoreCase("NONE")) {
					continue;
				}
				// MOD
				if (!(((SearchtoolPlayer)completeList[i]).getServerElement()).getMod().equalsIgnoreCase(filteredMod) && !filteredMod.equalsIgnoreCase("NONE")) {
					continue;
				}
				// this server is ok, lets keep it.
				else {
					tmp.add( completeList[i] );
				}
			}
			super.setFilteredList((Element[]) tmp.toArray(new Element[0]));
		}
	}

	private class ResultSettingsComparator implements Comparator {
		private SearchtoolPlayer o, o1;
		private ButtonTracker settings;
		private int x;

		public ResultSettingsComparator(ButtonTracker SETTINGS) {
			settings = SETTINGS;
		}

		/*
		 * comparator interface's method.
		 * sort depending on the current settings.
		 */
		public int compare(Object O, Object O1) {
			o = (SearchtoolPlayer) O;
			o1 = (SearchtoolPlayer) O1;
			switch (settings.getValueAsInt(ButtonTracker.SORTING_CRITERION)) {
				case SearchtoolTableModel.IP :
					{
						return o.compareTo(o1);
					}
				case SearchtoolTableModel.PING :
					{
						x = o.getServerElement().getPing().intValue() - o1.getServerElement().getPing().intValue();
						if (x == 0)
							return o.compareTo(o1);
						else
							return x;
					}
				case SearchtoolTableModel.MAP :
					{
						if (o.getServerElement().getMap().equals("-"))
							return 1;
						if (o1.getServerElement().getMap().equals("-"))
							return -1;
						x = o.getServerElement().getMap().compareTo(o1.getServerElement().getMap());
						if (x == 0)
							return o.compareTo(o1);
						else
							return x;
					}
				case SearchtoolTableModel.CURRENTPLAYERS :
					{
						x = o1.getServerElement().getCurrentplayers().intValue() - o.getServerElement().getCurrentplayers().intValue();
						if (x == 0)
							return o.compareTo(o1);
						else
							return x;
					}
				case SearchtoolTableModel.MAXPLAYERS :
					{
						x = o1.getServerElement().getMaxplayers().intValue() - o.getServerElement().getMaxplayers().intValue();
						if (x == 0)
							return o.compareTo(o1);
						else
							return x;
					}
			}
			return 0;
		}
	}
}
