/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package searchtool;

import searchtool.wrappers.*;
import searchtool.interfaces.*;
import searchtool.actions.*;
import searchtool.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.util.*;

/**
 * @author thorsten
 * 
 *  this class is a special table model (as compared to the SearchtoolTableModel) because
 *  it has to satisfy the need for a different table layout:
 *  the first column should be for the playernames and due to the possibility of more than one 
 *  searchresults per server  it has to be possible to store the same serverobject more than once.
 * 
 *  this is done by wrapping the ServerElement in an SearchtoolPlayer object.
 *
 */
public class SearchtoolResultTableModel extends SelectedElementTracker {

	public static final String PLAYER_COLUMN = "PLAYER";

	public SearchtoolResultTableModel(ElementListManager listmanager) {
		super(listmanager);
	}
	
	public void updateButtons(){
		SearchtoolPlayer selectedPlayer = (SearchtoolPlayer)super.getSelectedElement();
		ServerElement selectedElement = null;
		if(selectedPlayer != null)
			selectedElement = selectedPlayer.getServerElement();
		if (buttonbar != null) {
			if (selectedElement == null) {
				buttonbar.getButton(buttonbar.MAKE_FAVOURITE).setEnabled(false);
				buttonbar.getButton(buttonbar.MAKE_FAVOURITE).setSelected(false);
				buttonbar.getButton(buttonbar.MAKE_OUTSIDER).setEnabled(false);
				buttonbar.getButton(buttonbar.MAKE_OUTSIDER).setSelected(false);
				buttonbar.getButton(buttonbar.CONNECT).setEnabled(false);
				buttonbar.getButton(buttonbar.CONNECT).setSelected(false);
				buttonbar.getButton(buttonbar.DISPLAY_DETAILS).setEnabled(false);
				buttonbar.getButton(buttonbar.DISPLAY_DETAILS).setSelected(false);
			} else {
				// make sure enabled
				buttonbar.getButton(buttonbar.MAKE_FAVOURITE).setEnabled(true);
				buttonbar.getButton(buttonbar.MAKE_OUTSIDER).setEnabled(true);
				buttonbar.getButton(buttonbar.CONNECT).setEnabled(true);
				buttonbar.getButton(buttonbar.DISPLAY_DETAILS).setEnabled(true);
				// modify depending on list
				if (selectedElement.isFavourite())
					buttonbar.getButton(buttonbar.MAKE_FAVOURITE).setSelected(true);
				else
					buttonbar.getButton(buttonbar.MAKE_FAVOURITE).setSelected(false);
				if (selectedElement.isBlacklist())
					buttonbar.getButton(buttonbar.MAKE_OUTSIDER).setSelected(true);
				else
					buttonbar.getButton(buttonbar.MAKE_OUTSIDER).setSelected(false);
			}
		}
	}

	/*
	 * abstract methods implementations
	 */
	public int getColumnCount() {
		return 8;
	}

	SearchtoolPlayer tmpPlayer = null;
	public Object getValueAt(int row, int column) {
		tmpPlayer = (SearchtoolPlayer) super.getElement(row);
		if (tmpPlayer != null) {
			if (column == 0)
				return tmpPlayer.getPlayerName();
			else {
				switch (column) {
					case RESPONSE :
						return tmpPlayer.getServerElement().getResponse();
					case SERVERNAME :
						return tmpPlayer.getServerElement().getServername();
					case IP :
						return tmpPlayer.getServerElement().getIpAndPort();
					case MAP :
						return tmpPlayer.getServerElement().getMap();
					case PING :
						return tmpPlayer.getServerElement().getPing();
					case MAXPLAYERS :
						return tmpPlayer.getServerElement().getMaxplayers();
					case CURRENTPLAYERS :
						return tmpPlayer.getServerElement().getCurrentplayers();
					case PASSWORD :
						return tmpPlayer.getServerElement().getPassword();
				}
			}
		}
		return null;
	}

	public String getColumnName(int column) {
		if (column == 0)
			return PLAYER_COLUMN;
		else {
			switch (column) {
				case RESPONSE :
					return "RESPONSE";
				case SERVERNAME :
					return "SERVERNAME";
				case IP :
					return "IP:PORT";
				case MAP :
					return "MAP";
				case PING :
					return "PING";
				case MAXPLAYERS :
					return "MAXPLAYERS";
				case CURRENTPLAYERS :
					return "CURRENTPLAYERS";
				case PASSWORD :
					return "PASSWORD";
			}
			return null;
		}
	}
}
