/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
    
package searchtool;

import searchtool.util.IconLib;
import searchtool.halflife.*;
import searchtool.interfaces.*;
import searchtool.wrappers.*;
import javax.swing.*;
import javax.swing.table.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import java.net.*;

/**
 * the cell renderer for the jtable (SearchtoolTable) .  it takes care of the 'age' column, the 'ip:port'
 * and the 'password' column.  it can (should) be extended for further columns.
 * this class also provides a small set of images.
 */

public class SearchtoolTableCellRenderer implements TableCellRenderer {
    
    public static int RESPONSE_COLUMN=SearchtoolTableModel.RESPONSE,KEY_COLUMN=SearchtoolTableModel.PASSWORD,
    IP_COLUMN=SearchtoolTableModel.IP;
    private int column;
    
    private static ImageIcon REDBALL,GREENBALL,ORANGEBALL,BLUEBALL,CYANBALL, TEALBALL, REDKEY , GREENKEY;
    private static JLabel REDLABEL,GREENLABEL,ORANGELABEL,BLUELABEL,CYANLABEL,OLDLABEL,NEWLABEL,EMPTYLABEL,TEALLABEL,REDKEYLABEL,GREENKEYLABEL;
    // init
    static{
        REDBALL = IconLib.getRedballIcon();
        GREENBALL = IconLib.getGreenballIcon();
        ORANGEBALL = IconLib.getOrangeballIcon();
        BLUEBALL = IconLib.getBlueballIcon();
        CYANBALL = IconLib.getCyanballIcon();
        TEALBALL = IconLib.getTealballIcon();
        REDKEY = IconLib.getRedkeyIcon();
        GREENKEY = IconLib.getGreenkeyIcon();
        
        REDLABEL = new JLabel();
        GREENLABEL = new JLabel();
        ORANGELABEL = new JLabel();
        BLUELABEL = new JLabel();
        CYANLABEL = new JLabel();
        OLDLABEL = new JLabel();
        NEWLABEL = new JLabel();
        EMPTYLABEL = new JLabel();
        TEALLABEL = new JLabel();
        REDKEYLABEL = new JLabel();
        GREENKEYLABEL = new JLabel();
        
        BLUELABEL.setToolTipText("REQUIRED");
        CYANLABEL.setToolTipText("OPTIONAL");
        TEALLABEL.setToolTipText("SPECIAL");
        REDKEYLABEL.setToolTipText("PASSWORD REQUIRED");
        GREENKEYLABEL.setToolTipText("PUBLIC");
        REDLABEL.setToolTipText("NO SUCCESSFUL PING");
        GREENLABEL.setToolTipText("SUCCESSFUL PING");
        
        REDLABEL.setIcon(REDBALL);
        GREENLABEL.setIcon(GREENBALL);
        ORANGELABEL.setIcon(ORANGEBALL);
        BLUELABEL.setIcon(BLUEBALL);
        CYANLABEL.setIcon(CYANBALL);
        TEALLABEL.setIcon(TEALBALL);
        REDKEYLABEL.setIcon(REDKEY);
        GREENKEYLABEL.setIcon(GREENKEY);
        
        OLDLABEL.setForeground(Color.lightGray);
        OLDLABEL.setText("  old");
        
        NEWLABEL.setForeground(Color.green);
        NEWLABEL.setText("  new");
        
        EMPTYLABEL.setText("");
    }
    
    public SearchtoolTableCellRenderer(){
    }
    
        /*
         * interface method
         */
    
    public Component getTableCellRendererComponent(JTable table,Object value,boolean isSelected,boolean hasFocus,int row,int column){
        
        JLabel label = new JLabel();
        JButton panel;
        
        // <TMP>
        if ( value == null )
            return EMPTYLABEL;
        else if(column == RESPONSE_COLUMN){
            // </TMP>
            
            if(((Byte)value).byteValue() == ServerElement.RESPONSE_POSITIVE.byteValue()){
                if(isSelected){
                    JPanel returnMe = new JPanel(new GridLayout(1,1));
                    returnMe.setBackground(Color.darkGray);
                    returnMe.add(GREENLABEL);
                    return returnMe;
                }
                return GREENLABEL;
            }
            /*
            else if(((Byte)value).byteValue() == ServerElement.OLD.byteValue()){
                if(isSelected){
                    JPanel returnMe = new JPanel(new GridLayout(1,1));
                    returnMe.setBackground(Color.darkGray);
                    returnMe.add(EMPTYLABEL);
                    return returnMe;
                }
                return EMPTYLABEL;
            }
            */
            else if(((Byte)value).byteValue() == ServerElement.RESPONSE_NEGATIVE.byteValue()){
                if(isSelected){
                    JPanel returnMe = new JPanel(new GridLayout(1,1));
                    returnMe.setBackground(Color.darkGray);
                    returnMe.add(REDLABEL);
                    return returnMe;
                }
                return REDLABEL;
            }
            else{
                if(isSelected){
                    JPanel returnMe = new JPanel(new GridLayout(1,1));
                    returnMe.setBackground(Color.darkGray);
                    returnMe.add(TEALLABEL);
                    return returnMe;
                }
                // special
                return TEALLABEL;
            }
        }
        /*
        else if(column == IP_COLUMN){
            // here check if this server does exist in the favourites list
         
            ;// FIXME!
         
        }
         */
        else if(column == KEY_COLUMN){
            if(((Byte)value).equals(ServerElement.PASSWD_REQUIRED)){
                if(isSelected){
                    JPanel returnMe = new JPanel(new GridLayout(1,1));
                    returnMe.setBackground(Color.darkGray);
                    returnMe.add(REDKEYLABEL);
                    return returnMe;
                }
                return REDKEYLABEL;
            }
            else{
                if(isSelected){
                    JPanel returnMe = new JPanel(new GridLayout(1,1));
                    returnMe.setBackground(Color.darkGray);
                    returnMe.add(EMPTYLABEL);
                    return returnMe;
                }
                return EMPTYLABEL;
            }
        }
        else
            return label;
    }
    
        /*
         *  get
         */
    
    public JLabel getRedLabel(){
        return REDLABEL;
    }
    public JLabel getGreenLabel(){
        return GREENLABEL;
    }
    public JLabel getBlueLabel(){
        return BLUELABEL;
    }
    public JLabel getCyanLabel(){
        return CYANLABEL;
    }
    public JLabel getOrangeLabel(){
        return ORANGELABEL;
    }
    public JLabel getEmptyLabel(){
        return EMPTYLABEL;
    }
}

