/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package searchtool;

import searchtool.util.*;
import searchtool.components.*;
import searchtool.actions.*;
import searchtool.interfaces.*;
import searchtool.wrappers.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.*;
import java.util.*;

/**
 * @author thorsten
 *
 */
public class SearchtoolTableHeaderSortingController implements MouseListener, Comparator {

	// supported columns
	protected final int SERVERNAME = SearchtoolTableModel.SERVERNAME;
	protected final int IP = SearchtoolTableModel.IP;
	protected final int PING = SearchtoolTableModel.PING;
	protected final int MAP = SearchtoolTableModel.MAP;
	protected final int CURRENTPLAYERS = SearchtoolTableModel.CURRENTPLAYERS;
	protected final int MAXPLAYERS = SearchtoolTableModel.MAXPLAYERS;
	protected final int LASTCONNECTED = SearchtoolLastconnectedTableModel.LASTCONNECTED;

	private SearchtoolTable table = null;
	private SelectedElementTracker model = null;
	
	protected ButtonTracker buttonTracker = null;
	protected ServerElement o, o1;
	protected int x;

	public SearchtoolTableHeaderSortingController(SelectedElementTracker model, ButtonTracker buttonTracker) {
		this.model = model;
		this.buttonTracker = buttonTracker;
	}

	public void registerTable(SearchtoolTable table){
		this.table = table;
	}

	public SearchtoolTableHeaderSortingController(ButtonTracker buttonTracker) {
		this.buttonTracker = buttonTracker;
	}

	// MouseListener methods
	public void mouseClicked(MouseEvent e) {
		int selected_column;
		if (table != null) {
			selected_column = table.getColumnModel().getColumnIndexAtX(e.getX());
			
			// at this point check if the selected column is a valid (supported)
			// sorting criterion.
			checkSupportedColumn(selected_column);
			
			table.getTableHeader().repaint();
		}

	}

	protected void checkSupportedColumn(int selected_column) {

		if (selected_column == SERVERNAME
			|| selected_column == IP
			|| selected_column == PING
			|| selected_column == MAP
			|| selected_column == CURRENTPLAYERS
			|| selected_column == MAXPLAYERS
			|| selected_column == LASTCONNECTED) {
			doColumn(selected_column);
		}
	}

	protected void doColumn(int selected_column) {
		
		if (table != null)
			table.setSelectedColumn(selected_column);
		if (model.getListManager() != null && buttonTracker != null) {

			// if the selected column did not change , reverse sorting 
			if (buttonTracker.getValueAsInt(ButtonTracker.SORTING_CRITERION) == selected_column) {
				if (buttonTracker.getValue(ButtonTracker.REVERSE_SORT).equals(ButtonTracker.ENABLED)){
					buttonTracker.setValue(ButtonTracker.REVERSE_SORT, ButtonTracker.DISABLED);
				}
				else{
					buttonTracker.setValue(ButtonTracker.REVERSE_SORT, ButtonTracker.ENABLED);
				}
			}

			buttonTracker.setValue(ButtonTracker.SORTING_CRITERION, selected_column);

			// sort once
			model.getListManager().sortElementList();
			// update table
			// the following line causes trouble with the result table model !!!
			//((SearchtoolTableModel)table.getModel()).fireTableDataChanged();
		}
	}

	public void mousePressed(MouseEvent e) {
	}
	public void mouseReleased(MouseEvent e) {
	}
	public void mouseExited(MouseEvent e) {
	}
	public void mouseEntered(MouseEvent e) {
	}

	/**
	 * comparator interface's method.
	 * sort depending on the current settings.
	 */
	public int compare(Object O, Object O1) {
		o = (ServerElement) O;
		o1 = (ServerElement) O1;
		switch (buttonTracker.getValueAsInt(ButtonTracker.SORTING_CRITERION)) {
			case SearchtoolTableModel.SERVERNAME :
				{
					if (o.getMap().equals("-"))
						return 1;
					if (o1.getMap().equals("-"))
						return -1;
					return o.getServername().compareTo(o1.getServername());
				}
			case SearchtoolTableModel.IP :
				{
					return o.compareTo(o1);
				}
			case SearchtoolTableModel.PING :
				{
					x = o.getPing().intValue() - o1.getPing().intValue();
					if (x == 0)
						return o.compareTo(o1);
					else
						return x;
				}
			case SearchtoolTableModel.MAP :
				{
					if (o.getMap().equals("-"))
						return 1;
					if (o1.getMap().equals("-"))
						return -1;
					x = o.getMap().compareTo(o1.getMap());
					if (x == 0)
						return o.compareTo(o1);
					else
						return x;
				}
			case SearchtoolTableModel.CURRENTPLAYERS :
				{
					x = o1.getCurrentplayers().intValue() - o.getCurrentplayers().intValue();
					if (x == 0)
						return o.compareTo(o1);
					else
						return x;
				}
			case SearchtoolTableModel.MAXPLAYERS :
				{
					x = o1.getMaxplayers().intValue() - o.getMaxplayers().intValue();
					if (x == 0)
						return o.compareTo(o1);
					else
						return x;
				}
			case LASTCONNECTED :
				{
					long x = o1.getDate().getTime() - o.getDate().getTime();
					if (x == 0)
						return o.compareTo(o1);
					else
						return (int) x;
				}
		}
		return 0;
	}
}
