/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

package searchtool;

import searchtool.wrappers.*;
import searchtool.interfaces.*;
import searchtool.components.*;
import searchtool.actions.*;
import searchtool.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.util.*;

/**
 *
 * this class is a layer between the ServerListManager class (which basically holds, sorts and filters the data)
 * and the SearchtoolTable (which is the table the user gets to see).
 * depending on the current settings this layer decides what list the user gets to see.
 *
 * @author unascribed
 * @version 1.0
 */

public class SearchtoolTableModel extends SelectedElementTracker {

	private ServerListManager msl;
	private int columncount;

	public SearchtoolTableModel(ServerListManager MSL) {
		super(MSL);
		columncount = 8;
		bufferNewLists();
	}

	public void updateButtons() {
		if (buttonbar != null) {
			if (selectedElement == null) {
				buttonbar.getButton(buttonbar.MAKE_FAVOURITE).setEnabled(false);
				buttonbar.getButton(buttonbar.MAKE_FAVOURITE).setSelected(false);
				buttonbar.getButton(buttonbar.MAKE_OUTSIDER).setEnabled(false);
				buttonbar.getButton(buttonbar.MAKE_OUTSIDER).setSelected(false);
				buttonbar.getButton(buttonbar.CONNECT).setEnabled(false);
				buttonbar.getButton(buttonbar.CONNECT).setSelected(false);
				buttonbar.getButton(buttonbar.DISPLAY_DETAILS).setEnabled(false);
				buttonbar.getButton(buttonbar.DISPLAY_DETAILS).setSelected(false);
			} else {
				// make sure enabled
				buttonbar.getButton(buttonbar.MAKE_FAVOURITE).setEnabled(true);
				buttonbar.getButton(buttonbar.MAKE_OUTSIDER).setEnabled(true);
				buttonbar.getButton(buttonbar.CONNECT).setEnabled(true);
				buttonbar.getButton(buttonbar.DISPLAY_DETAILS).setEnabled(true);
				// modify depending on list
				if (((ServerElement) selectedElement).isFavourite())
					buttonbar.getButton(buttonbar.MAKE_FAVOURITE).setSelected(true);
				else
					buttonbar.getButton(buttonbar.MAKE_FAVOURITE).setSelected(false);
				if (((ServerElement) selectedElement).isBlacklist())
					buttonbar.getButton(buttonbar.MAKE_OUTSIDER).setSelected(true);
				else
					buttonbar.getButton(buttonbar.MAKE_OUTSIDER).setSelected(false);
			}
		}
	}

	/*
	 * abstract methods implementations
	 */
	public int getColumnCount() {
		return columncount;
	}

	public Object getValueAt(int row, int column) {
		// here check which list to show
		if (super.buttonTracker != null && super.buttonTracker.filterEnabled()) {
			if (row >= filteredList.length)
				return null;
			switch (column) {
				case RESPONSE :
					return ((ServerElement) filteredList[row]).getResponse();
				case SERVERNAME :
					return ((ServerElement) filteredList[row]).getServername();
				case IP :
					return ((ServerElement) filteredList[row]).getIpAndPort();
				case MAP :
					return ((ServerElement) filteredList[row]).getMap();
				case PING :
					return ((ServerElement) filteredList[row]).getPing();
				case MAXPLAYERS :
					return ((ServerElement) filteredList[row]).getMaxplayers();
				case CURRENTPLAYERS :
					return ((ServerElement) filteredList[row]).getCurrentplayers();
				case PASSWORD :
					return ((ServerElement) filteredList[row]).getPassword();
			}
		} else {
			if (row >= completeList.length)
				return null;
			switch (column) {
				case RESPONSE :
					return ((ServerElement) completeList[row]).getResponse();
				case SERVERNAME :
					return ((ServerElement) completeList[row]).getServername();
				case IP :
					return ((ServerElement) completeList[row]).getIpAndPort();
				case MAP :
					return ((ServerElement) completeList[row]).getMap();
				case PING :
					return ((ServerElement) completeList[row]).getPing();
				case MAXPLAYERS :
					return ((ServerElement) completeList[row]).getMaxplayers();
				case CURRENTPLAYERS :
					return ((ServerElement) completeList[row]).getCurrentplayers();
				case PASSWORD :
					return ((ServerElement) completeList[row]).getPassword();
			}
		}
		return null; // should not be reached
	}

	public String getColumnName(int column) {
		switch (column) {
			case RESPONSE :
				return "RESPONSE";
			case SERVERNAME :
				return "SERVERNAME";
			case IP :
				return "IP:PORT";
			case MAP :
				return "MAP";
			case PING :
				return "PING";
			case MAXPLAYERS :
				return "MAXPLAYERS";
			case CURRENTPLAYERS :
				return "CURRENTPLAYERS";
			case PASSWORD :
				return "PASSWORD";
		}
		return null;
	}
}
