/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *
 * Created on January 6, 2002, 1:47 PM
 */

package searchtool;

import searchtool.*;
import searchtool.interfaces.*;
import searchtool.util.*;
import searchtool.wrappers.*;
import searchtool.actions.*;
import javax.swing.event.*;
import javax.swing.*;
import java.util.*;
import java.lang.*;
import java.io.*;
// thirdparty
import gnu.trove.*;

/**
 *
 * @author  zoidberg
 * @version
 */
public class ServerListManager implements ElementListManager {

	// the basic list.
	private TLongObjectHashMap serverHash = new TLongObjectHashMap();
	// the basic list
	protected ArrayList completeList = new ArrayList();
	// the filtered list 
	protected ArrayList filteredServerList = new ArrayList();
	// the central settings object passed to the constructor
	protected SearchtoolSettings settings;
	// the comparator used to sort lists
	private Comparator comp;
	// a button tracker.  this object stores settings like sorting on/off and filtering on/off for this list
	private ButtonTracker buttonTracker;

	public ServerListManager(SearchtoolSettings SETTINGS, ButtonTracker BUTTONTRACKER) {
		settings = SETTINGS;
		buttonTracker = BUTTONTRACKER;
		comp = new SearchtoolTableHeaderSortingController(buttonTracker);
	}

	public synchronized void destroy() {
		serverHash.clear();
		completeList.clear();
		filteredServerList.clear();
	}

	/**
	 *  register different comparator object
	 */
	public synchronized void setComparator(SearchtoolTableHeaderSortingController controller) {
		comp = controller;
	}

	/**
	 *  get the button tracker object
	 */
	public synchronized ButtonTracker getButtonTracker() {
		return buttonTracker;
	}

	/**
	 * get the basic settings object.
	 */
	public synchronized SearchtoolSettings getSearchtoolSettings() {
		return settings;
	}
	/**
	 * return the serverlist.  it is sorted and complete ( because not filtered).
	 *
	 */
	public synchronized Element[] getCompleteList() {
		if (completeList != null)
			return (Element[]) completeList.toArray(new Element[0]);
		else
			return new Element[0];
	}

	/**
	 *  return the complete list in a hashmap.
	 */
	public synchronized TLongObjectHashMap getCompleteListAsMap() {
		TLongObjectHashMap returnMe = new TLongObjectHashMap(completeList.size());
		for (int i = 0; i < completeList.size(); i++) {
			returnMe.put(((Element) completeList.get(i)).getId(), completeList.get(i));
		}
		return returnMe;
	}

	/**
	 * return the filtered serverlist
	 */
	public synchronized Element[] getFilteredList() {
		if (filteredServerList != null)
			return (Element[]) filteredServerList.toArray(new Element[0]);
		else
			return new Element[0];
	}

	/**
	 * internal method to set filtered serverlist
	 */
	protected synchronized void setFilteredList(Element[] VALUE) {
		filteredServerList.clear();
		for (int i = 0; i < VALUE.length; i++) {
			filteredServerList.add(VALUE[i]);
		}
	}

	/**
	 * define a complete new serverlist.
	 * this list is a base for any other list.
	 * this method blocks until it is safe to access the serverlist.
	 */
	public synchronized void setCompleteList(Element[] SERVERLIST) {
		serverHash.clear();
		completeList.clear();
		for (int i = 0; i < SERVERLIST.length; i++) {
			serverHash.put(SERVERLIST[i].getId(), SERVERLIST[i]);
			completeList.add(SERVERLIST[i]);
		}
	}

	/**
	 * add element to main serverlist. after the next filter it will
	 * also appear (if not filtered of course) in the filtered list.
	 * NOTE:synchronization here is important.
	 * this method blocks until it is safe to access the serverlist.
	 */
	public synchronized void addElement(Element NEWELEMENT) {
		if (serverHash.contains(NEWELEMENT.getId()))
			return;
		serverHash.put(NEWELEMENT.getId(), NEWELEMENT);
		completeList.add(NEWELEMENT);
	}

	/**
	 * check if the given element does exist in the basic list already.
	 */
	public synchronized boolean isElementInList(Element AMIHERE) {
		return serverHash.containsKey(AMIHERE.getId());
	}

	/**
	 * remove element from serverlist.
	 * 
	 */
	public synchronized Element removeElement(Element REMOVEME) {
		if (serverHash.contains(REMOVEME.getId())) {
			completeList.remove(REMOVEME);
			filteredServerList.remove(REMOVEME);
			return (ServerElement) serverHash.remove(REMOVEME.getId());
		} else
			return null;
	}

	/**
	 * sort the serverlist with the the help of the Comparator 'comp'.
	 */
	public synchronized void sortElementList() {
		if (completeList != null) {
			Collections.sort(completeList, comp);
			if (buttonTracker.getValue(buttonTracker.REVERSE_SORT).equals(buttonTracker.ENABLED))
				Collections.reverse(completeList);
		}
	}

	/**
	 *  mark the given element that the filter does not filter it
	 */
	public synchronized void doNotFilterElement(Element doNotFilterMe) {
		((ServerElement) doNotFilterMe).setResponse(ServerElement.SPECIAL);
	}

	/**
	 * filter out servers depending on the settings.
	 */
	public synchronized void filterElementList() {
		if (completeList != null) {
			ArrayList tmp = new ArrayList();
			
			// init some stack variables for better performance
			String filteredMap = settings.getValue(SearchtoolSettings.FILTERED_MAP);
			String filteredMod = settings.getValue(SearchtoolSettings.FILTERED_MOD);

			boolean filterNotPingedServers = false;
			if (settings.getValueAsInt(SearchtoolSettings.FILTER_NOTPINGED_SERVERS) == SearchtoolSettings.ENABLED)
				filterNotPingedServers = true;

			boolean filterPublicServers = false;
			if (settings.getValueAsInt(SearchtoolSettings.FILTER_PUBLIC_SERVERS) == SearchtoolSettings.ENABLED)
				filterPublicServers = true;

			boolean filterPrivateServers = false;
			if (settings.getValueAsInt(SearchtoolSettings.FILTER_PRIVATE_SERVERS) == SearchtoolSettings.ENABLED)
				filterPrivateServers = true;

			boolean filterEmptyServers = false;
			if (settings.getValueAsInt(SearchtoolSettings.FILTER_EMPTY) == SearchtoolSettings.ENABLED)
				filterEmptyServers = true;

			boolean filterFullServers = false;
			if (settings.getValueAsInt(SearchtoolSettings.FILTER_FULL) == SearchtoolSettings.ENABLED)
				filterFullServers = true;

			int size = completeList.size();
			for (int i = 0; i < size; i++) {

				// SPECIAL (highest priority)
				if (((ServerElement) completeList.get(i)).getResponse().byteValue() == ServerElement.SPECIAL.byteValue()) {
					tmp.add(completeList.get(i));
					continue;
				}
				// AGE and RESPONSE_NEGATIVE
				if (!filterNotPingedServers
					&& /*(((ServerElement) completeList.get(i)).getResponse().byteValue() == ServerElement.OLD.byteValue()
												|| */
					 ((ServerElement) completeList.get(i)).getResponse().byteValue() == ServerElement.RESPONSE_NEGATIVE.byteValue()) {
					//System.out.println("response negative");
					continue;
				}
				// EMPTY SERVERS
				if (!filterEmptyServers && ((ServerElement) completeList.get(i)).getCurrentplayers().intValue() == 0) {
					//System.out.println("empty");
					continue;
				}
				// FULL SERVERS
				if (!filterFullServers && ((ServerElement) completeList.get(i)).getCurrentplayers().intValue() == ((ServerElement) completeList.get(i)).getMaxplayers().intValue()) {
					//System.out.println("full");
					continue;
				}
				// PUBLIC SERVERS
				if (!filterPublicServers && ((ServerElement) completeList.get(i)).getPassword() == ServerElement.NO_PASSWD) {
					//System.out.println("public");
					continue;
				}
				// PRIVATE SERVERS
				if (!filterPrivateServers && ((ServerElement) completeList.get(i)).getPassword() == ServerElement.PASSWD_REQUIRED) {
					//System.out.println("private");
					continue;
				}
				// MAP
				if (!((ServerElement) completeList.get(i)).getMap().equalsIgnoreCase(filteredMap) && !filteredMap.equalsIgnoreCase("NONE")) {
					continue;
				}
				// MOD
				if (!((ServerElement) completeList.get(i)).getMod().equalsIgnoreCase(filteredMod) && !filteredMod.equalsIgnoreCase("NONE")) {
					continue;
				}
				// this server is ok, lets keep it.
				else {
					tmp.add(completeList.get(i));
				}
			}
			filteredServerList = tmp;
		}
	}
}
