/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*
 *
 * Created on February 10, 2002, 9:25 PM
 */

package searchtool.actions;

import searchtool.components.GUI;
import searchtool.components.SearchtoolButtonBar;
import searchtool.actions.*;
import searchtool.protocol.*;
import searchtool.util.*;
import javax.swing.event.*;
import javax.swing.*;
import java.awt.event.*;
import java.net.*;
import searchtool.interfaces.*;
import java.util.*;
import searchtool.*;
import searchtool.wrappers.*;
import searchtool.interfaces.*;

/**
 * this class is in charge everytime the 'download serverlist' button
 * is pushed.  it starts and stops the download process.
 *
 * @author  zoidberg
 * @version
 */
public class DownloadServersActionListener implements ActionListener, FinalCall {

	// the controller that gets the downloaded servers once completed
	private ElementController elementController = null;
	// the game controller
	private GameController gameController = null;
	// the button controller to make sure to select/deselect all buttons at once
	private SynchronButtonController buttonController = null;
	// the task object that can be stopped via the taskmanager and by itself
	private TaskObject taskObject = null;
	// will soon point on the DownloadServerlist implementation
	private DownloadServerlist downloadServerlist = null;
	// the server request implementation
	private ServerRequest serverRequest = null;
	// the settings object
	private SearchtoolSettings settings = null;
	
	
	private String masterIp = null;
	private int masterPort;
	private int masterFilter;
	private JToggleButton toggleButton = null;

	/** Creates new DownloadServersAction */
	public DownloadServersActionListener(ElementController elementController, GameController gameController, SynchronButtonController buttonController, ServerRequest serverRequest) {
		this.elementController = elementController;
		this.buttonController = buttonController;
		this.gameController = gameController;
		this.serverRequest = serverRequest;
		settings = gameController.getSearchtoolSettings();
		downloadServerlist = new DownloadServerlist(elementController, gameController , this );
	} 

	public void actionPerformed(java.awt.event.ActionEvent actionEvent) {

		TaskManager taskmanager = Searchtool.getTaskManager();
		toggleButton = (JToggleButton) actionEvent.getSource();

		// STOP TASK IF RUNNING
		if ( downloadServerlist != null  && taskmanager.taskIsRunning(downloadServerlist.getDownloadTaskName())) {
			//stop task
			downloadServerlist.stopDownload();
			//taskmanager.stopAndRemoveTask(Searchtool.Constants.getHlDownloadServerlistTaskName());           
			buttonController.selectButtonsOfType(false,buttonController.DOWNLOAD_SERVERLIST);
		}

		// ELSE START TASK
		else {
			masterIp = settings.getValue(SearchtoolSettings.MASTERSERVER_IP);
			masterPort = settings.getValueAsInt(SearchtoolSettings.MASTERSERVER_PORT);		
			serverRequest.cleanUp();
			serverRequest.setIp(masterIp);
			serverRequest.setPort(masterPort);
			downloadServerlist.startDownload(serverRequest);
			buttonController.selectButtonsOfType(true,buttonController.DOWNLOAD_SERVERLIST);
		}
	}

	/**
	 *  the FinalCall method
	 */
	public void call() {
		buttonController.selectButtonsOfType(false,buttonController.DOWNLOAD_SERVERLIST);
	}
}