/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*
 *
 * Created on February 10, 2002, 9:20 PM
 */

package searchtool.actions;

import searchtool.components.*;
import searchtool.util.IconLib;
import javax.swing.event.*;
import javax.swing.*;
import java.awt.event.*;
import java.net.*;
import java.lang.*;
import java.io.*;
import java.util.*;
import searchtool.*;
import searchtool.wrappers.*;
import searchtool.protocol.*;
import searchtool.halflife.*;
import searchtool.interfaces.*;
import searchtool.actions.*;

/**
 * this class pings its way through a given list of servers.
 *
 * @author  zoidberg
 * @version
 */
public class PingServersActionListener implements ActionListener, FinalCall {

	// the toggle button to be set deselected after pinging
	private JToggleButton toggleButton = null;

	private SearchtoolSettings settings = null;
	private SearchtoolProtocol ping = null;
	private SearchtoolProtocol details = null;
	private SearchtoolProtocol rules = null;
	private GameController gameController = null;

	public PingServersActionListener(GameController gameController, SearchtoolSettings settings, SearchtoolProtocol ping, SearchtoolProtocol details, SearchtoolProtocol rules) {
		this.ping = ping;
		this.details = details;
		this.rules = rules;
		this.settings = settings;
		this.gameController = gameController;
	}

	public void call() {
		if (toggleButton != null)
			toggleButton.setSelected(false);
	}

	public void actionPerformed(ActionEvent e) {
		// store toggle button
		toggleButton = (JToggleButton) e.getSource();
		// check what tab is open and do the proper things.
		GUI gui = Searchtool.getGui();
		GameFrontend currentFrontend = gui.getCurrentGameFrontend();
		SearchtoolTab selectedTab = currentFrontend.getSelectedTab();
		ElementListManager listManager = ((SelectedElementTracker) selectedTab.getSearchtoolTable().getModel()).getListManager();
		int queryNumber = Searchtool.getGlobalSettings().getValueAsInt(SearchtoolSettings.SIMULTANEOUS_QUERYS);
		SearchtoolInfoCollector collector = new SearchtoolInfoCollector(listManager.getCompleteList(), ping, details,rules, queryNumber);
		TaskManager taskManager = Searchtool.getTaskManager();
		if (taskManager.taskIsRunning(gameController.PING_TASK_NAME)) {
			taskManager.stopAndRemoveTask(gameController.PING_TASK_NAME);
		} else {
			TaskObject taskObject = new TaskObject(gameController.PING_TASK_NAME, collector, this);
			Searchtool.getTaskManager().addTask(taskObject);
			taskObject.startTask();
		}
	}
}