/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*
 * HlSearchServerAction.java
 *
 * Created on February 24, 2002, 4:31 PM
 */

package searchtool.actions;

import searchtool.actions.*;
import searchtool.components.SearchtoolTable;
import searchtool.interfaces.*;
import searchtool.components.*;
import searchtool.util.*;
import searchtool.*;
import searchtool.wrappers.*;
import java.util.*;
import java.net.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * this class takes care of searching for a given server by ip and port
 * in the proper list.
 *
 * @author  zoidberg
 * @version
 */

public class SearchServerActionListener  implements ActionListener {

	private GUI gui;
	private SearchtoolDialogs.UserEnterServerDialog userEnterServerDialog;
	private ElementController controller;

	public SearchServerActionListener(ElementController controller) {
		gui = Searchtool.getGui();
		this.controller = controller;
		userEnterServerDialog = new SearchtoolDialogs.UserEnterServerDialog("Find server in list");
	}

	public void actionPerformed(java.awt.event.ActionEvent actionEvent) {
		GameFrontend currentFrontend = gui.getCurrentGameFrontend();
		SearchtoolTab selectedTab = currentFrontend.getSelectedTab();
		SearchtoolTable selectedTable = selectedTab.getSearchtoolTable();
		ElementListManager listManager = ((SelectedElementTracker) selectedTable.getModel()).getListManager();
		IpWrapper ipWrapper = userEnterServerDialog.askUser();
		// if user cancelled
		if (ipWrapper == null) {
			((JToggleButton) actionEvent.getSource()).setSelected(false);
			return;
		}
		// correct server entered --> search
		ServerElement element = controller.getServerElementFor(ipWrapper);
		if (element == null) {
			complainAboutNotExistingServer();
			((JToggleButton) actionEvent.getSource()).setSelected(false);
			return;
		}
		if (listManager.isElementInList(element)){
			// fine	
			selectedTable.selectElement(element);
		}
		else{
			// is element in favourite list ?
			if (element.isFavourite()){
				giveMessage("FAVOURITES");
			}
			// is element in blacklist ? 
			else if (element.isBlacklist()){
				giveMessage("BLACKLIST");
			}
			// in main list?
			else{
				giveMessage("MAINLIST");
			}
		}

		((JToggleButton) actionEvent.getSource()).setSelected(false);
		return;
	}

	private void complainAboutNotExistingServer() {
		JOptionPane optionPane = new JOptionPane(" ... the address you entered is not in any list.", JOptionPane.ERROR_MESSAGE);
		JDialog dialog = optionPane.createDialog(Searchtool.getGui(), "Excuse me, but ...");
		dialog.show();
	}
	private void giveMessage(String message){
		JOptionPane optionPane = new JOptionPane(" ... the address you entered is not in the selected list\nbut it has been found in: " + message, JOptionPane.ERROR_MESSAGE);
		JDialog dialog = optionPane.createDialog(Searchtool.getGui(), "Excuse me, but ...");
		dialog.show();
	}
}
