/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package searchtool.components;

import searchtool.components.*;
import searchtool.util.IconLib;
import searchtool.*;
import searchtool.wrappers.*;
import searchtool.halflife.wrappers.*;
import searchtool.halflife.protocol.*;
import searchtool.protocol.*;
import searchtool.interfaces.*;
import java.io.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.net.*;
// thirdparty
import gnu.trove.*;

/**
 * @author zoidberg
 *
 */
public class DetailsDialog implements TableCellRenderer {

	private SearchtoolProtocol ping = null;
	private SearchtoolProtocol rules = null;
	private SearchtoolProtocol details = null;
	private SearchtoolProtocol players = null;
	private JDialog dialog = null;
	private JOptionPane pane = null;
	private GUI gui = null;
	private Element element = null;
	private JList player_list = null;
	private JTable details_table = null;
	private JTable rules_table = null;
	private Object[][] data = null;
	private Vector columnames = null;
	private Vector rules_vector = null;
	private JScrollPane player_pane = null;
	private JScrollPane rules_pane = null;
	private JPanel data_panel = null;
	private JPanel rules_panel = null;
	private PingWatcher pingWatcher = null;
	private RulesTableSettings rulesTableSettings = null;
	private SearchtoolSettings settings = null;

	public DetailsDialog(
		SearchtoolProtocol ping,
		SearchtoolProtocol rules,
		SearchtoolProtocol details,
		SearchtoolProtocol players,
		Element element,
		File rulesTableSettingsFile,
		SearchtoolSettings settings) {
		this.ping = ping;
		this.details = details;
		this.rules = rules;
		this.players = players;
		this.element = element;
		this.settings = settings;
		rulesTableSettings = new RulesTableSettings(rulesTableSettingsFile);
		gui = Searchtool.getGui();
	}

	public void show() {
		if (element != null) {
			if (element instanceof ServerElement) {
				ping.setServerElement((ServerElement) element);
				details.setServerElement((ServerElement) element);
				rules.setServerElement((ServerElement) element);
				players.setServerElement((ServerElement) element);
				displayDetails((ServerElement) element);
			}
			if (element instanceof SearchtoolPlayer) {
				ping.setServerElement((ServerElement) (((SearchtoolPlayer) element).getServerElement()));
				details.setServerElement((ServerElement) (((SearchtoolPlayer) element).getServerElement()));
				players.setServerElement((ServerElement) (((SearchtoolPlayer) element).getServerElement()));
				rules.setServerElement((ServerElement) (((SearchtoolPlayer) element).getServerElement()));
				displayDetails((ServerElement) (((SearchtoolPlayer) element).getServerElement()));
			}
		}
	}

	/*
	* initialize the details window and start the (inner)worker class
	*/
	private void displayDetails(ServerElement serverElement) {

		player_list = new JList();
		player_list.setToolTipText("Players on server");
		player_list.setBackground(Searchtool.Constants.backgroundColor);
		data = new Object[10][2];
		data[0][0] = "PING";
		data[1][0] = "IP";
		data[2][0] = "PORT";
		data[3][0] = "DOMAIN/IP";
		data[4][0] = "MAP";
		data[5][0] = "MAX PLAYERS";
		data[6][0] = "CURRENT PL.";
		data[7][0] = "MOD";
		data[8][0] = "SERVERTYPE";
		data[9][0] = "OS";

		data[1][1] = serverElement.getIp();
		data[2][1] = serverElement.getPort() + "";
		// DETAILS TABLE
		details_table = new JTable(new InfoTableModel(data, new Object[] { "", "VALUE" }) {
			public Object getValueAt(int row, int column) {
				if (data[row][column] != null)
					return "" + data[row][column];
				else
					return "";
			}
		});
		details_table.setToolTipText("Details");
		details_table.setBackground(Searchtool.Constants.backgroundColor);
		details_table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		details_table.setSelectionBackground(Searchtool.Constants.backgroundColor);
		details_table.setPreferredSize(new Dimension(300, 300));
		details_table.setShowGrid(false);

		// RULES TABLE
		columnames = new Vector();
		rules_vector = new Vector();
		columnames.add("RULE");
		columnames.add("VALUE");
		rules_table = new JTable(new InfoTableModel(rules_vector, columnames));
		rules_table.setToolTipText("Click on a rule to highlight it");
		rules_table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		rules_table.setBackground(Searchtool.Constants.backgroundColor);
		rules_table.setSelectionBackground(Searchtool.Constants.backgroundColor);
		rules_table.getColumn("RULE").setCellRenderer(this);
		rules_table.addMouseListener(new RulesMouseListener());
		rules_table.setShowGrid(false);

		player_pane = new JScrollPane(player_list);
		player_pane.setBorder(BorderFactory.createEtchedBorder());
		rules_pane = new JScrollPane(rules_table);
		rules_pane.setBorder(BorderFactory.createEtchedBorder());
		data_panel = new JPanel();

		pingWatcher = new PingWatcher();
		details_table.setPreferredSize(new Dimension(300, 100));
		details_table.setBorder(BorderFactory.createEtchedBorder());
		JPanel sharedPanel = new JPanel();
		GridBagLayout layout = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		sharedPanel.setLayout(layout);
		c.gridx = c.gridy = 0;
		c.fill = c.BOTH;
		c.weighty = 0.4;
		c.weightx = 0.4;
		layout.setConstraints(details_table, c);
		sharedPanel.add(details_table);
		c.weighty = 1.0;
		c.gridy = 1;
		layout.setConstraints(pingWatcher, c);
		sharedPanel.add(pingWatcher);

		GridBagLayout layout1 = new GridBagLayout();
		GridBagConstraints c1 = new GridBagConstraints();
		c1.gridx = c1.gridy = 0;
		c1.gridheight = c1.REMAINDER;
		c1.gridwidth = 1;
		c1.fill = c1.BOTH;
		c1.weightx = 0.4;
		c1.weighty = 1;
		data_panel.setLayout(layout1);
		layout1.setConstraints(sharedPanel, c1);
		data_panel.add(sharedPanel);

		c1.gridx = 1;
		c1.weightx = 0.4;
		layout1.setConstraints(rules_pane, c1);
		data_panel.add(rules_pane);
		c1.gridx = 2;
		c1.weightx = 0.2;
		layout1.setConstraints(player_pane, c1);
		data_panel.add(player_pane);
		data_panel.setBackground(Color.darkGray);
		data_panel.setPreferredSize(new Dimension(800, 400));

		pane = new JOptionPane(data_panel, JOptionPane.INFORMATION_MESSAGE, JOptionPane.NO_OPTION, null, new Object[] { "connect", "close" });
		dialog = pane.createDialog(Searchtool.getGui(), "Info");

		// THREAD THAT KEEPS DATA UP TO DATE

		UpdateDetailsWorker runningman = new UpdateDetailsWorker(serverElement);
		Thread thread = new Thread(runningman);
		thread.start();

		dialog.pack();
		dialog.show();
		// maybe connect
		if (pane.getValue() != null) {
			if (pane.getValue().equals("connect"))
				new SearchtoolConnect(details, settings).connect(serverElement);
		}
		// if dialog is closed stop thread
		runningman.hold();
	}

	public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
		JLabel data = new JLabel("");
		if (rulesTableSettings.containsRule((String) value)) {
			data.setForeground(Color.red);
		}
		data.setText((String) value);
		return data;
	}

	/*
	 * this class keeps track of selected rules.
	 */
	private class RulesMouseListener implements MouseListener {
		public void mouseClicked(java.awt.event.MouseEvent mouseEvent) {
			int row = rules_table.getSelectedRow();
			if (row != -1) {
				String rule = (String) rules_table.getValueAt(row, 0);
				if (rulesTableSettings.containsRule(rule))
					rulesTableSettings.removeRule(rule);
				else
					rulesTableSettings.addRule(rule);
				rulesTableSettings.save();
				rules_table.repaint();
			}
		}
		public void mouseExited(java.awt.event.MouseEvent mouseEvent) {
		}
		public void mouseReleased(java.awt.event.MouseEvent mouseEvent) {
		}
		public void mousePressed(java.awt.event.MouseEvent mouseEvent) {
		}
		public void mouseEntered(java.awt.event.MouseEvent mouseEvent) {
		}
	}

	/*
	* worker class that keeps updating until holded
	*/
	private class UpdateDetailsWorker implements StoppableThread {

		private ServerElement serverElement;
		private boolean holded = false;

		protected UpdateDetailsWorker(ServerElement element) {
			serverElement = element;
		}

		public void hold() {
			holded = true;
		}
		public boolean isHolded() {
			return holded;
		};
		public void run() {
			// get domain if possible
			// but wrap in thread to prevent from blocking
			new Thread(new Runnable() {
				public void run() {
					try {
						data[3][1] = InetAddress.getByName(serverElement.getIp()).getHostName();
					} catch (Exception e) {
						return;
					}
				}
			}).start();

			StoppableThread detailsThread = new StoppableThread() {
				private boolean holded = false;
				public void hold() {
					holded = true;
				}
				public boolean isHolded() {
					return holded;
				}
				public void run() {
					try {
						while (!this.isHolded()) {
							refreshDetails();
							 ((DefaultTableModel) details_table.getModel()).fireTableCellUpdated(0, 1);
							Thread.sleep(500);
						}
					} catch (InterruptedException e) {
						return;
					}
				}
			};
			StoppableThread rulesThread = new StoppableThread() {
				private boolean holded = false;
				public void hold() {
					holded = true;
				}
				public boolean isHolded() {
					return holded;
				}
				public void run() {
					try {
						while (!this.isHolded()) {
							refreshRules();
							((DefaultTableModel) rules_table.getModel()).fireTableDataChanged();
							Thread.sleep(500);
						}
					} catch (InterruptedException e) {
						return;
					}
				}
			};
			StoppableThread playersThread = new StoppableThread() {
				private boolean holded = false;
				public void hold() {
					holded = true;
				}
				public boolean isHolded() {
					return holded;
				}
				public void run() {
					try {
						while (!this.isHolded()) {
							refreshPlayers();
							player_pane.validate();
							Thread.sleep(500);
						}
					} catch (InterruptedException e) {
						return;
					}
				}
			};
			StoppableThread pingThread = new StoppableThread() {
				private boolean holded = false;
				public void hold() {
					holded = true;
				}
				public boolean isHolded() {
					return holded;
				}
				public void run() {
					try {
						while (!this.isHolded()) {
							refreshPing();
							pingWatcher.repaint();
							//((DefaultTableModel) details_table.getModel()).fireTableCellUpdated(0, 1);
							 ((DefaultTableModel) details_table.getModel()).fireTableDataChanged();
							Thread.sleep(500);
						}
					} catch (InterruptedException e) {
						return;
					}
				}
			};

			new Thread(detailsThread).start();
			new Thread(rulesThread).start();
			new Thread(playersThread).start();
			new Thread(pingThread).start();

			while (!this.isHolded()) {
				try {
					Thread.sleep(500);
				} catch (Exception e) {
					;
				}
			}

			detailsThread.hold();
			rulesThread.hold();
			playersThread.hold();
			pingThread.hold();
		}

		private void refreshRules() {
			rules.update();
			rules_vector.clear();
			THashMap hushpuppy = serverElement.getRulesMap();
			Set keys = hushpuppy.keySet();
			Iterator i = keys.iterator();
			// vector of vectors
			while (i.hasNext()) {
				Vector x = new Vector();
				Object key = i.next();
				x.add(key);
				x.add(hushpuppy.get(key));
				rules_vector.add(x);
			}
			// move highlighted rules and values to top of table.
			for (int j = 0; j < rules_vector.size(); j++) {
				if (rulesTableSettings.containsRule((String) ((Vector) rules_vector.get(j)).get(0))) {
					rules_vector.add(0, rules_vector.remove(j));
				}
			}
		}

		private void refreshPing() {
			if (ping.update()){
				data[0][1] = serverElement.getPing().intValue() + "";
				pingWatcher.addPing(serverElement.getPing().intValue());
				ping.getServerElement().setResponse(ServerElement.RESPONSE_POSITIVE);
			} else {
				data[0][1] = "timeout";
				pingWatcher.addPing(PingWatcher.TIMEOUT_PING);
				ping.getServerElement().setResponse(ServerElement.RESPONSE_NEGATIVE);
			}
		}

		private void refreshPlayers() {

			players.update();
			Vector tmp = new Vector();
			Vector playervector = new Vector();
			String[] currentplayers = serverElement.getCurrentPlayerList();
			for (int i = 0; i < currentplayers.length; i++)
				playervector.add(currentplayers[i]);
			tmp = new Vector();
			player_list.removeAll();
			int rowcount = currentplayers.length;
			if (rowcount == 0)
				tmp.add("[ no players on server ]");
			for (int i = 0; i < rowcount; i++) {
				tmp.add(currentplayers[i]);
			}
			player_list.setListData(tmp);
			player_list.revalidate();
			data[6][1] = currentplayers.length + "";
			details_table.revalidate();
		}
		private void refreshDetails() {

			details.update();
			dialog.setTitle(serverElement.getServername());
			data[4][1] = serverElement.getMap();
			data[5][1] = serverElement.getMaxplayers() + "";
			data[7][1] = serverElement.getMod();
			data[8][1] = serverElement.getServerType();
			data[9][1] = serverElement.getOperatingSystem();
			details_table.repaint();
			details_table.revalidate();
		}
	}

	/*
	 * overriding one method disables cell editing in
	 * the tables this model is applyed to.
	 */
	private class InfoTableModel extends DefaultTableModel {

		public InfoTableModel(Object[][] a, Object[] b) {
			super(a, b);
		}

		public InfoTableModel(Vector a, Vector b) {
			super(a, b);
		}

		public boolean isCellEditable(int a, int b) {
			return false;
		}
	}

	/**
	 * this inner class handles the painting of the tiny
	 * ping - progress - graphic
	 */
	private class PingWatcher extends JLabel {

		protected static final int TIMEOUT_PING = -1;

		private int height, width;
		private ArrayList lineArray;
		private int goodBorder = 80;
		private int mediumBorder = 110;
		private int badBorder = 140;
		private int veryGoodBorder = 50;
		private int perfectBorder = 20;
		private int cutOut = 0;
		private boolean badPing = false;

		protected PingWatcher() {
			lineArray = new ArrayList();
			setBorder(BorderFactory.createEtchedBorder());
		}

		public void addPing(int newPing) {
			if (lineArray.size() >= width / 5)
				lineArray.clear();
			lineArray.add(new Integer(newPing));
		}

		public void paint(Graphics g) {
			super.paint(g);
			height = getHeight() - 8;
			width = getWidth() - 8;
			int x, y, ping;
			g.setColor(Color.lightGray);
			g.fillRect(4, 4, width, height);

			for (int i = 1; i < lineArray.size(); i++) {
				x = i * 5;
				ping = ((Integer) lineArray.get(i - 1)).intValue();
				y = height - ping;

				if (ping == TIMEOUT_PING) {
					; // skip this
				} else {
					g.setColor(Color.darkGray);
					if (ping < height)
						g.fillRect(x, y + cutOut, 5, ping - cutOut + 4);
					else
						g.fillRect(x, 4, 5, height);
				}
			}
			g.setColor(Color.red);
			g.drawLine(4, height - badBorder + cutOut, width, height - badBorder + cutOut);
			g.drawString("" + badBorder, width - 30, height - badBorder + cutOut);
			g.setColor(Color.blue);
			g.drawLine(4, height - mediumBorder + cutOut, width, height - mediumBorder + cutOut);
			g.drawString("" + mediumBorder, width - 30, height - mediumBorder + cutOut);
			g.setColor(Color.green.darker().darker());
			g.drawLine(4, height - goodBorder + cutOut, width, height - goodBorder + cutOut);
			g.drawString("" + goodBorder, width - 23, height - goodBorder + cutOut);
			g.setColor(Color.green.darker());
			g.drawLine(4, height - veryGoodBorder + cutOut, width, height - veryGoodBorder + cutOut);
			g.drawString("" + veryGoodBorder, width - 23, height - veryGoodBorder + cutOut);
			g.drawLine(4, height - perfectBorder + cutOut, width, height - perfectBorder + cutOut);
			g.drawString("" + perfectBorder, width - 23, height - perfectBorder + cutOut);
		}
	}
}
