/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */
package searchtool.components;

import searchtool.*;
import searchtool.interfaces.*;
import searchtool.wrappers.*;
import searchtool.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.*;
import java.net.*;
import java.util.*;

/**
 * @author thorsten
 * 
 * this class represents the panel on a buttonbar that can be used
 *  for a quick change of the filter values.
 */
public class FilterSettingsPanel extends JPanel implements ItemListener, ListCellRenderer, SettingsListener {

	private ModComboBox modBox = new ModComboBox();
	private MapComboBox mapBox = new MapComboBox();
	private SearchtoolSettings settings = null;
	private String[] installedMods = null;
	private String[] installedMaps = null;

	public FilterSettingsPanel(SearchtoolSettings settings) {

		this.settings = settings;
		installedMods = settings.getInstalledMods();
		installedMaps = settings.getInstalledMaps();

		setLayout(new GridLayout(1, 2));

		modBox.addItemListener(this);
		mapBox.addItemListener(this);

		modBox.setBorder(BorderFactory.createRaisedBevelBorder());
		mapBox.setBorder(BorderFactory.createRaisedBevelBorder());

		setPreferredSize(new Dimension(300, 30));
		setMaximumSize(new Dimension(300, 30));
		setMinimumSize(new Dimension(300, 30));

		modBox.setToolTipText("Modfilter");
		mapBox.setToolTipText("Mapfilter");

		modBox.setRenderer(this);
		mapBox.setRenderer(this);

		add(modBox);
		add(mapBox);

		modBox.updateBox();
		mapBox.updateBox();

		settings.addSettingsListener(this);

	}

	/**
		*  itemlistener method
		*/
	public void itemStateChanged(ItemEvent e) {

		if (e.getSource().equals(modBox)) {
			if (((JComboBox) e.getSource()).getSelectedItem() != null) {
				settings.setValue(SearchtoolSettings.FILTERED_MOD, (String) ((JComboBox) e.getSource()).getSelectedItem());
				settings.setValue(SearchtoolSettings.FILTERED_MAP, "NONE");

				installedMods = settings.getInstalledMods();
				installedMaps = settings.getInstalledMaps();

				modBox.updateBox();
				mapBox.updateBox();
			}
		}
		// if selected map changed
		else if (e.getSource().equals(mapBox)) {
			if (((JComboBox) e.getSource()).getSelectedItem() != null) {
				settings.setValue(SearchtoolSettings.FILTERED_MAP, (String) ((JComboBox) e.getSource()).getSelectedItem());
				mapBox.updateBox();
			}
		}
	}

	/**
	 *  cellrenderer method
	 */
	public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
		JLabel returnme = new JLabel((String) value);
		if (isSelected) {
			returnme.setForeground(Color.black);
		}
		if (index == -1)
			returnme.setIcon(IconLib.getFilterIcon());
		return returnme;
	}

	/**
	 *  settingslistener method
	 */
	public void settingsChanged(String key) {
		if (key.equals(SearchtoolSettings.FILTERED_MOD) ||  key.equals(SearchtoolSettings.FILTERED_MAP) /*|| key.equals(SearchtoolSettings.GAME_DIR)*/) {
			installedMods = settings.getInstalledMods();
			installedMaps = settings.getInstalledMaps();
			modBox.updateBox();
			mapBox.updateBox();
		}
	}

	/**
	 *  inner classes (comboboxes)
	 */
	private class ModComboBox extends JComboBox {
		public ModComboBox() {
			super();
			//updateBox();
		}
		protected void updateBox() {
			this.removeItemListener(FilterSettingsPanel.this);
			String mod = (String) settings.getValue(SearchtoolSettings.FILTERED_MOD);
			removeAllItems();
			addItem("NONE");
			if (installedMods != null) {
				for (int i = 0; i < installedMods.length; i++) {
					addItem(installedMods[i]);
				}
			}
			setSelectedItem(mod);
			this.addItemListener(FilterSettingsPanel.this);
		}
	}

	private class MapComboBox extends JComboBox {
		public MapComboBox() {
			super();
			//updateBox();
		}
		private void updateBox() {
			this.removeItemListener(FilterSettingsPanel.this);
			String map = (String) settings.getValue(SearchtoolSettings.FILTERED_MAP);
			removeAllItems();
			addItem("NONE");
			if (installedMaps != null) {
				for (int i = 0; i < installedMaps.length; i++) {
					addItem(installedMaps[i]);
				}
			}
			boolean containsMap = false;
			if (installedMaps != null) {
				for (int i = 0; i < installedMaps.length; i++) {
					if (installedMaps[i].equals(map)) {
						containsMap = true;
						break;
					}
				}
			}
			if (containsMap) {
				setSelectedItem(map);
			} else {
				setSelectedItem("NONE");
			}
			this.addItemListener(FilterSettingsPanel.this);
		}
	}
}
