/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * GUI.java
 *
 * Created on December 16, 2001, 9:24 PM
 */

package searchtool.components;

import searchtool.halflife.components.HlEditSettingsPanel;
import searchtool.TaskManager;
import searchtool.Searchtool;
import searchtool.actions.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import searchtool.interfaces.*;
import searchtool.wrappers.*;
import searchtool.halflife.*;
import searchtool.halflife.wrappers.*;
import java.util.*;
// thirdparty
import ise.java.awt.*;
import com.incors.plaf.kunststoff.*;

/**
 *
 * the graphical user interface that handles all the tabs
 *
 * @author  zoidberg
 * @version
 */
public class GUI extends JFrame implements ActionListener {

	private JLabel messageLabel = new JLabel();
	private TaskManager taskmanager = new TaskManager();
	private JMenuBar menuBar;
	private JMenu menu;
	private JMenuItem menuItem;
	private ButtonGroup menuItemGroup;
	private GameFrontend currentFrontend = null;
	private Vector gameFrontends = new Vector();
	private JRadioButtonMenuItem HLitem;
	private JRadioButtonMenuItem UTitem;
	//private JRadioButtonMenuItem T2item;
	private JRadioButtonMenuItem UT2003item;
	

	/** Creates new GUI */
	public GUI() {
		setVisible(false);
		setTitle("   searchtool " + Searchtool.Constants.version);
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				Searchtool.safeExit();
			}
		});

		//Create the menu bar.
		menuBar = new JMenuBar();
		//menuBar.setBorder( BorderFactory.createCompoundBorder(BorderFactory.createEmptyBorder(0,0,2,0) , BorderFactory.createRaisedBevelBorder()));
		//menuBar.setComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT); 
		setJMenuBar(menuBar);
		menuItemGroup = new ButtonGroup();
		menu = new JMenu("Games");
		menu.setBorder( BorderFactory.createCompoundBorder( BorderFactory.createRaisedBevelBorder() , BorderFactory.createEmptyBorder(2,5,2,5) ) );
		menu.setToolTipText("Choose your game");

		// HL
		HLitem = new JRadioButtonMenuItem(getNameForPrefix(Searchtool.Constants.HALFLIFE_PREFIX));
		HLitem.addActionListener(this);
		menuItemGroup.add(HLitem);
		menu.add(HLitem);
		// UT
		UTitem = new JRadioButtonMenuItem( getNameForPrefix(Searchtool.Constants.UNREAL_TOURNAMENT_PREFIX));
		UTitem.addActionListener(this);
		menuItemGroup.add(UTitem);
		menu.add(UTitem);
		/*
		// T2
		T2item = new JRadioButtonMenuItem( getNameForPrefix(Searchtool.Constants.TRIBES2_PREFIX));
		T2item.addActionListener(this);
		menuItemGroup.add(T2item);
		menu.add(T2item);
		*/
		// UT2003
		UT2003item = new JRadioButtonMenuItem( getNameForPrefix(Searchtool.Constants.UT2003_PREFIX ));
		UT2003item.addActionListener(this);
		menuItemGroup.add(UT2003item);
		menu.add(UT2003item);
		menuBar.add(menu);
		menuItemGroup = new ButtonGroup();

		getContentPane().setBackground(Color.darkGray);
		// do the layout
		getContentPane().setLayout(new BorderLayout());
		setSize(1024, 700);
		getContentPane().add(messageLabel, BorderLayout.SOUTH);
		// try to set the proper look and feel
		setLookAndFeel();
	}

	public GameFrontend getCurrentGameFrontend() {
		return currentFrontend;
	}


	private String getNameForPrefix(String prefix){
		for ( int i = 0; i < Searchtool.Constants.frontendPrefixNameMap.length; i++ ){
			if ( Searchtool.Constants.frontendPrefixNameMap[i][0].equals(prefix) )
				return Searchtool.Constants.frontendPrefixNameMap[i][1];
		}
		return null;
	}
	
	private String getPrefixForName(String name){
		for ( int i = 0; i < Searchtool.Constants.frontendPrefixNameMap.length; i++ ){
			if ( Searchtool.Constants.frontendPrefixNameMap[i][1].equals(name) )
				return Searchtool.Constants.frontendPrefixNameMap[i][0];
		}
		return null;
	}
	
	

	public synchronized void displayFrontend(GameFrontend frontend) {
		if (currentFrontend != null) {
			getContentPane().remove(currentFrontend);
			currentFrontend = null;
		}
		else{
			if ( frontend.getName().equals(Searchtool.Constants.UNREAL_TOURNAMENT_PREFIX) ){
				UTitem.setSelected(true);	
			}	
			else if ( frontend.getName().equals(Searchtool.Constants.UT2003_PREFIX) ){
				UT2003item.setSelected(true);	
			}	
			else /*( frontend.getName().equals(Searchtool.Constants.HALFLIFE_PREFIX) )*/{
				HLitem.setSelected(true);	
			}	
		}
		getContentPane().add(frontend);
		getContentPane().repaint();
		currentFrontend = frontend;
		currentFrontend.revalidate();
		currentFrontend.setVisible(true);
		currentFrontend.repaint();
		setTitle("   searchtool " + Searchtool.Constants.version + "        [ " + getNameForPrefix( currentFrontend.getName() ) + " ]");

	}
	
	public synchronized GameFrontend getGameFrontend(String frontendName){
		for ( int i = 0; i < gameFrontends.size(); i++){
			if ( ( (GameFrontend)gameFrontends.get(i)).getName().equals(frontendName) ){
				return (GameFrontend)gameFrontends.get(i);
			}
		}	
		return null;
	}
	
	/**
	 *  used from within <code> GameController </code> class.
	 */
	public synchronized GameFrontend setupNewGameFrontend(String gameName) {
		GameFrontend newFrontend = new GameFrontend(gameName);
		gameFrontends.add(newFrontend);
		newFrontend.setVisible(false);
		return newFrontend;
	}
	
	public synchronized void removeGameFrontend(String gameName){
		for ( int i = 0; i < gameFrontends.size(); i++){
			if ( ( (GameFrontend)gameFrontends.get(i)).getName().equals(gameName) ){
				gameFrontends.remove(i);	
			}
		}	
	}

	public void giveMessage(String MESSAGE) {
		messageLabel.setText(MESSAGE);
		messageLabel.validate();
		messageLabel.repaint();
	}

	public void actionPerformed(ActionEvent e) {
		String gameName = getPrefixForName( ((JMenuItem) e.getSource()).getText() );
		if ( ! gameName.equals( this.getCurrentGameFrontend().getName() ) )
			Searchtool.openGameFrontend(gameName,false);
	}

	public static boolean setLookAndFeel() {

		SearchtoolSettings settings = Searchtool.getGlobalSettings();
		String wantedLookAndFeel = settings.getValue(SearchtoolSettings.LOOK_AND_FEEL);
		//String wantedLookAndFeel = "com.incors.plaf.kunststoff.KunststoffLookAndFeel";
		if (wantedLookAndFeel != null) {
			// maybe change default value to proper java LookAndFeel
			if (wantedLookAndFeel.equals("Kunststoff") || wantedLookAndFeel.equals("default")) { // default value of configfile
				wantedLookAndFeel = "com.incors.plaf.kunststoff.KunststoffLookAndFeel";
				settings.setValue(SearchtoolSettings.LOOK_AND_FEEL, wantedLookAndFeel);
			}
			try {
				UIManager.setLookAndFeel(wantedLookAndFeel);
				if (Searchtool.getGui() != null) // WARNING, this method is called in constructor
					SwingUtilities.updateComponentTreeUI(Searchtool.getGui());
			} catch (Exception e) {
				JOptionPane pane = new JOptionPane("sorry, this LookAndFeel is not supported\nby your java installation", JOptionPane.NO_OPTION);
				JDialog dialog = pane.createDialog(Searchtool.getGui(), "unsupported LookAndFeel");
				dialog.show();
				return false;
			}
			settings.save();
			return true;
		}
		return false;
	}
}
