/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
package searchtool.components;

import searchtool.halflife.components.HlEditSettingsPanel;
import searchtool.TaskManager;
import searchtool.Searchtool;
import searchtool.actions.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import searchtool.interfaces.*;
import searchtool.wrappers.*;
import searchtool.halflife.*;
import searchtool.halflife.wrappers.*;
import java.util.*;
// thirdparty
import ise.java.awt.*;
import com.incors.plaf.kunststoff.*;

/**
 * @author thorsten
 *
*/

public class GameFrontend extends JPanel implements MouseListener {

	private JTabbedPane tubby = new JTabbedPane();
	private String gameName = null;
	// keep track which tab is front, which one comes next, etc.
	private Vector tabVector = new Vector();

	public GameFrontend(String NAME) {
		super();
		gameName = NAME;
		// listen for tab selections
		tubby.addMouseListener(this);
		setLayout(new BorderLayout());
		add(tubby, BorderLayout.CENTER);
	}

	public String getName() {
		return gameName;
	}

	public String getSelectedTabName() {
		int index = tubby.getSelectedIndex();
		return tubby.getComponent(index).getName();
	}
	public void addTab(SearchtoolTab tab) {
		tubby.addTab(tab.getName(), tab);
		tubby.setSelectedIndex(tubby.indexOfTab(tab.getName()));
		// store name
		newTabAdded(tab.getName());
	}
	public void addTab(String name, Component tab) {
		tab.setName(name);
		tubby.addTab(name, tab);
		tubby.setSelectedIndex(tubby.indexOfTab(name));
		// store name
		newTabAdded(name);
	}
	public void selectTab(String name) {
		int index = tubby.indexOfTab(name);
		tubby.setSelectedIndex(index);
		// update tabVector
		tabSelected(name);
	}
	public void removeTab(String tabname) {
		tubby.removeTabAt(tubby.indexOfTab(tabname));
		// update tabVector
		oldTabRemoved(tabname);
		selectOnTopTab();
	}
	public boolean tabIsOpen(String tabname) {
		if (tubby.indexOfTab(tabname) != -1)
			return true;
		else
			return false;
	}
	public SearchtoolTab getSearchtoolTab(String name) {
		int index = tubby.indexOfTab(name);
		if (index >= 0) {
			return (SearchtoolTab) tubby.getComponent(index);
		} else
			return null;
	}

	public SearchtoolTab getSelectedTab() {
		return getSearchtoolTab(getSelectedTabName());
	}

	private void newTabAdded(String name) {
		tabVector.add(name);
	}

	private void oldTabRemoved(String name) {
		for (int i = 0; i < tabVector.size(); i++)
			if (tabVector.get(i).equals(name)) {
				tabVector.remove(i);
				break;
			}
	}

	private void tabSelected(String name) {
		for (int i = 0; i < tabVector.size(); i++)
			if (tabVector.get(i).equals(name)) {
				tabVector.remove(i);
				break;
			}
		tabVector.add(name);
	}

	private void selectOnTopTab() {
		int size = tabVector.size();
		selectTab((String) tabVector.get(size - 1));
	}

	// mouse listener methods
	public void mouseClicked(java.awt.event.MouseEvent mouseEvent) {
		if (tubby.getSelectedComponent() != null)
			if (tubby.getSelectedComponent().getName() != null)
				tabSelected(tubby.getSelectedComponent().getName());
	}
	public void mouseExited(java.awt.event.MouseEvent mouseEvent) {
	}
	public void mouseReleased(java.awt.event.MouseEvent mouseEvent) {
	}
	public void mousePressed(java.awt.event.MouseEvent mouseEvent) {
	}
	public void mouseEntered(java.awt.event.MouseEvent mouseEvent) {
	}
}
