/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
 * SearchPlayersPanel.java
 *
 * Created on January 17, 2002, 5:21 PM
 */
package searchtool.components;

import searchtool.util.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import searchtool.*;
import searchtool.wrappers.*;
import searchtool.interfaces.*;
import searchtool.protocol.*;
// thirdparty
import gnu.trove.*;

/**
 * the panel that contains all components that 
 * controll the search process.
 *
 * @author  zoidberg
 * @version
 */
public class SearchPlayersPanel extends JPanel implements ActionListener, FinalCall {
	private JTextField player_names_field = new JTextField();
	private JButton startsearch = new JButton("Start search");
	private JButton stopsearch = new JButton("Stop search");
	private JProgressBar progressbar = new JProgressBar();
	private GameController gameController = null;
	private String[] searchstrings = null;
	private SearchtoolProtocol players = null;
	private SearchtoolProtocol details = null;
	private GUI gui = null;

	/** Creates new SearchPlayersPanel */
	public SearchPlayersPanel(GameController gameController, SearchtoolProtocol players, SearchtoolProtocol details) {
		this.gameController = gameController;
		this.players = players;
		this.details = details;
		setLayout(new GridLayout(5, 1));

		player_names_field.setPreferredSize(new Dimension(800, 25));
		progressbar.setPreferredSize(new Dimension(800, 25));

		stopsearch.setEnabled(false);
		stopsearch.addActionListener(this);
		startsearch.addActionListener(this);
		// preset with saved string, if there is ...
		player_names_field.setText(gameController.getSearchtoolSettings().getValue(SearchtoolSettings.SEARCHSTRING));

		JLabel message = new JLabel("Enter one or more playernames, separated by spaces:");
		GridBagConstraints c = new GridBagConstraints();
		GridBagLayout layout = new GridBagLayout();
		setLayout(layout);
		c.anchor = c.WEST;
		c.gridx = c.gridy = 0;
		c.gridwidth = 4;
		layout.setConstraints(message, c);
		add(message);
		c.gridy = 1;
		layout.setConstraints(player_names_field, c);
		add(player_names_field);

		c.gridy = 2;
		c.gridwidth = 4;
		layout.setConstraints(progressbar, c);
		add(progressbar);

		c.anchor = c.CENTER;
		c.gridy = 3;
		c.gridwidth = 1;
		layout.setConstraints(startsearch, c);
		add(startsearch);
		c.gridx = 1;
		layout.setConstraints(stopsearch, c);
		add(stopsearch);
	}

	public JButton getStartSearchButton() {
		return startsearch;
	}
	public JButton getStopSearchButton() {
		return stopsearch;
	}
	public JProgressBar getProgressBar() {
		return progressbar;
	}

	public String[] getSearchStrings() {
		if (player_names_field.getText().equals(""))
			return null;
		else {
			StringTokenizer token = new StringTokenizer(player_names_field.getText());
			searchstrings = new String[token.countTokens()];
			for (int i = 0; i < searchstrings.length; i++) {
				searchstrings[i] = token.nextToken().trim();
			}
			return searchstrings;
		}
	}

	public void call() {
		stopsearch.doClick();
	}

	public void actionPerformed(java.awt.event.ActionEvent actionEvent) {

		final GUI gui = Searchtool.getGui();
		final TaskManager taskmanager = Searchtool.getTaskManager();
		final String SEARCH_TABNAME = gameController.SEARCH_TABNAME;
		final String SEARCHTASK_NAME = gameController.SEARCH_TASK_NAME;
		final SearchtoolResultTableModel resultTableModel = gameController.getResultTableModel();
		//
		// run search
		//
		if (actionEvent.getActionCommand().equals("Start search")) {

			// get searchstrings
			final String[] searchstrings = getSearchStrings();

			// if nothing to search for, exit now!
			if (searchstrings == null)
				return;
			if (searchstrings.length == 0)
				return;

			String searchString = "";
			for (int i = 0; i < getSearchStrings().length; i++)
				searchString += getSearchStrings()[i] + " ";
			gameController.getSearchtoolSettings().setValue(SearchtoolSettings.SEARCHSTRING, searchString);
			gameController.getSearchtoolSettings().save();

			// disable start button
			getStartSearchButton().setEnabled(false);
			// enable stop button
			getStopSearchButton().setEnabled(true);

			// get serverlist and make sure to search through the comlete list
			Object[] servers = gameController.getElementController().getCompleteServerList().getValues();
			ServerElement[] serverElements = new ServerElement[servers.length];
			for (int i = 0; i < servers.length; i++) {
				serverElements[i] = (ServerElement) servers[i];
			}

			// clear possible old results
			resultTableModel.getListManager().setCompleteList(new ServerElement[0]);

			// create worker object that does the work.
			int queryNumber = gameController.getSearchtoolSettings().getValueAsInt(SearchtoolSettings.SIMULTANEOUS_QUERYS);
			SearchtoolPlayerCollector collector = new SearchtoolPlayerCollector(serverElements, players,details, searchstrings, resultTableModel, queryNumber);
			collector.registerProgressBar(progressbar);
			// register task
			TaskObject to = new TaskObject(SEARCHTASK_NAME, collector, this);
			Searchtool.getTaskManager().addTask(to);
			// go!
			to.startTask();
		}
		//
		// stop search
		//
		if (actionEvent.getActionCommand().equals("Stop search")) {
			// disable stopsearchbutton
			getStopSearchButton().setEnabled(false);

			// stop searching
			Searchtool.getTaskManager().stopAndRemoveTask(SEARCHTASK_NAME);

			// reenable startsearchbutton
			getStartSearchButton().setEnabled(true);
		}

		//
		// close searchtab
		//

		if (actionEvent.getActionCommand().equals("close tab")) {

			String tmp = "";
			if (getSearchStrings() != null) {
				for (int i = 0; i < getSearchStrings().length; i++)
					tmp += (getSearchStrings()[i] + " ");
			}
			gameController.getSearchtoolSettings().setValue(SearchtoolSettings.SEARCHSTRING, tmp);
			gameController.getSearchtoolSettings().save();

			// close tab
			GameFrontend currentFrontend = gui.getCurrentGameFrontend();
			currentFrontend.removeTab(SEARCH_TABNAME);

			// close running search, if there is.
			Searchtool.getTaskManager().stopAndRemoveTask(SEARCHTASK_NAME);
		}
	}
}