/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */
package searchtool.components;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.*;
import searchtool.util.*;
import searchtool.*;
import searchtool.wrappers.*;
import java.awt.Toolkit;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Locale;
import java.net.*;

/**
 *  A set of dialogs used by searchtool.
 * 
 * @author thorsten
 *
 */
public class SearchtoolDialogs {
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	////////////// HELP DIALOG
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	public static class HelpDialog extends JPanel implements HyperlinkListener {

		private JEditorPane editorPane = new JEditorPane();
		private JScrollPane scrollpane = null;

		/** Creates new HelpPanel */
		public HelpDialog(String pathToHelpHtml) {
			try {
				URL url = ClassLoader.getSystemResource(pathToHelpHtml);
				editorPane.setPage(url);
			} catch (Exception e) {
				e.printStackTrace();
			}
			editorPane.setEditable(false);
			editorPane.addHyperlinkListener(this);
			add(editorPane);
		}

		public void showHelpDialog(boolean gplOption) {
			JDialog dialog;
			final boolean finalGplOption = gplOption;
			scrollpane = new JScrollPane(this);
			scrollpane.setPreferredSize(new Dimension(550, 600));
			JButton gplButton = new JButton("read GPL");
			gplButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					new HelpDialog("searchtool/files/gpl.html").showHelpDialog( ! finalGplOption );
				}
			});
			JOptionPane pane;
			if ( finalGplOption )
				pane = new JOptionPane(scrollpane, JOptionPane.INFORMATION_MESSAGE,JOptionPane.DEFAULT_OPTION,null,new Object[]{"close",gplButton});
			else
				pane = new JOptionPane(scrollpane,JOptionPane.INFORMATION_MESSAGE,JOptionPane.DEFAULT_OPTION,null,new Object[]{"close"});
			dialog = pane.createDialog(Searchtool.getGui(), "Help");
			dialog.show();
		}

		/**
		 *  follow (local) links
		 */
		public void hyperlinkUpdate(HyperlinkEvent e) {
			if (e.getEventType().equals(HyperlinkEvent.EventType.ACTIVATED)) {
				try {
					editorPane.setPage(e.getURL());
				} catch (Exception f) {
					f.printStackTrace();
				}
			}
		}
	}

	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	////////////// CONNECT DIALOG
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	public static class ConnectDialog extends JPanel {
		
		public final static int CONNECT_BUTTON=0,
		CANCEL_BUTTON=1;

		private JComponent component;
		private JFrame frame = null;
		private JPanel buttonPanel = new JPanel(new BorderLayout());
		private JTextPane textPane = new JTextPane();
		private JButton cancelButton = new JButton("cancel");
		private JButton connectButton = new JButton("connect anyway");

		public ConnectDialog() {
			this.component = component;
			setLayout(new BorderLayout());
			textPane.setEditable(false);
			textPane.setBackground(Searchtool.Constants.backgroundColor);
			add(textPane, BorderLayout.CENTER);
			buttonPanel.add(cancelButton, BorderLayout.WEST);
			buttonPanel.add(connectButton, BorderLayout.EAST);
			add(buttonPanel, BorderLayout.SOUTH);
		}

		public void setText(String text) {
			textPane.setText(text);
			textPane.repaint();
		}
		
		public void setActionListener(ActionListener listener, int target){
			if(target == CONNECT_BUTTON)
				connectButton.addActionListener(listener);
			if(target == CANCEL_BUTTON)
				cancelButton.addActionListener(listener);
		}

		/**
		*  Show a dialog that displays the given panel on the searchtool logo
		*  @param contentPanel is the panel to be displayed on the dialog
		*/
		public void showDialog() {
			frame = new JFrame("connecting");
			frame.setSize(new Dimension(300, 150));
			frame.setContentPane(this);
			int screenWidth = Toolkit.getDefaultToolkit().getScreenSize().width;
			int screenHeight = Toolkit.getDefaultToolkit().getScreenSize().height;
			frame.setLocation(screenWidth / 2 - frame.getSize().width / 2, screenHeight / 2 - frame.getSize().height / 2);
			frame.show();
		}

		public void closeDialog() {
			frame.dispose();
		}

	}

	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	////////////// ERROR DIALOG
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	public static class ErrorDialog {

		private JOptionPane pane = null;
		private JDialog dialog = null;

		public ErrorDialog(String errorMessage) {
			pane = new JOptionPane(errorMessage, JOptionPane.NO_OPTION);
			dialog = pane.createDialog(Searchtool.getGui(), "oops");
		}

		public void showErrorDialog() {
			dialog.show();
		}

	}

	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	////////////// ENTER SERVER DIALOG
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	public static class UserEnterServerDialog {
		private DomainInputPanel domainInputPanel = new DomainInputPanel();
		private JPanel contentPanel = new JPanel();
		private String title = "";

		public UserEnterServerDialog(String title) {
			// create the input panel
			this.title = title;
			contentPanel.setLayout(new GridLayout(2, 1));
			contentPanel.add(new JLabel(" Enter server by IP :  PORT or DOMAIN : PORT"));
			contentPanel.add(domainInputPanel);
		}

		/**
		 *  return a valid IpWrapper or null
		 *  if the entered ip was not correct.
		 */
		public IpWrapper askUser() {
			SwingUtilities.updateComponentTreeUI(contentPanel);
			JOptionPane optionPane = new JOptionPane(contentPanel, JOptionPane.QUESTION_MESSAGE);
			JDialog dialog = optionPane.createDialog(Searchtool.getGui(), title);
			dialog.show();
			if (optionPane.getValue() == null)
				return null; // window close
			if (optionPane.getValue() instanceof Integer) {
				IpWrapper returnMe = domainInputPanel.getAddress();
				if (returnMe == null) {
					complainAboutInput();
					return askUser();
				}
				return returnMe;
			}
			return null;
		}

		/**
		 *  complain about wrong input.
		 */

		public void complainAboutInput() {
			JOptionPane optionPane = new JOptionPane(" ... the address you entered is not valid", JOptionPane.ERROR_MESSAGE);
			JDialog dialog = optionPane.createDialog(Searchtool.getGui(), "Excuse me, but ...");
			dialog.show();
		}

		private class DomainInputPanel extends JPanel {

			private JTextField domainField = new JTextField();
			private JTextField port = new JTextField(5);

			private DomainInputPanel() {
				domainField.setPreferredSize(new Dimension(150, 20));
				GridBagLayout layout = new GridBagLayout();
				GridBagConstraints cons = new GridBagConstraints();
				setLayout(layout);
				///////////////////////
				cons.gridx = cons.gridy = 0;
				cons.fill = cons.HORIZONTAL;
				//////////////////////
				cons.gridwidth = 6;
				cons.weightx = 1.0;
				cons.gridx = cons.RELATIVE;
				layout.setConstraints(domainField, cons);
				add(domainField);
				///////////////////////
				cons.gridx = cons.RELATIVE;
				cons.gridwidth = 1;
				cons.weightx = 0.5;
				JLabel doubleDot = new JLabel(" : ");
				layout.setConstraints(doubleDot, cons);
				add(doubleDot);
				///////////////////////
				cons.gridx = cons.RELATIVE;
				cons.gridwidth = 2;
				cons.weightx = 1.0;
				layout.setConstraints(port, cons);
				add(port);
			}

			private IpWrapper getAddress() {
				IpWrapper ipWrapper;
				try {
					int portValue = Integer.parseInt(port.getText());
					InetAddress testAddress = InetAddress.getByName(domainField.getText().trim());
					byte[] byteAddress = testAddress.getAddress();
					ipWrapper = new IpWrapper(byteAddress[0] & 255, byteAddress[1] & 255, byteAddress[2] & 255, byteAddress[3] & 255, portValue);
				} catch (Exception e) {
					return null;
				}
				return ipWrapper;
			}
		}
	}

	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	////////////// PASSWORD DIALOG
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	public static class PasswordDialog {

		/**
		 *  Displays a dialog that asks the user to enter a password
		 *  @return returns null if cancelled or dialog closed.  returns a <code>String</code> that holds the entered password.
		 */
		public String showPasswordDialog() {
			// setup dialog
			JTextField passwordField = new JTextField();
			passwordField.setPreferredSize(new Dimension(300, 20));
			JLabel passwordMessage = new JLabel("  password required:");
			JPanel passwordPanel = new JPanel();
			passwordPanel.setLayout(new BorderLayout());
			passwordPanel.add(passwordMessage, BorderLayout.CENTER);
			passwordPanel.add(passwordField, BorderLayout.SOUTH);
			JOptionPane pane = new JOptionPane(passwordPanel, JOptionPane.QUESTION_MESSAGE, JOptionPane.CANCEL_OPTION);
			JDialog passwordDialog = pane.createDialog(Searchtool.getGui(), "PASSWORD");

			// show dialog
			passwordDialog.show();

			// return response
			if (pane.getValue() == null) {
				// dialog cancelled --> 
				return null;
			}
			if (pane.getValue() instanceof Integer) {
				// some result
				if (((Integer) pane.getValue()).intValue() == JOptionPane.CANCEL_OPTION) {
					// dialog cancelled -->
					return null;
				} else {
					// get password
					return passwordField.getText().trim();
				}
			}
			return null;
		}
	}
}
