/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

package searchtool.components;

import searchtool.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import searchtool.*;
import searchtool.wrappers.*;
import searchtool.interfaces.*;
import java.io.*;
import java.util.*;
import java.net.*;
// thirdparty
import ise.java.awt.*;

/**
 * @author zoidberg
 *
 */
public class SearchtoolSettingsPanel extends JPanel {
	
	public FilterPanel filterPanel = null;
	public StartupPanel startupPanel = null;
	public MiscPanel miscPanel = null;
	public GlobalPanel globalPanel = null;
	public HelpBrowser helpBrowser = null;

	public SearchtoolSettingsPanel(SearchtoolSettings settings, String HTML_HELPFILE_PATH) {
		LambdaLayout layout = new LambdaLayout();
		LambdaLayout.Constraints con = new LambdaLayout.Constraints();

		con.a = 8; // NORTH-WEST
		con.s = "wh";
		con.x = 0;
		setLayout(layout);

		// upper left
		con.w = 2;
		filterPanel = new FilterPanel(settings);
		add(filterPanel, con);

		// upper center
		con.x = 2;
		con.w = 6;
		startupPanel = new StartupPanel(settings);
		add(startupPanel, con);

		// upper right
		con.x = 8;
		con.w = 2;
		miscPanel = new MiscPanel(settings);
		add(miscPanel, con);

		// lower left
		con.x = 0;
		con.w = 3;
		con.y = 1;
		globalPanel = new GlobalPanel();
		add(globalPanel, con);

		// lower center
		con.x = 3;
		con.w = 7;
		helpBrowser = new HelpBrowser(HTML_HELPFILE_PATH);
		add(helpBrowser, con);
		
	}

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	////////	FILTER SETTINGS
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	public static class FilterPanel extends JPanel implements ItemListener {
		private SearchtoolSettings settings = null;
		// GUI STUFF
		private TitledBorder titleBorder = BorderFactory.createTitledBorder("Local filter settings");
		private Border emptyBorder = BorderFactory.createEmptyBorder(5, 5, 5, 5);
		private LambdaLayout layout = new LambdaLayout();
		private LambdaLayout.Constraints con = new LambdaLayout.Constraints();

		private JCheckBox emptyServers = new JCheckBox("empty servers");
		private JCheckBox fullServers = new JCheckBox("full servers");

		private JCheckBox publicServers = new JCheckBox("public servers");
		private JCheckBox privateServers = new JCheckBox("private servers");

		private JCheckBox notPingedServers = new JCheckBox("not checked servers");

		public FilterPanel(SearchtoolSettings settings) {
			this.settings = settings;
			this.setPreferredSize(new Dimension(200,150));
			
			
			// setup panel
			setLayout(layout);
			setBorder(new CompoundBorder(titleBorder, emptyBorder));
			con.a = 8; // NORTH-WEST
			con.s = "wh";
			add(new JLabel("Show ... "), con);
			con.x = 1;
			con.y = 1;
			add(layout.createVerticalStrut(1), con);
			con.y = 2;
			add(emptyServers, con);
			con.y = 3;
			add(fullServers, con);
			con.y = 4;
			add(layout.createVerticalStrut(5), con);
			con.y = 5;
			add(publicServers, con);
			con.y = 6;
			add(privateServers, con);
			con.y = 7;
			add(layout.createVerticalStrut(5), con);
			//con.y = 8;
			//add(layout.createVerticalStrut(10), con);
			con.y = 8;
			add(notPingedServers, con);

			// init settings
			if (settings.getValueAsInt(SearchtoolSettings.FILTER_EMPTY) == SearchtoolSettings.ENABLED)
				emptyServers.setSelected(true);
			if (settings.getValueAsInt(SearchtoolSettings.FILTER_FULL) == SearchtoolSettings.ENABLED)
				fullServers.setSelected(true);

			if (settings.getValueAsInt(SearchtoolSettings.FILTER_PUBLIC_SERVERS) == SearchtoolSettings.ENABLED)
				publicServers.setSelected(true);
			if (settings.getValueAsInt(SearchtoolSettings.FILTER_PRIVATE_SERVERS) == SearchtoolSettings.ENABLED)
				privateServers.setSelected(true);

			if (settings.getValueAsInt(SearchtoolSettings.FILTER_NOTPINGED_SERVERS) == SearchtoolSettings.ENABLED)
				notPingedServers.setSelected(true);

			// add action listeners	
			publicServers.addItemListener(this);
			privateServers.addItemListener(this);
			emptyServers.addItemListener(this);
			fullServers.addItemListener(this);
			notPingedServers.addItemListener(this);
		}

		public void itemStateChanged(ItemEvent e) {

			// update settings
			if (e.getSource().equals(emptyServers)) {
				if (e.getStateChange() == e.DESELECTED) {
					settings.setValue(SearchtoolSettings.FILTER_EMPTY, SearchtoolSettings.DISABLED);
				} else {
					settings.setValue(SearchtoolSettings.FILTER_EMPTY, SearchtoolSettings.ENABLED);
				}
			}
			if (e.getSource().equals(fullServers)) {
				if (e.getStateChange() == e.DESELECTED) {
					settings.setValue(SearchtoolSettings.FILTER_FULL, SearchtoolSettings.DISABLED);
				} else {
					settings.setValue(SearchtoolSettings.FILTER_FULL, SearchtoolSettings.ENABLED);
				}
			}
			if (e.getSource().equals(publicServers)) {
				if (e.getStateChange() == e.DESELECTED) {
					if (!privateServers.isSelected()) {
						publicServers.setSelected(true);
						return;
					}
					settings.setValue(SearchtoolSettings.FILTER_PUBLIC_SERVERS, SearchtoolSettings.DISABLED);
				} else {
					settings.setValue(SearchtoolSettings.FILTER_PUBLIC_SERVERS, SearchtoolSettings.ENABLED);
				}
			}
			if (e.getSource().equals(privateServers)) {
				if (e.getStateChange() == e.DESELECTED) {
					if (!publicServers.isSelected()) {
						privateServers.setSelected(true);
						return;
					}
					settings.setValue(SearchtoolSettings.FILTER_PRIVATE_SERVERS, SearchtoolSettings.DISABLED);
				} else {
					settings.setValue(SearchtoolSettings.FILTER_PRIVATE_SERVERS, SearchtoolSettings.ENABLED);
				}
			}
			if (e.getSource().equals(notPingedServers)) {
				if (e.getStateChange() == e.DESELECTED) {
					settings.setValue(SearchtoolSettings.FILTER_NOTPINGED_SERVERS, SearchtoolSettings.DISABLED);
				} else {
					settings.setValue(SearchtoolSettings.FILTER_NOTPINGED_SERVERS, SearchtoolSettings.ENABLED);
				}
			}
		}
	}

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	////////	STARTUP SETTINGS
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	public static class StartupPanel extends JPanel implements ChangeListener, ActionListener {
		private SearchtoolSettings settings = null;
		// GUI STUFF
		private TitledBorder titleBorder = BorderFactory.createTitledBorder("Startup settings");
		private Border emptyBorder = BorderFactory.createEmptyBorder(5, 5, 5, 5);
		private LambdaLayout layout = new LambdaLayout();
		private LambdaLayout.Constraints con = new LambdaLayout.Constraints();
		public JTextField pathToGameDir = new JTextField();
		private JTextField commandField = new JTextField();
		public JButton applyButton0 = new JButton("apply");
		private JButton applyButton1 = new JButton("apply");
		private JSlider connectionTimeout = new JSlider(0, 600);
		private JLabel timeoutStatus = new JLabel("");

		public StartupPanel(SearchtoolSettings settings) {
			this.settings = settings;
			
			this.setPreferredSize(new Dimension(300,150));
			
			
			pathToGameDir.setPreferredSize(new Dimension(100,20));
			commandField.setPreferredSize(new Dimension(100,20));
			
			// init
			connectionTimeout.setValue(settings.getValueAsInt(SearchtoolSettings.SHUTDOWN_TIMEOUT));
			timeoutStatus.setText(connectionTimeout.getValue() + " seconds");

			connectionTimeout.addChangeListener(this);
			applyButton0.addActionListener(this);
			applyButton1.addActionListener(this);

			con.a = 8; // NORTH-WEST
			con.s = "wh";
			con.x = 1;
			con.w = 10;
			setLayout(layout);
			setBorder(new CompoundBorder(titleBorder, emptyBorder));
			add(new JLabel("Path to game dir: "), con);
			con.y = 1;
			con.w = 7;
			add(pathToGameDir, con);
			con.x = 8;
			con.w = 3;
			con.a = 2;
			add(applyButton0, con);
			con.a = 8;
			con.x = 1;
			con.w = 7;
			con.y = 2;
			add(layout.createVerticalStrut(10), con);
			con.y = 3;
			add(new JLabel("Startup command : "), con);
			add(commandField, con);
			con.y = 4;
			add(commandField, con);
			con.x = 8;
			con.w = 3;
			con.a = 2;
			add(applyButton1, con);
			con.a = 8;
			con.x = 1;
			con.y = 5;
			con.w = 5;
			add(layout.createVerticalStrut(20), con);
			con.y = 6;
			add(new JLabel("Time to wait before exit after connect: "), con);
			con.y = 7;
			add(layout.createVerticalStrut(5), con);
			con.y = 8;
			add(connectionTimeout, con);
			con.x = 6;
			con.w = 5;
			add(timeoutStatus, con);

			commandField.setText(settings.getValue(SearchtoolSettings.CONNECT_COMMAND));
			pathToGameDir.setText(settings.getValue(settings.GAME_DIR));

		}

		public void actionPerformed(ActionEvent e) {
			if ( e.getSource().equals(applyButton1) )
				settings.setValue(SearchtoolSettings.CONNECT_COMMAND, commandField.getText().trim());
			if ( e.getSource().equals(applyButton0) ){
				settings.setValue(SearchtoolSettings.GAME_DIR, pathToGameDir.getText().trim());
				// default settings
				settings.setValue(SearchtoolSettings.FILTERED_MOD, "NONE");
				settings.setValue(SearchtoolSettings.FILTERED_MAP, "NONE");
			}
		}

		public void stateChanged(ChangeEvent e) {
			settings.setValue(SearchtoolSettings.SHUTDOWN_TIMEOUT, connectionTimeout.getValue());
			timeoutStatus.setText(connectionTimeout.getValue() + " seconds");
			timeoutStatus.validate();
		}

	}

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	////////	MISC SETTINGS
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	public static class MiscPanel extends JPanel implements ActionListener {
		private SearchtoolSettings settings = null;
		// GUI STUFF
		private TitledBorder titleBorder = BorderFactory.createTitledBorder("Misc settings");
		private Border emptyBorder = BorderFactory.createEmptyBorder(5, 5, 5, 5);
		private LambdaLayout layout = new LambdaLayout();
		private LambdaLayout.Constraints con = new LambdaLayout.Constraints();
		private JTextField masterPort = new JTextField();
		private JTextField masterIp = new JTextField();
		private JButton applyButton = new JButton("apply");

		public MiscPanel(SearchtoolSettings settings) {
			this.settings = settings;
			// GUI
			setPreferredSize(new Dimension(200, 150));
			
			masterPort.setPreferredSize(new Dimension(60,20));
			masterIp.setPreferredSize(new Dimension(60,20));		
			
			setLayout(layout);
			setBorder(new CompoundBorder(titleBorder, emptyBorder));
			con.s = "wh";
			con.a = 8; // NORTH-WEST
			applyButton.addActionListener(this);

			// init
			masterPort.setText(settings.getValue(SearchtoolSettings.MASTERSERVER_PORT));
			masterIp.setText(settings.getValue(SearchtoolSettings.MASTERSERVER_IP));

			con.w = 6;
			add(new JLabel("Masterserver IP: "), con);
			con.y = 1;
			add(masterIp, con);
			con.y = 2;
			add(layout.createVerticalStrut(10), con);
			con.y = 3;
			add(new JLabel("Masterserver PORT: "), con);
			con.y = 4;
			add(masterPort, con);
			con.y = 5;
			add(layout.createVerticalStrut(10), con);
			con.y = 6;
			con.x = 4;
			con.w = 2;
			add(applyButton, con);
		}

		public void actionPerformed(ActionEvent e) {
			try {
				InetAddress address = InetAddress.getByName(masterIp.getText().trim());
				Integer.parseInt(masterPort.getText().trim());
				// fine so far
				settings.setValue(SearchtoolSettings.MASTERSERVER_IP, masterIp.getText().trim());
				settings.setValue(SearchtoolSettings.MASTERSERVER_PORT, masterPort.getText().trim());
			} catch (Exception f) {
				JOptionPane optionPane = new JOptionPane("No valid address", JOptionPane.ERROR_MESSAGE);
				JDialog dialog = optionPane.createDialog(Searchtool.getGui(), "Sorry");
				dialog.show();
			}
		}
	}

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	////////	GLOBAL SETTINGS
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	public static class GlobalPanel extends JPanel implements ItemListener, ChangeListener {
		private SearchtoolSettings globalSettings = null;
		private TitledBorder titleBorder = BorderFactory.createTitledBorder("Global settings");
		private Border emptyBorder = BorderFactory.createEmptyBorder(5, 5, 5, 5);
		private LambdaLayout layout = new LambdaLayout();
		private LambdaLayout.Constraints con = new LambdaLayout.Constraints();
		
		// not using the default JSlider constructor in the following case is a workaround
		// to some weird exception on some (mostly windows) systems !!!
		private JSlider sortFilterInterval = new JSlider( new DefaultBoundedRangeModel(1500,0,300, 5000) );
		
		private JLabel sortFilterStatus = new JLabel();
		private JSlider simultaneousQueries = new JSlider(1, 100);
		private JLabel simultaneousQueriesStatus = new JLabel();
		private JComboBox lookBox = new JComboBox();
		private JComboBox bindBox = new JComboBox();

		public GlobalPanel() {
			globalSettings = Searchtool.getGlobalSettings();
			setPreferredSize(new Dimension(300, 200));
			// init
			simultaneousQueries.setValue(globalSettings.getValueAsInt(SearchtoolSettings.SIMULTANEOUS_QUERYS));
			simultaneousQueriesStatus.setText(simultaneousQueries.getValue() + " Threads");
			sortFilterInterval.setValue(globalSettings.getValueAsInt(SearchtoolSettings.SORTANDFILTER_INTERVAL));
			sortFilterStatus.setText(sortFilterInterval.getValue() + " Ms");
			// LookAndFeel settings       
			UIManager.LookAndFeelInfo[] info = UIManager.getInstalledLookAndFeels();
			for (int i = 0; i < info.length; i++) {
				lookBox.addItem(info[i].getName());
			}
			for (int i = 0; i < info.length; i++) {
				if (info[i].getName().equals(UIManager.getLookAndFeel().getName()))
					lookBox.setSelectedIndex(i);
			}
			bindBox.addItem("Connect to selected server");
			bindBox.addItem("Display details of selected server");
			bindBox.addItem("Add/remove selected server to/from favourites");
			bindBox.addItem("Add/remove selected server to/from outsiders");

			bindBox.setPreferredSize(new Dimension(150, 20));
			lookBox.setPreferredSize(new Dimension(150, 20));

			if (globalSettings.getValueAsInt(SearchtoolSettings.DOUBLECLICK_BIND) == SearchtoolSettings.CONNECT)
				bindBox.setSelectedItem("Connect to selected server");
			else if (globalSettings.getValueAsInt(SearchtoolSettings.DOUBLECLICK_BIND) == SearchtoolSettings.MAKE_FAVOURITE)
				bindBox.setSelectedItem("Add/remove selected server to/from favourites");
			else if (globalSettings.getValueAsInt(SearchtoolSettings.DOUBLECLICK_BIND) == SearchtoolSettings.MAKE_BLACKLIST)
				bindBox.setSelectedItem("Add/remove selected server to/from outsiders");
			else if (globalSettings.getValueAsInt(SearchtoolSettings.DOUBLECLICK_BIND) == SearchtoolSettings.GET_DETAILS)
				bindBox.setSelectedItem("Display details of selected server");

			sortFilterInterval.addChangeListener(this);
			simultaneousQueries.addChangeListener(this);
			bindBox.addItemListener(this);
			lookBox.addItemListener(this);

			setLayout(layout);
			setBorder(new CompoundBorder(titleBorder, emptyBorder));
			con.a = 8; // NORTH-WEST
			con.s = "wh";
			con.w = 10;

			add(new JLabel("Choose Look & Feel: "), con);
			con.y = 1;
			add(lookBox, con);
			con.y = 2;
			add(layout.createVerticalStrut(10), con);
			con.y = 3;
			add(new JLabel("Bind Doubleclick to: "), con);
			con.y = 4;
			add(bindBox, con);
			con.y = 5;
			add(layout.createVerticalStrut(10), con);
			con.y = 6;
			add(new JLabel("Choose sort/filter interval: "), con);
			con.y = 7;
			con.w = 7;
			add(layout.createVerticalStrut(5), con);
			con.y = 8;
			add(sortFilterInterval, con);
			con.x = 7;
			con.w = 3;
			add(sortFilterStatus, con);
			con.x = 0;
			con.y = 9;
			add(layout.createVerticalStrut(10), con);
			con.y = 10;
			con.w = 7;
			add(new JLabel("Choose simultaneous query number: "), con);
			con.y = 11;
			add(layout.createVerticalStrut(5), con);
			con.y = 12;
			add(simultaneousQueries, con);
			con.x = 7;
			con.w = 3;
			add(simultaneousQueriesStatus, con);
			con.y = 13;
			con.h = 1;
			add(layout.createVerticalStrut(190), con);
		}

		public void stateChanged(ChangeEvent e) {
			if (e.getSource().equals(simultaneousQueries)) {
				globalSettings.setValue(SearchtoolSettings.SIMULTANEOUS_QUERYS, simultaneousQueries.getValue());
				simultaneousQueriesStatus.setText(simultaneousQueries.getValue() + " Threads");
				simultaneousQueriesStatus.validate();
			}
			if (e.getSource().equals(sortFilterInterval)) {
				globalSettings.setValue(SearchtoolSettings.SORTANDFILTER_INTERVAL, sortFilterInterval.getValue());
				sortFilterStatus.setText(sortFilterInterval.getValue() + " Ms");
				sortFilterStatus.validate();
			}
		}

		public void itemStateChanged(ItemEvent e) {
			if (e.getSource().equals(bindBox)) {
				if (bindBox.getSelectedItem().equals("Connect to selected server"))
					globalSettings.setValue(SearchtoolSettings.DOUBLECLICK_BIND, "" + SearchtoolSettings.CONNECT);
				else if (bindBox.getSelectedItem().equals("Display details of selected server"))
					globalSettings.setValue(SearchtoolSettings.DOUBLECLICK_BIND, "" + SearchtoolSettings.GET_DETAILS);
				else if (bindBox.getSelectedItem().equals("Add/remove selected server to/from favourites"))
					globalSettings.setValue(SearchtoolSettings.DOUBLECLICK_BIND, "" + SearchtoolSettings.MAKE_FAVOURITE);
				else if (bindBox.getSelectedItem().equals("Add/remove selected server to/from outsiders"))
					globalSettings.setValue(SearchtoolSettings.DOUBLECLICK_BIND, "" + SearchtoolSettings.MAKE_BLACKLIST);
			}

			if (e.getSource().equals(lookBox)) {
				if (e.getStateChange() == e.SELECTED) {
					String previousSelectedLookAndFeelName = UIManager.getLookAndFeel().getName();
					Object selectedElement = lookBox.getSelectedItem();
					UIManager.LookAndFeelInfo[] lookFeelInfo = UIManager.getInstalledLookAndFeels();
					for (int i = 0; i < lookFeelInfo.length; i++) {
						if (lookFeelInfo[i].getName().equals(selectedElement)) {
							Searchtool.getGlobalSettings().setValue(SearchtoolSettings.LOOK_AND_FEEL, lookFeelInfo[i].getClassName());
							if (!Searchtool.getGui().setLookAndFeel())
								lookBox.setSelectedItem(previousSelectedLookAndFeelName);
							break;
						}
					}
				}
			}
		}
	}

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	////////	HELP BROWSER
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	public static class HelpBrowser extends JScrollPane implements HyperlinkListener {

		private JEditorPane editorPane = new JEditorPane();
		private TitledBorder titleBorder = BorderFactory.createTitledBorder("Help");
		private Border emptyBorder = BorderFactory.createEmptyBorder(5, 5, 5, 5);

		public HelpBrowser(String HTML_HELPFILE_PATH) {
			//setSize(new Dimension(300,300));
			setBorder(new CompoundBorder(titleBorder, emptyBorder));

			try {
				URL url = ClassLoader.getSystemResource(HTML_HELPFILE_PATH);
				editorPane.setPage(url);
			} catch (Exception e) {
				e.printStackTrace();
			}
			editorPane.setEditable(false);
			//editorPane.setMaximumSize(new Dimension(615,300));
			//editorPane.setMinimumSize(new Dimension(615, 300));
			editorPane.setPreferredSize(new Dimension(500, 200));
			editorPane.setBorder(BorderFactory.createEtchedBorder());
			this.setViewportView(editorPane);
			//addHyperlinkListener(this);
			setOpaque(false);
		}
		/**
		*  follow (local) links
		*/
		public void hyperlinkUpdate(HyperlinkEvent e) {
			if (e.getEventType().equals(HyperlinkEvent.EventType.ACTIVATED)) {
				try {
					editorPane.setPage(e.getURL());
				} catch (Exception f) {
					f.printStackTrace();
				}
			}
		}
	}
}
