/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */

package searchtool.components;

import searchtool.halflife.components.*;
import searchtool.util.IconLib;
import searchtool.actions.*;
import searchtool.*;
import javax.swing.JPanel;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import searchtool.wrappers.*;
import searchtool.interfaces.*;

/**
 * a wrapper class for a common tab structure.
 * it combines one JToolBar and one SearchtoolTable(JTable)
 * on a panel and gives it a name.
 */

public class SearchtoolTab extends JPanel {

	public static final int MAKE_FAVOURITE = 0, MAKE_BLACKLIST = 1, CONNECT = 2, GET_DETAILS = 3;
	public static final int FAVOURITE_TAB = 0, BLACKLIST_TAB = 1, MAINLIST_TAB = 2, CONNECTED_TAB = 3,PLAYERSEARCH_TAB=4;
	private static String[] NAMES = new String[]{"favourites","outsiders","main","history","playersearch"}; 
	private int tabType = MAINLIST_TAB;

	private SearchtoolButtonBar buttonbar;
	private SearchtoolTable table;
	private SelectedElementTracker model;
	private Component panel;
	private String name;
	private JScrollPane scrollpane;

	private JToggleButton[] buttons = new JToggleButton[4];

	public SearchtoolTab(SearchtoolButtonBar BUTTONBAR, SearchtoolTable TABLE,  int TYPE) {
		setOpaque(false);
		buttonbar = BUTTONBAR;
		table = TABLE;
		name = NAMES[TYPE];
		model = (SelectedElementTracker) table.getModel();
		tabType = TYPE;
		// register buttonbar
		table.registerButtonBar(buttonbar);
		// register popup menu
		table.registerPopupMenu(new SearchtoolPopupMenu(buttonbar));
		// layout
		setLayout(new BorderLayout());
		buttonbar.setPreferredSize(new Dimension(300, 70));
		scrollpane = new JScrollPane(table);
		add(buttonbar, BorderLayout.NORTH);
		add(scrollpane, BorderLayout.CENTER);
		// restore saved buttonsettings if there is a registered button-tracker
		ElementListManager listManager = model.getListManager();
		ButtonTracker tracker = listManager.getButtonTracker();
		if (tracker != null) {
			if (tracker.getValue(ButtonTracker.SORT_BUTTON).equals(ButtonTracker.ENABLED)) {
				buttonbar.getButton(SearchtoolButtonBar.SORT).setSelected(true);
			}
			if (tracker.getValue(ButtonTracker.LOCAL_FILTER_BUTTON).equals(ButtonTracker.ENABLED)) {
				buttonbar.getButton(SearchtoolButtonBar.LOCAL_FILTER).setSelected(true);
			}
		}
	}

	/*
	 * use this constructor to pass an extra panel to the tab.  the panel will appear between
	 * the buttonbar and the table.
	 */
	public SearchtoolTab(SearchtoolButtonBar BUTTONBAR, Component PANEL, SearchtoolTable TABLE,  int TYPE) {
		setOpaque(false);
		buttonbar = BUTTONBAR;
		table = TABLE;
		name = NAMES[TYPE];
		model = (SelectedElementTracker) table.getModel();
		tabType = TYPE;
		// register buttonbar
		table.registerButtonBar(buttonbar);
		// register popup menu
		table.registerPopupMenu(new SearchtoolPopupMenu(buttonbar));
		// layout
		setLayout(new BorderLayout());
		buttonbar.setPreferredSize(new Dimension(600, 50));
		JPanel tmpPanel = new JPanel();
		GridBagConstraints c = new GridBagConstraints();
		GridBagLayout layout = new GridBagLayout();
		c.gridx = c.gridy = 0;
		c.ipady = 20;
		c.anchor = c.WEST;
		c.fill = c.BOTH;
		c.weightx = 1.5;
		tmpPanel.setLayout(layout);
		c.weighty = 0.3;
		layout.setConstraints(buttonbar, c);
		tmpPanel.add(buttonbar);
		c.gridy = 1;
		layout.setConstraints(PANEL, c); 
		tmpPanel.add(PANEL);
		scrollpane = new JScrollPane(TABLE);
		add(tmpPanel, BorderLayout.NORTH);
		add(scrollpane, BorderLayout.CENTER);
		
		ElementListManager listManager = model.getListManager();
		// restore saved buttonsettings if there is a registered button-tracker
		ButtonTracker tracker = null;
		if(listManager != null)
			tracker = listManager.getButtonTracker();
		if (tracker != null) {
			if (tracker.getValue(ButtonTracker.SORT_BUTTON).equals(ButtonTracker.ENABLED)) {
				buttonbar.getButton(SearchtoolButtonBar.SORT).setSelected(true);
			}
			if (tracker.getValue(ButtonTracker.LOCAL_FILTER_BUTTON).equals(ButtonTracker.ENABLED)) {
				buttonbar.getButton(SearchtoolButtonBar.LOCAL_FILTER).setSelected(true);
			}
		}
	}

	public int getTabType() {
		return tabType;
	}
	
	public SearchtoolTable getSearchtoolTable(){
		return table;
	}

	public static String getTabName(int TABTYPE){
		return NAMES [TABTYPE];
	}

	/*
	 * return tabname
	 */
	public String getName() {
		return name;
	}
	/*
	 *  set tabname
	 */
	public void setName(String name){
		this.name = name;
	}
	
}
