/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package searchtool.gamespy;

import searchtool.*;
import searchtool.interfaces.*;
import searchtool.protocol.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.text.*;
import searchtool.wrappers.*;
import searchtool.util.*;

/**
 * @author thorsten
 *
 */
public class GamespyDetails extends SearchtoolProtocol {

	private static String query_string = "\\info\\";
	private static byte[] sndbyte = query_string.getBytes();
	private static final byte RESPONSE_SEPARATOR = 92; // the separator of the response string: "\"

	protected static final byte[] SERVERNAME, MAPNAME, MOD, CURRENTPLAYERS, MAXPLAYERS, GAMEMODE, HOSTPORT;
	static {
		SERVERNAME = "hostname".getBytes();
		MAPNAME = "mapname".getBytes();
		MOD = "gametype".getBytes();
		CURRENTPLAYERS = "numplayers".getBytes();
		MAXPLAYERS = "maxplayers".getBytes();
		GAMEMODE = "gamemode".getBytes();
		HOSTPORT = "hostport".getBytes();
	}

	private byte[] getValueFor(byte[] VARIABLE, byte[] response) {
		int position = 0;
		for (position = 0; position < 400; position++) {
			if (position == response.length - 1)
				break; // end of response
			if (response[position] == VARIABLE[0]) {
				for (int i = 1; i < VARIABLE.length; i++) {
					if (response[position + i] != VARIABLE[i]) {
						break;
					}
					if (i == VARIABLE.length - 1) {
						position = position + VARIABLE.length + 1; // skip variable and separator
						// check if skipped char was separator, if not ---> break               
						if (response[position - 1] != RESPONSE_SEPARATOR) {
							break;
						}
						int tmp = position;
						while (response[position] != RESPONSE_SEPARATOR && response[position] != 0)
							position++;
						byte[] returnme = new byte[position - tmp];
						System.arraycopy(response, tmp, returnme, 0, returnme.length);
						//System.out.println(new String(VARIABLE) + "              " + new String(returnme));
						return returnme;
					}
				}
			}
		}
		return null;
	}

	protected void fill() throws Exception {
		DatagramSocket socket = null;
		DatagramPacket snd_packet = null;
		DatagramPacket get_packet = null;
		InetAddress inet_address = null;

		try {
			byte[] getbyte = new byte[2000];
			inet_address = InetAddress.getByName(super.getServerElement().getIp());
			socket = new DatagramSocket();
			snd_packet = new DatagramPacket(sndbyte, sndbyte.length, inet_address, super.getServerElement().getPort().intValue());
			get_packet = new DatagramPacket(getbyte, getbyte.length);
			socket.send(snd_packet);
			socket.setSoTimeout(2000);
			socket.receive(get_packet);
			
			//System.out.println(super.getServerElement().getIp() + "  " + super.getServerElement().getPort() +  "\n");
			//System.out.println(new String(getbyte).trim()+"\n\n\n");
			

			byte[] tmpbyte = getValueFor(SERVERNAME, getbyte);
			if (tmpbyte != null) {
				super.element.setServername(new String(tmpbyte));
			}
			tmpbyte = getValueFor(MAPNAME, getbyte);
			if (tmpbyte != null) {
				super.element.setMap(new String(tmpbyte));
			}
			tmpbyte = getValueFor(CURRENTPLAYERS, getbyte);
			if (tmpbyte != null) {
				super.element.setCurrentplayers(Integer.parseInt(new String(tmpbyte)));
			}
			tmpbyte = getValueFor(MAXPLAYERS, getbyte);
			if (tmpbyte != null) {
				super.element.setMaxplayers(Integer.parseInt(new String(tmpbyte)));
			}
			tmpbyte = getValueFor(HOSTPORT,getbyte);
			if ( tmpbyte != null){
				super.element.setGamePort(Integer.parseInt(new String(tmpbyte).trim()));
			}
			tmpbyte = getValueFor(MOD,getbyte);
			if ( tmpbyte != null){
				super.element.setMod(new String(tmpbyte));
			}
			
			//tmpbyte = getValueFor(GAMEMODE,getbyte);
			//if ( ! new String(tmpbyte).equals("openplaying") )
				//System.out.println(new String(tmpbyte));

			//System.out.println(new String(getbyte).trim());
			socket.close();
		} catch (Exception e) {
			socket.close();
			throw e;
		}
	}
}
