/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
package searchtool.gamespy;

import searchtool.*;
import searchtool.interfaces.*;
import searchtool.protocol.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.text.*;
import searchtool.wrappers.*;
import searchtool.util.*;

/**
 * @author thorsten
 *
 */
public class GamespyPlayers extends SearchtoolProtocol {

	private static String query_string = "\\players\\";
	private static byte[] sndbyte = query_string.getBytes();
	private static final byte RESPONSE_SEPARATOR = 92; // the separator of the response string: "\"

	public void fill() throws Exception {
		DatagramSocket socket = null;
		DatagramPacket snd_packet = null;
		DatagramPacket get_packet = null;
		InetAddress inet_address = null;
		Vector playerVector = new Vector();

		try {
			byte[] getbyte = new byte[2000];
			inet_address = InetAddress.getByName(super.getServerElement().getIp());
			socket = new DatagramSocket();
			snd_packet = new DatagramPacket(sndbyte, sndbyte.length, inet_address, super.getServerElement().getPort().intValue());
			get_packet = new DatagramPacket(getbyte, getbyte.length);

			socket.send(snd_packet);
			socket.setSoTimeout(5000);

			boolean morePackages = true;
			do {
				socket.receive(get_packet);
				//System.out.println(new String(getbyte).trim() + "\n\n\n");
				String[] responseData = parseResponse(getbyte);
				morePackages = processResponse(responseData, playerVector);
				// reset
				Arrays.fill(getbyte,(byte)0);
			} while (morePackages);
			
			getServerElement().setCurrentPlayerList( (String[]) playerVector.toArray(new String[0]) );

			socket.close();
		} catch (Exception e) {
			socket.close();
			throw e;
		}
	}

	private int currentNumber = 0;
	private int maximumNumber = -1;
	private boolean processResponse(String[] response, Vector playerVector) throws Exception {
		// check for final package
		for ( int i = response.length - 1; i >= 0; i-- ){
			// only if final package :
			if ( response[i].equals("final") ){
				for ( int j = response.length - 1; j >= 0; j--){
					if ( response[j].equals("queryid") ){
						int position = response[ j + 1].indexOf(".");
						maximumNumber = Integer.parseInt((String) response[ j + 1].substring(position + 1));
					}
				}
			}
		}

		// store package
		currentNumber ++;

		// process players
		for ( int i = 0; i < response.length; i++){
			if ( response[i].startsWith("player_") ){
				playerVector.add(response[ i + 1 ]);	
			}				
		}

		// tell if all packages received
		if (currentNumber == maximumNumber) {
			// clear
			currentNumber = 0;
			maximumNumber = -1;
			return false;
		} else
			return true;
	}

	private String[] parseResponse(byte[] getbyte) {
		StringBuffer stringBuffer = new StringBuffer();
		ArrayList list = new ArrayList();
		for (int i = 1 /* omit initial separator*/; i < getbyte.length; i++) {
			if ( getbyte[i] == 0 ){
				list.add(stringBuffer.toString());
				stringBuffer.delete(0,stringBuffer.length());
				break;	
			}
			if (getbyte[i] == RESPONSE_SEPARATOR ) {
				
				// if multiple backslashes in a row ...
				if ( getbyte[i+1] == RESPONSE_SEPARATOR )
					continue;
				
				list.add(stringBuffer.toString());
				stringBuffer.delete(0, stringBuffer.length());			
			} else {
				stringBuffer.append((char)getbyte[i]);
			}
		}
		return (String[]) list.toArray(new String[0]);
	}
}
