/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package searchtool.gamespy;

import java.io.*;
import java.net.*;
import java.util.*;

import searchtool.components.*;
import searchtool.wrappers.*;
import searchtool.protocol.*;
import searchtool.util.*;
// thirdparty
import gnu.trove.*;

/**
 * @author thorsten
 *
 */
public class GamespyServerRequest extends ServerRequest {

	private static String request = null;
	private static byte[] requestBytes = null;
	boolean sessionIncomplete = true;

	public GamespyServerRequest(SearchtoolSettings settings, String ip, int port, String filter, String game) {
		super(ip, port, filter);
		super.settings = settings;
		// only create once to save some work
		if ( request == null ){
			request = "\\list\\gamename\\"+game.toLowerCase()+"\\final";
			requestBytes = request.getBytes();
		}
	}

	public GamespyServerRequest(SearchtoolSettings settings, String game) {
		super(settings);
		// only create once to save some work
		if ( request == null ){
			request = "\\list\\gamename\\"+game.toLowerCase()+"\\final";
			requestBytes = request.getBytes();
		}
	}

	public boolean hasNext() {
		return sessionIncomplete;
	}
	public ServerElement[] getNext(boolean masterlevelFilter) throws Exception {
		NonBlockingConnectSocket blockingThread = new NonBlockingConnectSocket(super.masterIp, super.masterPort);
		blockingThread.start();
		int counter = 0;
		while ( true ){
			Thread.sleep(300);	
			counter += 300;
			if ( counter >= 6000 )
				throw new Exception("connection timedout");
			if ( blockingThread.getSocket() != null /*&& blockingThread.getSocket().isConnected()*/ )
				break;
		}
		OutputStream out = blockingThread.getSocket().getOutputStream();
		InputStream in = blockingThread.getSocket().getInputStream();
		out.write(requestBytes);
		StringBuffer resultBuffer = new StringBuffer();
		int read = in.read();
		while (read != -1) {
			//System.out.print(read);
			resultBuffer.append((char) read);
			read = in.read();
		}
		in.close();
		out.close();
		blockingThread.getSocket().close();
		//System.out.println(resultBuffer.toString());
		ArrayList ipList = new ArrayList();
		StringTokenizer tokenizer = new StringTokenizer(resultBuffer.toString(), "\\");
		while (tokenizer.hasMoreTokens()) {
			if (tokenizer.nextToken().equals("ip"))
				ipList.add(tokenizer.nextToken());
		}
		//System.out.println(ipList.size());
		tokenizer = null;
		super.clearTempServerlist();
		int position;
		String address;
		String ip;
		int port;
		int ip1, ip2, ip3, ip4;
		for (int i = 0; i < ipList.size(); i++) {
			address = (String) ipList.get(i);
			position = address.indexOf(":");
			ip = address.substring(0, position);
			port = Integer.parseInt(address.substring(position + 1));
			tokenizer = new StringTokenizer(ip, ".");
			ip1 = Integer.parseInt(tokenizer.nextToken());
			ip2 = Integer.parseInt(tokenizer.nextToken());
			ip3 = Integer.parseInt(tokenizer.nextToken());
			ip4 = Integer.parseInt(tokenizer.nextToken());
			super.storeServerAddress( new ServerElement(new IpWrapper(ip1, ip2, ip3, ip4, port)) );
		}
		sessionIncomplete = false;
		return super.getStoredServerElements();
	}

	public void cleanUp() {
		sessionIncomplete = true;
	}
}
