/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package searchtool.halflife;

import searchtool.halflife.components.*;
import searchtool.*;
import searchtool.components.*;
import searchtool.interfaces.*;
import searchtool.halflife.*;
import searchtool.util.*;
import searchtool.wrappers.*;
import searchtool.halflife.wrappers.*;
import searchtool.halflife.protocol.*;
import searchtool.actions.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
// third party
import com.jclark.xml.parse.*;
import com.jclark.xml.parse.base.*;
import com.jclark.xml.parse.io.ApplicationImpl;
import com.jclark.xml.output.*;

/**
 * @author thorsten
 *
 */
public class HlGameController extends GameController {

	public HlGameController(String gamePrefix) {
		super(gamePrefix);
		// override settings
		super.settings = new HlSettings(super.configFile);
		super.settings.load();
		// override components
		super.editSettingsPanel = new HlEditSettingsPanel((HlSettings) settings,"searchtool/files/" + gamePrefix + "_settings.html" );
		super.searchPlayersPanel = new SearchPlayersPanel( this, new HL_Players(), new HL_Details() );
	}

	protected void initActionListeners(){
		super.initActionListeners();
		// override few actions
		super.addServer = new AddServerToListActionListener(super.elementController, this, new HlServerElement());
		super.getServerlist = new DownloadServersActionListener(super.elementController, this, super.buttonController, new HL_WON_Connection(super.settings));
		super.editSettings = new EditSettingsActionListener(super.editSettingsPanel, EDITSETTINGS_TABNAME, super.buttonController);
		super.getDetails = new GetDetailsActionListener(new HL_Ping(), new HL_Rules(), new HL_Details(), new HL_Players(), this.rulesConfigFile,super.settings);
		super.pingServerList = new PingServersActionListener(this, super.settings, new HL_Ping(null), new HL_Details(null), null /* no rules for list refresh */);
		super.connect = new ConnectActionListener(new HL_Details(), super.settings);	
	}
	
	protected void initBlacklistTable(){
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// BLACKLIST TABLE
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		
		super.blacklistButtontracker = new ButtonTracker(super.blackListButtonSettingsFile);
		super.blacklistManager = new HlServerListManager((HlSettings) super.settings, super.blacklistButtontracker);
		// register manager
		super.elementController.registerListManager(super.blacklistManager, ElementController.BLACK_LIST);
		super.blacklistTablemodel = new HlTableModel((HlServerListManager)super.blacklistManager);
		super.blacklistTable = new SearchtoolTable(super.blacklistTablemodel);

		// wrap tablemodel in thread to make sure it gets updated
		Thread hl_tablemodel_thread = new Thread(super.blacklistTablemodel);
		hl_tablemodel_thread.start();
		
		super.outsiderHeaderListener = new HlTableHeaderSortingController(blacklistTablemodel,blacklistButtontracker);
		super.blacklistTable.getTableHeader().addMouseListener(outsiderHeaderListener);
		super.outsiderHeaderListener.registerTable(blacklistTable);
		super.blacklistManager.setComparator(super.outsiderHeaderListener);

		// add cell renderers
		String columname = super.blacklistTable.getColumnName(SearchtoolTableModel.RESPONSE);
		super.blacklistTable.getColumn(columname).setCellRenderer(new HlTableCellRenderer());
		columname = super.blacklistTable.getColumnName(SearchtoolTableModel.PASSWORD);
		super.blacklistTable.getColumn(columname).setCellRenderer(new HlTableCellRenderer());
		columname = super.blacklistTable.getColumnName(HlTableModel.SECURE);
		super.blacklistTable.getColumn(columname).setCellRenderer(new HlTableCellRenderer());

		// customize columsizes
		super.blacklistTable.getColumn("SECURE").setPreferredWidth(50);
	}
	
	protected void initFavouritelistTable(){
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// FAVOURITELIST TABLE
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

		super.favouritelistButtontracker = new ButtonTracker(super.favouriteListButtonSettingsFile);
		super.favouritelistManager = new HlServerListManager((HlSettings) super.settings, super.favouritelistButtontracker);
		// register listmanager
		super.elementController.registerListManager(super.favouritelistManager, ElementController.FAVOURITE_LIST);
		super.favouritelistTablemodel = new HlTableModel((HlServerListManager)super.favouritelistManager);
		super.favouritelistTable = new SearchtoolTable(super.favouritelistTablemodel);

		// wrap tablemodel in thread to make sure it gets updated
		Thread hl_tablemodel_thread = new Thread(super.favouritelistTablemodel);
		hl_tablemodel_thread.start();
		
		super.favouriteHeaderListener = new HlTableHeaderSortingController(favouritelistTablemodel,favouritelistButtontracker);
		super.favouritelistTable.getTableHeader().addMouseListener(favouriteHeaderListener);
		super.favouriteHeaderListener.registerTable(favouritelistTable);
		super.favouritelistManager.setComparator(super.favouriteHeaderListener);

		// add cell renderers
		String columname = super.favouritelistTable.getColumnName(SearchtoolTableModel.RESPONSE);
		super.favouritelistTable.getColumn(columname).setCellRenderer(new HlTableCellRenderer());
		columname = super.favouritelistTable.getColumnName(SearchtoolTableModel.PASSWORD);
		super.favouritelistTable.getColumn(columname).setCellRenderer(new HlTableCellRenderer());
		columname = super.favouritelistTable.getColumnName(HlTableModel.SECURE);
		super.favouritelistTable.getColumn(columname).setCellRenderer(new HlTableCellRenderer());

		// customize columsizes 
		super.favouritelistTable.getColumn("SECURE").setPreferredWidth(50);
	}
	
	protected void initMainlistTable(){
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// MAIN TABLE
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

		super.mainlistButtontracker = new ButtonTracker(super.mainListButtonSettingsFile);
		super.serverListManager = new HlServerListManager((HlSettings) super.settings, super.mainlistButtontracker);
		// register listmanager
		super.elementController.registerListManager(super.serverListManager, ElementController.MAIN_LIST);
		super.mainlistTablemodel = new HlTableModel((HlServerListManager)super.serverListManager);
		// wrap tablemodel in thread to make sure it gets updated
		Thread hl_tablemodel_thread = new Thread(super.mainlistTablemodel);
		hl_tablemodel_thread.start();
		// create table
		super.hl_table = new SearchtoolTable(super.mainlistTablemodel);
		
		super.mainHeaderListener = new HlTableHeaderSortingController(mainlistTablemodel,mainlistButtontracker);
		super.hl_table.getTableHeader().addMouseListener(mainHeaderListener);
		super.mainHeaderListener.registerTable(hl_table);
		super.serverListManager.setComparator(super.mainHeaderListener);


		// add cell renderers
		String columname = super.hl_table.getColumnName(SearchtoolTableModel.RESPONSE);
		super.hl_table.getColumn(columname).setCellRenderer(new HlTableCellRenderer());
		columname = super.hl_table.getColumnName(SearchtoolTableModel.PASSWORD);
		super.hl_table.getColumn(columname).setCellRenderer(new HlTableCellRenderer());
		columname = super.hl_table.getColumnName(HlTableModel.SECURE);
		super.hl_table.getColumn(columname).setCellRenderer(new HlTableCellRenderer());

		// customize columsizes
		super.hl_table.getColumn("SECURE").setPreferredWidth(50);
		
	}
	
	protected void initSearchresultlistTable(){
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// SEARCHRESULT TABLE
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

		super.resultlistButtontracker = new ButtonTracker(super.resultListButtonSettingsFile);
		// initialize the customtablemodle
		super.resultlistManager = new HlResultListManager(super.settings, super.resultlistButtontracker);
		super.resultlistTablemodel = new HlResultTableModel(super.resultlistManager);
		super.searchresultlistTable = new SearchtoolTable(super.resultlistTablemodel);

		Thread thread = new Thread(super.resultlistTablemodel);
		thread.start();
		
		super.resultHeaderListener = new HlTableHeaderSortingController(resultlistTablemodel,resultlistButtontracker);
		super.searchresultlistTable.getTableHeader().addMouseListener(resultHeaderListener);
		super.resultHeaderListener.registerTable(searchresultlistTable);
		super.resultlistManager.setComparator(super.resultHeaderListener);

		// custom renderers
		String columname = super.resultlistTablemodel.getColumnName(SearchtoolTableModel.PASSWORD);
		super.searchresultlistTable.getColumn(columname).setCellRenderer(new HlTableCellRenderer());
		columname = super.resultlistTablemodel.getColumnName(HlTableModel.SECURE);
		super.searchresultlistTable.getColumn(columname).setCellRenderer(new HlTableCellRenderer());
		// customize columsizes

		super.searchresultlistTable.getColumn("SECURE").setPreferredWidth(50);
		super.searchresultlistTable.getColumn("PLAYER").setPreferredWidth(200);	
	}
	
	protected void initLastconnectedlistTable(){
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// LASTCONNECTED TABLE
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

		super.lastconnectedlistButtontracker = new ButtonTracker(super.lastconnectedListButtonSettingsFile);
		// initialize the customtablemodel
		super.lastconnectedlistManager = new HlServerListManager((HlSettings)super.settings, super.lastconnectedlistButtontracker);
		// register listmanager
		super.elementController.registerListManager(super.lastconnectedlistManager, ElementController.CONNECTED_LIST);
		super.lastconnectedlistTablemodel= new HlLastconnectedTableModel((HlServerListManager)super.lastconnectedlistManager);
		Thread thread = new Thread(super.lastconnectedlistTablemodel); // wrap model in thread to take care of
		thread.start(); // the updating of the table
		super.lastconnectedTable = new SearchtoolTable(super.lastconnectedlistTablemodel);

		super.lastconnectedSortingController = new HlTableHeaderSortingController(lastconnectedlistTablemodel,lastconnectedlistButtontracker);
		super.lastconnectedTable.getTableHeader().addMouseListener(lastconnectedSortingController);
		super.lastconnectedSortingController.registerTable(lastconnectedTable);
		super.lastconnectedlistManager.setComparator(super.lastconnectedSortingController);

		// add cell renderers
		String columname = super.lastconnectedlistTablemodel.getColumnName(SearchtoolTableModel.RESPONSE);
		super.lastconnectedTable.getColumn(columname).setCellRenderer(new HlTableCellRenderer());
		columname = super.lastconnectedlistTablemodel.getColumnName(SearchtoolTableModel.PASSWORD);
		super.lastconnectedTable.getColumn(columname).setCellRenderer(new HlTableCellRenderer());
		columname = super.lastconnectedlistTablemodel.getColumnName(HlTableModel.SECURE);
		super.lastconnectedTable.getColumn(columname).setCellRenderer(new HlTableCellRenderer());
		
		// customize columsizes
		lastconnectedTable.getColumn("SECURE").setPreferredWidth(50);
		lastconnectedTable.getColumn("CONNECTIONDATE").setPreferredWidth(200);
	}

	/**
	 *  return the xmlparser of this game controller
	 */
	public ServerElement.ListDataElementParserCallback getXMLParser() {
		return new HlServerElement.HlDataElementParserCallback();
	}

	/**
	 *  return the xmlwriter of this game controller
	 */
	public ServerElement.ListDataElementXMLWriter getXMLWriter() {
		return new HlServerElement.HlDataElementXMLWriter(serverFile);
	}
}
