/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */
package searchtool.halflife;

import searchtool.*;
import searchtool.interfaces.*;
import searchtool.halflife.wrappers.*;
import searchtool.util.*;
import searchtool.wrappers.*;
import searchtool.actions.*;
import javax.swing.event.*;
import javax.swing.*;
import java.util.*;
import java.lang.*;
import java.io.*;
// thirdparty
import gnu.trove.*;

/**
 * @author thorsten
 *
 */
public class HlResultListManager extends SearchtoolResultListManager {

	public HlResultListManager(SearchtoolSettings SETTINGS, ButtonTracker BUTTONTRACKER) {
		super(SETTINGS, BUTTONTRACKER);
	}

	/*
	 * filter out servers depending on the settings.
	 */
	public synchronized void filterElementList() {
		ArrayList tmp = new ArrayList(completeList.size());
		// some stack variables for better performance
		String filteredMap = super.settings.getValue(SearchtoolSettings.FILTERED_MAP);
		String filteredMod = super.settings.getValue(SearchtoolSettings.FILTERED_MOD);

		boolean filterNotPingedServers = false;
		if (super.settings.getValueAsInt(SearchtoolSettings.FILTER_NOTPINGED_SERVERS) == SearchtoolSettings.ENABLED)
			filterNotPingedServers = true;

		boolean filterEmptyServers = false;
		if (super.settings.getValueAsInt(SearchtoolSettings.FILTER_EMPTY) == SearchtoolSettings.ENABLED)
			filterEmptyServers = true;

		boolean filterFullServers = false;
		if (super.settings.getValueAsInt(SearchtoolSettings.FILTER_FULL) == SearchtoolSettings.ENABLED)
			filterFullServers = true;

		boolean filterPublicServers = false;
		if (super.settings.getValueAsInt(SearchtoolSettings.FILTER_PUBLIC_SERVERS) == SearchtoolSettings.ENABLED)
			filterPublicServers = true;

		boolean filterPrivateServers = false;
		if (super.settings.getValueAsInt(SearchtoolSettings.FILTER_PRIVATE_SERVERS) == SearchtoolSettings.ENABLED)
			filterPrivateServers = true;

		boolean filterSecureServers = false;
		if (super.settings.getValueAsInt(HlSettings.FILTER_SECURE_SERVERS) == SearchtoolSettings.ENABLED)
			filterSecureServers = true;

		boolean filterUnsecureServers = false;
		if (super.settings.getValueAsInt(HlSettings.FILTER_UNSECURE_SERVERS) == SearchtoolSettings.ENABLED)
			filterUnsecureServers = true;

		int size = completeList.size();

		for (int i = 0; i < size; i++) {
			// SPECIAL (highest priority)
			if (((HlServerElement) ((SearchtoolPlayer)completeList.get(i)).getServerElement()).getResponse().byteValue() == ServerElement.SPECIAL.byteValue()) {
				tmp.add(completeList.get(i));
				continue;
			}
			// AGE and RESPONSE_NEGATIVE
			if (!filterNotPingedServers
				&&/* (((HlServerElement) ((SearchtoolPlayer)completeList.get(i)).getServerElement()).getResponse().byteValue() == ServerElement.OLD.byteValue()
					|| */((HlServerElement) ((SearchtoolPlayer)completeList.get(i)).getServerElement()).getResponse().byteValue() == ServerElement.RESPONSE_NEGATIVE.byteValue()) {
				//System.out.println("old");
				continue;
			}
			// EMPTY SERVERS
			if (!filterEmptyServers && ((HlServerElement) ((SearchtoolPlayer)completeList.get(i)).getServerElement()).getCurrentplayers().intValue() == 0) {
				//System.out.println("empty");
				continue;
			}
			// FULL SERVERS
			if (!filterFullServers && ((HlServerElement) ((SearchtoolPlayer)completeList.get(i)).getServerElement()).getCurrentplayers().intValue() == ((HlServerElement) completeList.get(i)).getMaxplayers().intValue()) {
				//System.out.println("full");
				continue;
			}
			// PUBLIC SERVERS
			if (!filterPublicServers && ((HlServerElement) ((SearchtoolPlayer)completeList.get(i)).getServerElement()).getPassword() == ServerElement.NO_PASSWD) {
				//System.out.println("public");
				continue;
			}
			// PRIVATE SERVERS
			if (!filterPrivateServers && ((HlServerElement) ((SearchtoolPlayer)completeList.get(i)).getServerElement()).getPassword() == ServerElement.PASSWD_REQUIRED) {
				//System.out.println("private");
				continue;
			}
			// SECURE SERVERS
			if (!filterSecureServers && ((HlServerElement)((SearchtoolPlayer)completeList.get(i)).getServerElement()).getSecure() == HlServerElement.SECURE) {
				//System.out.println("secure");
				continue;
			}
			// UNSECURE SERVERS
			if (!filterUnsecureServers && ((HlServerElement) ((SearchtoolPlayer)completeList.get(i)).getServerElement()).getSecure() == HlServerElement.UNSECURE) {
				//System.out.println("unsecure");
				continue;
			}
			// MAP
			if (!((HlServerElement) ((SearchtoolPlayer)completeList.get(i)).getServerElement()).getMap().equals(filteredMap) && !filteredMap.equals("NONE")) {
				//System.out.println("map");
				continue;
			}
			// MOD
			if (!((HlServerElement) ((SearchtoolPlayer)completeList.get(i)).getServerElement()).getMod().equals(filteredMod) && !filteredMod.equals("NONE")) {
				// filteredmod set to value of game's directory. see hlinfocollectprocess
				//System.out.println("mod");
				continue;
			}
			// if we got this far this server is ok, lets keep it.
			tmp.add(completeList.get(i));
		}
		super.filteredServerList = tmp;
	}
}
