/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * HlServerListManager.java
 *
 * Created on January 5, 2002, 12:06 AM
 */

package searchtool.halflife;

import searchtool.components.GUI;
import searchtool.*;
import searchtool.util.*;
import searchtool.halflife.protocol.*;
import searchtool.halflife.wrappers.HlServerElement;
import searchtool.halflife.wrappers.HlSettings;
import searchtool.wrappers.*;
import searchtool.interfaces.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import java.io.*;
import java.net.*;

/**
 *
 * @author  zoidberg
 * @version
 */
public class HlServerListManager extends ServerListManager {

	public static final String SECURE_ATT = "SEC", MOD_ATT = "MOD";

	// as always, the central settings object.
	private HlSettings hlsettings;

	/** Creates new HlServerListManager */
	public HlServerListManager(HlSettings SETTINGS, ButtonTracker BUTTONTRACKER) {
		super((SearchtoolSettings) SETTINGS, BUTTONTRACKER);
		hlsettings = SETTINGS;
	}

	/*
	 * overridden method from superclass 
	 */
	public synchronized void filterElementList() {

		ArrayList tmp = new ArrayList(completeList.size());
		// some stack variables for better performance
		String filteredMap = super.settings.getValue(SearchtoolSettings.FILTERED_MAP);
		String filteredMod = super.settings.getValue(SearchtoolSettings.FILTERED_MOD);

		boolean filterNotPingedServers = false;
		if (super.settings.getValueAsInt(SearchtoolSettings.FILTER_NOTPINGED_SERVERS) == SearchtoolSettings.ENABLED)
			filterNotPingedServers = true;

		boolean filterEmptyServers = false;
		if (super.settings.getValueAsInt(SearchtoolSettings.FILTER_EMPTY) == SearchtoolSettings.ENABLED)
			filterEmptyServers = true;

		boolean filterFullServers = false;
		if (super.settings.getValueAsInt(SearchtoolSettings.FILTER_FULL) == SearchtoolSettings.ENABLED)
			filterFullServers = true;

		boolean filterPublicServers = false;
		if (super.settings.getValueAsInt(SearchtoolSettings.FILTER_PUBLIC_SERVERS) == SearchtoolSettings.ENABLED)
			filterPublicServers = true;

		boolean filterPrivateServers = false;
		if (super.settings.getValueAsInt(SearchtoolSettings.FILTER_PRIVATE_SERVERS) == SearchtoolSettings.ENABLED)
			filterPrivateServers = true;

		boolean filterSecureServers = false;
		if (super.settings.getValueAsInt(HlSettings.FILTER_SECURE_SERVERS) == SearchtoolSettings.ENABLED)
			filterSecureServers = true;

		boolean filterUnsecureServers = false;
		if (super.settings.getValueAsInt(HlSettings.FILTER_UNSECURE_SERVERS) == SearchtoolSettings.ENABLED)
			filterUnsecureServers = true;

		int size = completeList.size();

		for (int i = 0; i < size; i++) {
			// SPECIAL (highest priority)
			if (((HlServerElement) completeList.get(i)).getResponse().byteValue() == ServerElement.SPECIAL.byteValue()) {
				tmp.add(completeList.get(i));
				continue;
			}
			// AGE and RESPONSE_NEGATIVE
			if (!filterNotPingedServers
				&& /*(((HlServerElement) completeList.get(i)).getResponse().byteValue() == ServerElement.OLD.byteValue()
					|| */((HlServerElement) completeList.get(i)).getResponse().byteValue() == ServerElement.RESPONSE_NEGATIVE.byteValue()) {
				//System.out.println("response negative");
				continue;
			}
			// EMPTY SERVERS
			if (!filterEmptyServers && ((HlServerElement) completeList.get(i)).getCurrentplayers().intValue() == 0) {
				//System.out.println("empty");
				continue;
			}
			// FULL SERVERS
			if (!filterFullServers && ((HlServerElement) completeList.get(i)).getCurrentplayers().intValue() == ((HlServerElement) completeList.get(i)).getMaxplayers().intValue()) {
				//System.out.println("full");
				continue;
			}
			// PUBLIC SERVERS
			if (!filterPublicServers && ((HlServerElement) completeList.get(i)).getPassword() == ServerElement.NO_PASSWD) {
				//System.out.println("public");
				continue;
			}
			// PRIVATE SERVERS
			if (!filterPrivateServers && ((HlServerElement) completeList.get(i)).getPassword() == ServerElement.PASSWD_REQUIRED) {
				//System.out.println("private");
				continue;
			}
			// SECURE SERVERS
			if (!filterSecureServers && ((HlServerElement) completeList.get(i)).getSecure() == HlServerElement.SECURE) {
				//System.out.println("secure");
				continue;
			}
			// UNSECURE SERVERS
			if (!filterUnsecureServers && ((HlServerElement) completeList.get(i)).getSecure() == HlServerElement.UNSECURE) {
				//System.out.println("unsecure");
				continue;
			}
			// MAP
			if (!((HlServerElement) completeList.get(i)).getMap().equals(filteredMap) && !filteredMap.equals("NONE")) {
				//System.out.println("map");
				continue;
			}
			// MOD
			if (!((HlServerElement) completeList.get(i)).getMod().equals(filteredMod) && !filteredMod.equals("NONE")) {
				// filteredmod set to value of game's directory. see hlinfocollectprocess
				//System.out.println("mod");
				continue;
			}
			// if we got this far this server is ok, lets keep it.
			tmp.add(completeList.get(i));
		}
		super.filteredServerList = tmp;
	}
}
