/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
 * HlEditSettingsPanel.java
 *
 * Created on January 16, 2002, 7:21 PM
 */
package searchtool.halflife.components;

import searchtool.components.*;
import searchtool.components.GUI;
import searchtool.components.SearchtoolButtonBar;
import searchtool.util.IconLib;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import searchtool.*;
import searchtool.wrappers.*;
import searchtool.interfaces.*;
import searchtool.halflife.components.*;
import searchtool.halflife.wrappers.*;
import java.io.*;
import java.util.*;
import java.net.*;
// thirdparty
import ise.java.awt.*;

/**
 * this panel contains all the components that build up
 * the 'edit settings' dialog.
 *
 *
 * @author  zoidberg
 * @version
 */
public class HlEditSettingsPanel extends JPanel {

	private String TABNAME = GameController.EDITSETTINGS_TABNAME;

	/** Creates new HlEditSettingsPanel */
	public HlEditSettingsPanel(SearchtoolSettings hlsettings, String HTML_HELPFILE_PATH) {

		LambdaLayout layout = new LambdaLayout();
		LambdaLayout.Constraints con = new LambdaLayout.Constraints();

		con.a = 8; // NORTH-WEST
		con.s = "wh";
		con.x = 0;
		setLayout(layout);

		// upper left
		con.w = 2;
		add(new HlFilterPanel(hlsettings), con);

		// upper center
		con.x = 2;
		con.w = 6;
		add(new SearchtoolSettingsPanel.StartupPanel(hlsettings), con);

		// upper right
		con.x = 8;
		con.w = 2;
		add(new SearchtoolSettingsPanel.MiscPanel(hlsettings), con);

		// lower left
		con.x = 0;
		con.w = 3;
		con.y = 1;
		add(new SearchtoolSettingsPanel.GlobalPanel(), con);

		// lower center
		con.x = 3;
		con.w = 7;
		add(new SearchtoolSettingsPanel.HelpBrowser(HTML_HELPFILE_PATH), con);

	}

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	////////	FILTER SETTINGS
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	private class HlFilterPanel extends JPanel implements ItemListener {
		private SearchtoolSettings hlsettings = null;
		// GUI STUFF
		private TitledBorder titleBorder = BorderFactory.createTitledBorder("Local filter settings");
		private Border emptyBorder = BorderFactory.createEmptyBorder(5, 5, 5, 5);
		private KappaLayout layout = new KappaLayout();
		private KappaLayout.Constraints con = new KappaLayout.Constraints();

		private JCheckBox emptyServers = new JCheckBox("empty servers");
		private JCheckBox fullServers = new JCheckBox("full servers");

		private JCheckBox secureServers = new JCheckBox("secure servers");
		private JCheckBox unsecureServers = new JCheckBox("unsecure servers");

		private JCheckBox publicServers = new JCheckBox("public servers");
		private JCheckBox privateServers = new JCheckBox("private servers");
		
		private JCheckBox notPingedServers = new JCheckBox("not checked servers");

		private HlFilterPanel(SearchtoolSettings hlsettings) {
			this.hlsettings = hlsettings;
			// setup panel
			setLayout(layout);
			setBorder(new CompoundBorder(titleBorder, emptyBorder));
			con.a = 8; // NORTH-WEST
			con.s = "wh";
			add(new JLabel("Show ... "), con);
			con.x = 1;
			con.y = 1;
			add(layout.createVerticalStrut(1), con);
			con.y = 2;
			add(emptyServers, con);
			con.y = 3;
			add(fullServers, con);
			con.y = 4;
			add(layout.createVerticalStrut(10), con);
			con.y = 5;
			add(publicServers, con);
			con.y = 6;
			add(privateServers, con);
			con.y = 7;
			add(layout.createVerticalStrut(10), con);
			con.y = 8;
			add(secureServers, con);
			con.y = 9;
			add(unsecureServers, con);
			con.y = 10;
			add(layout.createVerticalStrut(10), con);
			con.y = 11;
			add(notPingedServers,con);
			
			// init settings
			if (hlsettings.getValueAsInt(SearchtoolSettings.FILTER_EMPTY) == SearchtoolSettings.ENABLED)
				emptyServers.setSelected(true);
			if (hlsettings.getValueAsInt(SearchtoolSettings.FILTER_FULL) == SearchtoolSettings.ENABLED)
				fullServers.setSelected(true);

			if (hlsettings.getValueAsInt(SearchtoolSettings.FILTER_PUBLIC_SERVERS) == SearchtoolSettings.ENABLED)
				publicServers.setSelected(true);
			if (hlsettings.getValueAsInt(SearchtoolSettings.FILTER_PRIVATE_SERVERS) == SearchtoolSettings.ENABLED)
				privateServers.setSelected(true);

			if (hlsettings.getValueAsInt(HlSettings.FILTER_SECURE_SERVERS) == SearchtoolSettings.ENABLED)
				secureServers.setSelected(true);
			if (hlsettings.getValueAsInt(HlSettings.FILTER_UNSECURE_SERVERS) == SearchtoolSettings.ENABLED)
				unsecureServers.setSelected(true);
				
			if (hlsettings.getValueAsInt(SearchtoolSettings.FILTER_NOTPINGED_SERVERS) == SearchtoolSettings.ENABLED)
				notPingedServers.setSelected(true);

			// add action listeners	
			publicServers.addItemListener(this);
			privateServers.addItemListener(this);
			emptyServers.addItemListener(this);
			fullServers.addItemListener(this);
			secureServers.addItemListener(this);
			unsecureServers.addItemListener(this);
			notPingedServers.addItemListener(this);
		}

		public void itemStateChanged(ItemEvent e) {					
			
			// update settings
			if (e.getSource().equals(emptyServers)) {
				if (e.getStateChange() == e.DESELECTED) {
					hlsettings.setValue(SearchtoolSettings.FILTER_EMPTY, SearchtoolSettings.DISABLED);
				} else {
					hlsettings.setValue(SearchtoolSettings.FILTER_EMPTY, SearchtoolSettings.ENABLED);
				}
			}
			if (e.getSource().equals(fullServers)) {
				if (e.getStateChange() == e.DESELECTED) {
					hlsettings.setValue(SearchtoolSettings.FILTER_FULL, SearchtoolSettings.DISABLED);
				} else {
					hlsettings.setValue(SearchtoolSettings.FILTER_FULL, SearchtoolSettings.ENABLED);
				}
			}
			if (e.getSource().equals(publicServers)) {
				if (e.getStateChange() == e.DESELECTED) {
					if (!privateServers.isSelected()) {
						publicServers.setSelected(true);
						return;
					}
					hlsettings.setValue(SearchtoolSettings.FILTER_PUBLIC_SERVERS, SearchtoolSettings.DISABLED);
				} else {
					hlsettings.setValue(SearchtoolSettings.FILTER_PUBLIC_SERVERS, SearchtoolSettings.ENABLED);
				}
			}
			if (e.getSource().equals(privateServers)) {
				if (e.getStateChange() == e.DESELECTED) {
					if (!publicServers.isSelected()) {
						privateServers.setSelected(true);
						return;
					}
					hlsettings.setValue(SearchtoolSettings.FILTER_PRIVATE_SERVERS, SearchtoolSettings.DISABLED);
				} else {
					hlsettings.setValue(SearchtoolSettings.FILTER_PRIVATE_SERVERS, SearchtoolSettings.ENABLED);
				}
			}
			if (e.getSource().equals(secureServers)) {
				if (e.getStateChange() == e.DESELECTED) {
					if (!unsecureServers.isSelected()) {
						secureServers.setSelected(true);
						return;
					}
					hlsettings.setValue(HlSettings.FILTER_SECURE_SERVERS, SearchtoolSettings.DISABLED);
				} else {
					hlsettings.setValue(HlSettings.FILTER_SECURE_SERVERS, SearchtoolSettings.ENABLED);
				}
			}
			if (e.getSource().equals(unsecureServers)) {
				if (e.getStateChange() == e.DESELECTED) {
					if (!secureServers.isSelected()) {
						unsecureServers.setSelected(true);
						return;
					}
					hlsettings.setValue(HlSettings.FILTER_UNSECURE_SERVERS, SearchtoolSettings.DISABLED);
				} else {
					hlsettings.setValue(HlSettings.FILTER_UNSECURE_SERVERS, SearchtoolSettings.ENABLED);
				}
			}
			if (e.getSource().equals(notPingedServers)){
				if( e.getStateChange() == e.DESELECTED){
					hlsettings.setValue(SearchtoolSettings.FILTER_NOTPINGED_SERVERS,SearchtoolSettings.DISABLED);	
				}	
				else{
					hlsettings.setValue(SearchtoolSettings.FILTER_NOTPINGED_SERVERS,SearchtoolSettings.ENABLED);	
				}		
			}
		}
	}
}