/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
 * HL_WON_Connection.java
 *
 * Created on December 16, 2001, 2:30 PM
 */

package searchtool.halflife.protocol;

import java.io.*;
import java.net.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.*;
import searchtool.wrappers.*;
import searchtool.halflife.wrappers.*;
import searchtool.interfaces.*;
import searchtool.protocol.*;
import searchtool.util.*;
// thirdparty
import gnu.trove.*;

/**
 *
 * @author  zoidberg
 * @version
 */
public class HL_WON_Connection extends ServerRequest {
	private DatagramSocket socket = null;
	private DatagramPacket snd_packet = null;
	private DatagramPacket get_packet = null;
	private InetAddress inet_address = null;
	private byte[] sndbyte;
	private byte[] getbyte = new byte[2048];
	private int[] position = null;
	private String command;
	private int servercount = 0;
	private boolean finished = false;
	

	/** Creates new HL_WON_Connection */

	public HL_WON_Connection(SearchtoolSettings SETTINGS , String MASTER_IP, int MASTER_PORT, String FILTER) {
		super(MASTER_IP,MASTER_PORT,FILTER);
		super.settings = SETTINGS;
	}
	
	public void cleanUp(){
		position = null;
		servercount = 0;
		finished = false;
	}
	
	public HL_WON_Connection(SearchtoolSettings SETTINGS){
		super(SETTINGS);
	}

	public boolean hasNext() {
		if (position == null)
			return true;
		if (position[0] == 0
			&& position[1] == 0
			&& position[2] == 0
			&& position[3] == 0)
			return false;
		else
			return true;
	}
	
	private String getFilterString() {
		String filter = "";
		if (!(settings.getValueAsInt(settings.FILTER_EMPTY) == (settings.ENABLED)))
			filter += "\\empty\\1";
		if (!(settings.getValueAsInt(settings.FILTER_FULL) == (settings.ENABLED)))
			filter += "\\full\\1";
		if (!settings.getValue(settings.FILTERED_MOD).equals("NONE") && settings.getValue(settings.FILTERED_MOD) != null) {
			filter += "\\gamedir\\";
			filter += settings.getValue(settings.FILTERED_MOD);
			if (!settings.getValue(settings.FILTERED_MAP).equals("") && !settings.getValue(settings.FILTERED_MAP).equals("NONE") && settings.getValue(settings.FILTERED_MAP) != null) {
				filter += "\\map\\";
				filter += settings.getValue(settings.FILTERED_MAP);
			}
		}
		return filter;
	}
	
	public ServerElement[] getNext( boolean masterlevelFilter ){
		super.clearTempServerlist();
		try{
			if ( masterlevelFilter )
				receive ( getFilterString() );
			else
				receive ( "" );
		}catch(Exception e){
			//e.printStackTrace();
			return new ServerElement[0];
		}
		return  super.getStoredServerElements();
	}

	private void receive(String filter)
		throws Exception /* a bunch of 'em*/ {

		if(position == null)
			position = new int[]{0,0,0,0};

		inet_address = InetAddress.getByName(masterIp);
		socket = new DatagramSocket();

		ByteArrayOutputStream byteout = new ByteArrayOutputStream();
		DataOutputStream out = new DataOutputStream(byteout);

		out.writeByte((byte) '1'); // do not use the 'c' character here
		// caused only trouble for me. (event though it is mentioned in the
		// half-life SDK to use the 'c'.  '1' isn't mentioned there)
		for (int i = 0; i < 4; i++)
			out.writeByte(position[i]);
		out.writeBytes(filter);
		out.close();
		byteout.close();
		sndbyte = byteout.toByteArray();

		snd_packet =
			new DatagramPacket(
				sndbyte,
				sndbyte.length,
				inet_address,
				masterPort);
				
		get_packet = new DatagramPacket(getbyte, getbyte.length);

		socket.send(snd_packet);
		socket.setSoTimeout(2000);
		socket.receive(get_packet);
		socket.close();

		ByteArrayInputStream bytein = new ByteArrayInputStream(getbyte);
		DataInputStream in = new DataInputStream(bytein);

		in.skip(6);
		for (int i = 0; i < 4; i++){
			position[i] = in.readUnsignedByte();
		}

		int[] ipArray = new int[5];

		while (true) {
			ipArray[0] = in.readUnsignedByte();
			ipArray[1] = in.readUnsignedByte();
			ipArray[2] = in.readUnsignedByte();
			ipArray[3] = in.readUnsignedByte();
			ipArray[4] = in.readUnsignedShort();
			
			if(ipArray[0] == 0 && ipArray[1] == 0 && ipArray[2] == 0 && ipArray[3] == 0 && ipArray[4] == 0)
				break;
				
			IpWrapper ipWrapper = new IpWrapper(ipArray[0]  , ipArray[1] , ipArray[2] , ipArray[3] ,  ipArray[4]);
			
			ServerElement element =  new HlServerElement( ipWrapper ) ;
			super.storeServerAddress( element );
			servercount++;
		}
		in.close();
		bytein.close();
	}
}
