/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
 * HlServerElement.java
 *
 * Created on January 6, 2002, 2:44 PM
 */

package searchtool.halflife.wrappers;

import java.util.*;
import javax.swing.*;
import searchtool.*;
import searchtool.wrappers.*;
import searchtool.interfaces.*;
import java.io.*;
// third party
import com.jclark.xml.parse.*;
import com.jclark.xml.output.*;
import com.jclark.xml.parse.base.*;
import com.jclark.xml.sax.*;

/**
 *
 * @author  zoidberg
 * @version
 */
public class HlServerElement extends ServerElement {

	public static Byte SECURE, UNSECURE, NOT_NEEDED, UNKNOWN;
	static {
		UNSECURE = new Byte((byte) 0);
		SECURE = new Byte((byte) 1);
		NOT_NEEDED = new Byte((byte) 3);
		UNKNOWN = NOT_NEEDED;
	}
	private static String defaultMod = "valve";
	private static String defaultGamedir = defaultMod;
	private static String defaultValue = "unknown";

	private String mod = defaultMod,
		protocolVersion = "-1",
		serverType = defaultValue,
		operatingSystem = defaultValue,
		proxyTarget = defaultValue,
		description = defaultValue,
		proxyAddress = defaultValue;
	private boolean lan = false, runningMod = false, proxy = false, cldll = false, svonly = false;
	private Byte secure = UNKNOWN;

	/** Creates new HlServerElement */
	public HlServerElement() {
		super();
		super.setMod(defaultMod); // override superclass value
	}

	public HlServerElement(IpWrapper ipWrapper) {
		super(ipWrapper);
		super.setMod(defaultMod);
	}

	public HlServerElement(IpWrapper ipWrapper, String SERVERNAME, int PING, String MAP, int MAXPLAYERS, int CURRENTPLAYERS, Byte PASSWORD, Byte SECURITY, String MOD) {
		super(ipWrapper, SERVERNAME, PING, MAP, MAXPLAYERS, CURRENTPLAYERS, PASSWORD);
		secure = SECURITY;
		super.setMod(MOD);
	}

	public synchronized ServerElement getNewServerElement() {
		return new HlServerElement();
	}

	public synchronized ServerElement getNewServerElement(IpWrapper ipWrapper) {
		return new HlServerElement(ipWrapper);
	}

	/*
	 *  get
	 */
	public String getProtocolVersion() {
		return protocolVersion;
	}
	public String getServerType() {
		return serverType;
	}
	public String getOperatingSystem() {
		return operatingSystem;
	}
	public String getProxyTarget() {
		return proxyTarget;
	}
	public boolean getLan() {
		return lan;
	}
	public String getDescription() {
		return description;
	}
	public String getModVersion() {
		return mod;
	}
	public boolean getRunningMod() {
		return runningMod;
	}
	public boolean hasProxy() {
		return proxy;
	}
	public String getProxyAddress() {
		return proxyAddress;
	}
	public boolean hasCldll() {
		return cldll;
	}
	public boolean isSvonly() {
		return svonly;
	}
	public Byte getSecure() {
		return secure;
	}

	/*
	 *  set
	 */
	public void setProtocolVersion(String VALUE) {
		protocolVersion = VALUE;
	}
	public void setServerType(String VALUE) {
		serverType = VALUE;
	}
	public void setOperatingSystem(String VALUE) {
		operatingSystem = VALUE;
	}
	public void setProxyTarget(String VALUE) {
		proxyTarget = VALUE;
	}
	public void setLan(boolean VALUE) {
		lan = VALUE;
	}
	public void setDescription(String VALUE) {
		description = VALUE;
	}
	public void setModVersion(String VALUE) {
		mod = VALUE;
	}
	public void setRunningMod(boolean VALUE) {
		runningMod = VALUE;
	}
	public void setProxy(boolean VALUE) {
		proxy = VALUE;
	}
	public void setProxyAddress(String VALUE) {
		proxyAddress = VALUE;
	}
	public void setCldll(boolean VALUE) {
		cldll = VALUE;
	}
	public void setSvonly(boolean VALUE) {
		svonly = VALUE;
	}
	public void setSecure(byte VALUE) {
		if (VALUE == UNSECURE.byteValue())
			secure = UNSECURE;
		else if (VALUE == UNKNOWN.byteValue())
			secure = UNKNOWN;
		else
			secure = SECURE;
	}
	public void setSecure(Byte VALUE) {
		secure = VALUE;
	}

	public static class HlDataElementParserCallback extends ServerElement.ListDataElementParserCallback {

		public HlDataElementParserCallback() {
			super();
		}

		/**
		 *  called at start of element
		 */
		public void startElement(StartElementEvent e) {
			if (e.getName().equals(Searchtool.Constants.serverTag)) {
				ServerElement tmp = parseElement(e);
				storeElement(tmp);
			} else if (e.getName().equals(Searchtool.Constants.serverCountTag)) {
				if (isProgressBarRegistered()) {
					try {
						initializeStatusBar(Integer.parseInt(e.getAttributeValue(Searchtool.Constants.numberAtt)));
					} catch (Exception f) {
						f.printStackTrace();
					}
				}
			}
		}
		protected ServerElement parseElement(StartElementEvent e) {
			try {
				HlServerElement listDataElement = new HlServerElement();
				String value = e.getAttributeValue("IP1");
				int ip1 = 0, ip2 = 0, ip3 = 0, ip4 = 0, port = 0;

				value = e.getAttributeValue("IP1");
				if (value != null) {
					ip1 = Integer.parseInt(value);
					value = null;
				}
				value = e.getAttributeValue("IP2");
				if (value != null) {
					ip2 = Integer.parseInt(value);
					value = null;
				}
				value = e.getAttributeValue("IP3");
				if (value != null) {
					ip3 = Integer.parseInt(value);
					value = null;
				}
				value = e.getAttributeValue("IP4");
				if (value != null) {
					ip4 = Integer.parseInt(value);
					value = null;
				}
				value = e.getAttributeValue("PRT");
				if (value != null) {
					port = Integer.parseInt(value);
					value = null;
				}
				listDataElement.setIpWrapper(new IpWrapper(ip1, ip2, ip3, ip4, port));
				value = e.getAttributeValue("PNG");
				if (value != null) {
					listDataElement.setPing(Integer.parseInt(value));
					value = null;
				}
				value = e.getAttributeValue("MAP");
				if (value != null) {
					listDataElement.setMap(value);
					value = null;
				}
				value = e.getAttributeValue("NME");
				if (value != null) {
					listDataElement.setServername(value);
					value = null;
				}
				value = e.getAttributeValue("CRP");
				if (value != null) {
					listDataElement.setCurrentplayers(Integer.parseInt(value));
					value = null;
				}
				value = e.getAttributeValue("MXP");
				if (value != null) {
					listDataElement.setMaxplayers(Integer.parseInt(value));
					value = null;
				}
				value = e.getAttributeValue("PWD");
				if (value != null) {
					int tmp = Integer.parseInt(value);
					if (tmp == ServerElement.NO_PASSWD.byteValue())
						listDataElement.setPassword(ServerElement.NO_PASSWD);
					else if (tmp == ServerElement.PASSWD_REQUIRED.byteValue())
						listDataElement.setPassword(ServerElement.PASSWD_REQUIRED);
					value = null;
				}
				value = e.getAttributeValue("DTE");
				if (value != null) {
					listDataElement.setDate(new Date(Long.parseLong(value)));
					value = null;
				}
				value = e.getAttributeValue("HST");
				if (value != null) {
					listDataElement.setLastConnectedlist(Boolean.valueOf(value).booleanValue());
					value = null;
				}
				value = e.getAttributeValue("BLK");
				if (value != null) {
					listDataElement.setBlacklist(Boolean.valueOf(value).booleanValue());
					value = null;
				}
				value = e.getAttributeValue("FVR");
				if (value != null) {
					listDataElement.setFavouritelist(Boolean.valueOf(value).booleanValue());
					value = null;
				}
				value = e.getAttributeValue("MOD");
				if (value != null) {
					listDataElement.setMod(value);
				}

				int secure = Integer.parseInt(e.getAttributeValue("SEC"));
				if (secure == HlServerElement.SECURE.intValue())
					listDataElement.setSecure(HlServerElement.SECURE);
				else
					listDataElement.setSecure(HlServerElement.UNSECURE);

				increaseStatusBar();
				return listDataElement;
			} catch (Exception f) {
				increaseStatusBar();
				System.out.println("could not parse element " + f.getMessage());
				return null;
			}
		}
	}
	public static class HlDataElementXMLWriter extends ServerElement.ListDataElementXMLWriter {
		public HlDataElementXMLWriter(File serverfile) {
			super(serverfile);
		}
		public void writeElement(ServerElement saveMe) {
			try {
				super.writeElement(saveMe);
				xmlWriter.attribute("SEC", ((HlServerElement) saveMe).getSecure().intValue() + "");
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
}
