/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
 * HLSettings.java
 *
 * Created on January 16, 2002, 7:23 PM
 */
package searchtool.halflife.wrappers;

import searchtool.*;
import searchtool.components.SearchtoolDialogs;
import searchtool.interfaces.*;
import searchtool.wrappers.*;
import searchtool.util.*;
import java.io.*;
import java.util.*;
/**
 * the settings object for half-life support.
 * settings will be loaded from file as soon as the constructor is
 * called.
 *
 * @author  zoidberg
 * @version
 */
public class HlSettings extends SearchtoolSettings {

	public static final String FILTER_SECURE_SERVERS = "FILTER_SECURE_SERVERS", FILTER_UNSECURE_SERVERS = "FILTER_UNSECURE_SERVERS";

	private String[] MOD_DIRS;
	private static String[] directoriesThatAreNoMods = new String[16];
	static {
		// NORMAL VERSION
		directoriesThatAreNoMods[0] = "auth";
		directoriesThatAreNoMods[1] = "dmc";
		directoriesThatAreNoMods[2] = "gldrv";
		directoriesThatAreNoMods[3] = "maps";
		directoriesThatAreNoMods[4] = "logos";
		directoriesThatAreNoMods[5] = "sound";
		directoriesThatAreNoMods[6] = "uplink";
		directoriesThatAreNoMods[7] = "gfx";
		// (additional) RETAIL VERSION
		directoriesThatAreNoMods[8] = "classes";
		directoriesThatAreNoMods[9] = "cl_dlls";
		directoriesThatAreNoMods[10] = "events";
		directoriesThatAreNoMods[11] = "manual";
		directoriesThatAreNoMods[12] = "models";
		directoriesThatAreNoMods[13] = "PODBot";
		directoriesThatAreNoMods[14] = "sounds";
		directoriesThatAreNoMods[15] = "sprites";
	}

	/** Creates new HLSettings */
	public HlSettings(File settingsfile) {
		super(settingsfile);
		this.initValues();
	}

	/**
	 *  init variables with default values
	 */
	protected void initValues() {
		super.initValues();
		setValue(MASTERSERVER_IP, "half-life.west.won.net");
		setValue(MASTERSERVER_PORT, "27010");
		setValue(CONNECT_COMMAND, "(sample) wine /path/to/hl.exe -- hl.exe -console -toconsole -nocdaudio");
		setValue(SEARCHSTRING, "MpdWa-Wolpers");
		setValue(GAME_DIR, "/path/to/Half-Life");
		setValue(MASTERLEVEL_FILTER, "0");
		setValue(FILTERED_MOD, "NONE");
		setValue(FILTERED_MAP, "NONE");
		setValue(SHUTDOWN_TIMEOUT, "0");
		setValue(SELECTED_COLUMN, "2");
		setValue(FILTER_SECURE_SERVERS, ENABLED);
		setValue(FILTER_UNSECURE_SERVERS, ENABLED);
	}

	public String[] getInstalledMods() {
		File hldir = new File(getValue(GAME_DIR));
		Vector tmp = new Vector();
		if (hldir.exists() && hldir.isDirectory()) {
			File[] content = hldir.listFiles();
			for (int i = 0; i < content.length; i++) {
				// check if mod or something else
				if (content[i].isDirectory()) {
					String dirname = content[i].getName();
					for (int j = 0; j < directoriesThatAreNoMods.length; j++) {
						if (dirname.equals(directoriesThatAreNoMods[j]))
							break;
						// possible mod
						else if (j == directoriesThatAreNoMods.length - 1)
							tmp.add(dirname);
					}
				}
			}
			String[] returnme = new String[tmp.size()];
			for (int i = 0; i < returnme.length; i++)
				returnme[i] = (String) tmp.get(i);
			return returnme;
		} else {
			return null;
		}
	}

	public String[] getInstalledMaps() {
		Vector tmp = new Vector();
		// figure out path to map directory
		String MAPDIR = "";
		MAPDIR += getValue(GAME_DIR) + File.separator;
		MAPDIR += getValue(FILTERED_MOD) + File.separator + "maps";
		File mapdir = new File(MAPDIR);
		if (mapdir.exists() && mapdir.isDirectory()) {
			File[] content = mapdir.listFiles();
			for (int i = 0; i < content.length; i++) {
				String filename = content[i].getName();
				if (filename.endsWith("bsp")) {
					int index = filename.indexOf(".bsp");
					if ( index > 0 ){
						tmp.add(filename.substring(0, index));
					}
				}
			}
			return (String[]) tmp.toArray(new String[0]);
		} else
			return null;
	}

	public String getConnectPostfix(ServerElement connectToMe) {
		// maybe get password
		String connectPostfix = "";
		if (connectToMe.getPassword().intValue() == ServerElement.PASSWD_REQUIRED.intValue()) {
			String passwd  = new SearchtoolDialogs.PasswordDialog().showPasswordDialog();
			if ( passwd != null )
				connectPostfix += (" +password \"" + passwd + "\" ");
		}
		connectPostfix += (" -game " + connectToMe.getMod());
		// in HL gameport == queryport ...
		connectPostfix += (" +connect " + connectToMe.getIpAndPort());
		return connectPostfix;
	}
}