/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */

package searchtool.interfaces;

import searchtool.wrappers.*;
import searchtool.*;
import searchtool.actions.*;
import java.util.*;
// thirdparty
import gnu.trove.*;

/**
 * @author thorsten
 *
 */
public interface ElementListManager {
	
	// set the complete element list
	public void setCompleteList(Element[] vector);
	
	// get the complete element list
	public Element[] getCompleteList();
	
	// get the complete list as a hashmap.
	public TLongObjectHashMap getCompleteListAsMap();
	
	// get the filtered list
	public Element[] getFilteredList();
	
	// add a new element
	public void addElement(Element element);
	
	// check whether the list contains an object describing the same element
	public boolean isElementInList(Element element);
	
	// remove the given element or any object that describes the same element.
	public Element removeElement(Element element);
	
	// sort the element list
	public void sortElementList();
	
	// filter the element list
	public void filterElementList();
	
	// return the settings object
	public SearchtoolSettings getSearchtoolSettings();
	
	// return the buttontracker
	public ButtonTracker getButtonTracker();
	
	// mark an element that makes it not being filtered
	public void doNotFilterElement(Element doNotFilterMe);

}
