/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
    
/*
 * SearchtoolProtocol.java
 *
 * Created on December 16, 2001, 2:20 PM
 */

package searchtool.protocol;

import searchtool.wrappers.*;

/**
 *
 * @author  zoidberg
 * @version 
 */
public abstract class SearchtoolProtocol {

	protected ServerElement element;
	protected boolean timedout;
	
	public SearchtoolProtocol(ServerElement element){
		this.element = element;
	}
	
	public SearchtoolProtocol(){
	}
	
	public ServerElement getServerElement(){
		return element;
	}
	
	public void setServerElement(ServerElement element){
		this.element = element;
		timedout = false;
	}
	
    protected abstract void fill() throws Exception;
    
    public boolean hasTimedout(){
    	return timedout;
    }
    
    public void setTimedout(boolean timedout){
    	this.timedout = timedout;
    }
    
    /**
	 *  this method will try to fill the details object given.  if this process throws an exception for 
	 *  whatever reason the object will be marked as 'timedout'  and 'false' will be returned.
	 *  @return true if fill successfull, false otherwise
	 *  @param the SearchtoolProtocol that is supposed to be filled
	 */
	public boolean update(){
		try{
			this.fill();
			// no timeout, otherwise exception would have been thrown !
			this.setTimedout(false);
			// success !
			return true;
		}catch(Exception e){
			//e.printStackTrace();
			//System.out.println(e.getClass());
			this.setTimedout(true);
			// failure !
			return false;
		}
	}
}

