/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package searchtool.protocol;

import searchtool.wrappers.*;
import searchtool.util.*;
import gnu.trove.*;

/**
 * @author thorsten
 *
 */
public abstract class ServerRequest {
	
	protected String masterIp = "127.0.0.1";
	protected int masterPort = 1;
	protected SearchtoolSettings settings = null;
	protected TLongObjectHashMap oldServersMap = null;
	
	private TLongObjectHashMap newServersMap = new TLongObjectHashMap();
	
	public ServerRequest(){
	}
	
	public ServerRequest(SearchtoolSettings settings){
		this.settings = settings;
	}
	
	public ServerRequest(String masterIp, int masterPort, String filterString){
	}
	
	public void setIp(String masterIp){
		this.masterIp = masterIp;
	}
	
	public void setPort(int masterPort){
		this.masterPort = masterPort;
	}
	
	public abstract void cleanUp();
	
	/**
	 *  register an old list of elements.  if a downloaded server address is already available
	 *  in the registered list the existing reference will be used.  this way old data can be
	 *  keept.
	 */
	public void registerOldElementList(TLongObjectHashMap map){
		oldServersMap = map;
	}
	
	/**
	 *  check if there is a next package of servers to get
	 */
	public abstract boolean hasNext();
	
	/**
	 *  get the next package of servers
	 *  @param sets the filter enabled or not
	 */
	public abstract ServerElement[] getNext( boolean masterlevelFilter) throws Exception;

	/**
	 *  store a server temporarily.  if the server already does exist in a registered
	 *  element list the old dataset will be used.
	 *  @param the element to be stored
	 */
	protected void storeServerAddress(ServerElement element) throws Exception{
		if ( oldServersMap != null ) {
			if ( oldServersMap.contains( element.getId() ) ){
				newServersMap.put( element.getId(), oldServersMap.get( element.getId() ) );	
			}	
			else{
				newServersMap.put( element.getId(), element );	
			}
		}
		else{
			newServersMap.put( element.getId(), element );	
		}
	}
	
	/**
	 *  return the stored elements by <code>storeServerAddress()</code>
	 */
	protected ServerElement[] getStoredServerElements(){
		long[] keys = newServersMap.keys();
		ServerElement[] elements = new ServerElement[keys.length];
		for ( int i = 0; i < keys.length; i++){
			elements[i] = (ServerElement)newServersMap.get(keys[i]);
		}	
		return elements;
	}
	
	/**
	 *  clear the temporarily stored serverlist
	 */
	protected void clearTempServerlist(){
		newServersMap.clear();
	}

}
