/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package searchtool.tribes2.protocol;

import java.io.*;
import java.net.*;
import java.util.*;

import searchtool.components.*;
import searchtool.wrappers.*;
import searchtool.tribes2.wrappers.*;
import searchtool.protocol.*;

// thirdparty
import gnu.trove.*;

/**
 * @author thorsten
 *
 */
public class Tribes2MasterQuery extends ServerRequest {

	private static int idCounter = 0;
	private int index = 255;
	private boolean[] packageResponseMap = null;

	public Tribes2MasterQuery(Tribes2Settings settings, String ip, int port, String filter, String game) {
		super(ip, port, filter);
		super.settings = settings;
	}

	public Tribes2MasterQuery(Tribes2Settings settings, String game) {
		super(settings);
	}

	public boolean hasNext() {
		if (packageResponseMap == null)
			return true;
		for (int i = 0; i < packageResponseMap.length; i++) {
			if (packageResponseMap[i] == false)
				return true;
		}
		return false;
	}
	public ServerElement[] getNext(boolean masterlevelFilter) throws Exception {

		DatagramSocket socket = null;
		DatagramPacket snd_packet = null;
		DatagramPacket get_packet = null;
		InetAddress inet_address = null;

		ByteArrayOutputStream out = new ByteArrayOutputStream();
		DataOutputStream dataOut = new DataOutputStream(out);

		dataOut.writeByte(6); //type
		dataOut.writeByte(128); // tell master filter information follows
		dataOut.writeInt(idCounter);
		if (packageResponseMap != null) {
			for (int i = 0; i < packageResponseMap.length; i++) {
				if (packageResponseMap[i] == false) {
					index = i;
				}
			}
		}
		dataOut.writeByte(index); // packet index
		//System.out.println(index);
		/*
		if (masterlevelFilter) {
			String filteredMod = settings.getValue(settings.FILTERED_MOD);
			if (filteredMod != null && !filteredMod.equals("NONE")) {
				dataOut.writeByte(filteredMod.length()); // length of string
				dataOut.writeBytes(filteredMod); // gametype

				String filteredMap = settings.getValue(settings.FILTERED_MAP);
				if (filteredMap != null && !filteredMap.equals("NONE")) {
					dataOut.writeByte(filteredMap.length()); // length of string
					dataOut.writeBytes(filteredMap); // gametype
				} else {
					dataOut.writeByte(3); // length of string
					dataOut.writeBytes("any"); // missiontype
				}
			} else {
				dataOut.writeByte(3); // length of string
				dataOut.writeBytes("any"); // gametype
				dataOut.writeByte(3); // length of string
				dataOut.writeBytes("any"); // missiontype
			}
		} else {
			dataOut.writeByte(3); // length of string
			dataOut.writeBytes("any"); // gametype
			dataOut.writeByte(3); // length of string
			dataOut.writeBytes("any"); // missiontype
		}
		*/

		dataOut.writeByte(0); // minimum players
		dataOut.writeByte(255); //maximum players
		dataOut.writeInt(settings.getValueAsInt(((Tribes2Settings) settings).REGION_MASK)); // region mask
		//dataOut.writeByte(63);
		//dataOut.writeByte(0);
		//dataOut.writeByte(0);
		//dataOut.writeByte(0);
		dataOut.writeInt(0); // build version 
		dataOut.writeByte(0); // status flag
		dataOut.writeByte(255); // max bots
		dataOut.writeShort(0); // min cpu speed
		dataOut.writeByte(0); // buddy count

		out.close();
		dataOut.close();

		byte[] sndbyte = out.toByteArray();

		try {
			byte[] getbyte = new byte[2000];
			inet_address = InetAddress.getByName(super.masterIp);
			socket = new DatagramSocket();
			snd_packet = new DatagramPacket(sndbyte, sndbyte.length, inet_address, super.masterPort);
			get_packet = new DatagramPacket(getbyte, getbyte.length);
			socket.send(snd_packet);
			socket.setSoTimeout(2000);
			socket.receive(get_packet);
			ByteArrayInputStream in = new ByteArrayInputStream(getbyte);
			DataInputStream dataIn = new DataInputStream(in);
			dataIn.skipBytes(6);
			index = dataIn.readUnsignedByte();
			//System.out.println(index);
			int maxIndex = dataIn.readUnsignedByte();
			if (packageResponseMap == null) {
				packageResponseMap = new boolean[maxIndex];
				// init
				for (int i = 0; i < packageResponseMap.length; i++) {
					packageResponseMap[i] = false;
				}
			}
			packageResponseMap[index] = true;
			int serverCount = dataIn.readUnsignedShort(); // server count (very unreliable)
			super.clearTempServerlist();
			int ip1, ip2, ip3, ip4, port,tmpRead;
			tmpRead = dataIn.readUnsignedByte();;
			while (dataIn.available() > 0) {
				if ( tmpRead == 0 )
					break;
				ip1 = tmpRead;
				ip2 = dataIn.readUnsignedByte();
				ip3 = dataIn.readUnsignedByte();
				ip4 = dataIn.readUnsignedByte();
				port = dataIn.readUnsignedShort();
				super.storeServerAddress(new ServerElement(new IpWrapper(ip1, ip2, ip3, ip4, port)));
				tmpRead = dataIn.readUnsignedByte();
			}
			in.close();
			dataIn.close();
			return super.getStoredServerElements();
		} catch (Exception e) {
			e.printStackTrace();
			socket.close();
			//e.printStackTrace();
			throw e;
		}
	}

	public void cleanUp() {
		super.clearTempServerlist();
		packageResponseMap = null;
		index = 255;
		idCounter++;
	}
}
