/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package searchtool.unreal_tournament;

import searchtool.*;
import searchtool.unreal_tournament.protocol.*;
import searchtool.unreal_tournament.wrappers.*;
import searchtool.components.*;
import searchtool.gamespy.*;
import searchtool.actions.*;
import searchtool.unreal_tournament.protocol.*;

public class UnrealTournamentGameController extends GameController{
	
	public UnrealTournamentGameController(String gamePrefix){
		super(gamePrefix);
		// override settings
		super.settings = new UnrealTournamentSettings(super.configFile);
		super.settings.load();
		// override component due to different settings object
		super.editSettingsPanel = new SearchtoolSettingsPanel(settings,"searchtool/files/" + gamePrefix + "_settings.html" );
		// override component due to different protocol class
		super.searchPlayersPanel = new SearchPlayersPanel( this, new GamespyPlayers(), new UnrealTournamentDetails() );
	}
	
	protected void initMainButtonBar(){
		super.initMainButtonBar();
		super.mainlistButtonbar.getButton(mainlistButtonbar.MASTERSERVER_FILTER).setEnabled(false);
	}
	
	protected void initFavouriteButtonBar(){
		super.initFavouriteButtonBar();
		super.favouritelistButtonbar.getButton(favouritelistButtonbar.MASTERSERVER_FILTER).setEnabled(false);
	}
	
	protected void initOutsiderButtonBar(){
		super.initOutsiderButtonBar();
		super.outsiderlistButtonbar.getButton(outsiderlistButtonbar.MASTERSERVER_FILTER).setEnabled(false);
	}
	
	protected void initSearchButtonBar(){
		super.initSearchButtonBar();
		super.searchplayerlistButtonbar.getButton(searchplayerlistButtonbar.MASTERSERVER_FILTER).setEnabled(false);
	}
	
	protected void initLastconnectedButtonBar(){
		super.initLastconnectedButtonBar();
		super.lastconnectedlistButtonbar.getButton(lastconnectedlistButtonbar.MASTERSERVER_FILTER).setEnabled(false);
	}
	
	protected void initActionListeners(){
		super.initActionListeners();		
		super.getDetails = null;
		super.getDetails = new GetDetailsActionListener(new GamespyPing(), new UnrealTournamentRules(), new UnrealTournamentDetails(), new GamespyPlayers(), rulesConfigFile,settings);
		super.pingServerList = null;
		super.pingServerList = new PingServersActionListener(this, settings, new GamespyPing(), new UnrealTournamentDetails(), new UnrealTournamentRules());
	}
	
}