/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package searchtool.unreal_tournament.wrappers;

import searchtool.*;
import searchtool.components.SearchtoolDialogs;
import searchtool.interfaces.*;
import searchtool.wrappers.*;
import searchtool.util.*;
import java.io.*;
import java.util.*;
// thirdparty
import gnu.trove.*;

/**
 * @author thorsten
 *
 */
public class UnrealTournamentSettings extends SearchtoolSettings {

	//private String[] utMaps = new String[] { "CTF", "DM", "AS", "DOM" };

	public UnrealTournamentSettings(File settingsfile) {
		super(settingsfile);
		initValues();
	}

	/**
	 *  init variables with default values
	 */
	protected void initValues() {
		super.initValues();
		setValue(MASTERSERVER_IP, "unreal.epicgames.com");
		setValue(MASTERSERVER_PORT, "28900");
		setValue(CONNECT_COMMAND, "(sample)  /path/to/unreal.exe <optional parameters here>");
		setValue(SEARCHSTRING, "MpdWa-Wolpers");
		setValue(GAME_DIR, "/path/to/unreal");
		setValue(MASTERLEVEL_FILTER, "0");
		setValue(FILTERED_MOD, "NONE");
		setValue(FILTERED_MAP, "NONE");
		setValue(SHUTDOWN_TIMEOUT, "0");
		setValue(SELECTED_COLUMN, "2");
	}

	public String[] getInstalledMods() {
		String MAPDIR = "";
		MAPDIR += getValue(GAME_DIR) + File.separator + "Maps";
		File mapdir = new File(MAPDIR);
		THashSet set = new THashSet();
		if (mapdir.exists() && mapdir.isDirectory()) {
			File[] files = mapdir.listFiles();
			for ( int i = 0; i < files.length; i++ ){
				if ( files[i].getName().endsWith("unr") ){
					int index = files[i].getName().indexOf("-");
					if ( index > 0 ){
						set.add( files[i].getName().substring( 0 , index ) );
					}
				}
			}
			/*
			for ( int i = 0; i < utMaps.length; i++ ){
				set.remove( utMaps[i] );
			}
			set.add("UT");
			*/
			Object[] mods = set.toArray();
			String[] returnme = new String[ mods.length ];
			for ( int i = 0; i < mods.length; i++ ){
				returnme[i] = (String) mods[i];	
			}
			return returnme;
		} else {
			return null;
		}
	}

	public String[] getInstalledMaps() {
		Vector tmp = new Vector();
		// figure out path to map directory
		String MAPDIR = "";
		MAPDIR += getValue(GAME_DIR) + File.separator + "Maps";
		File mapdir = new File(MAPDIR);
		String filteredMod = this.getValue(this.FILTERED_MOD);
		//String[] installedMods = this.getInstalledMods();
		if (mapdir.exists() && mapdir.isDirectory()) {
			File[] content = mapdir.listFiles();
			for (int i = 0; i < content.length; i++) {
				String filename = content[i].getName();
				if (filename.endsWith("unr")) {
					/*
					// now we have mapfiles, now check which mod we are looking for
					// special unreal case
					if (filteredMod.equals("UT")) {
						for (int j = 0; j < utMaps.length; j++) {
							if (filename.startsWith(utMaps[j])) {
								tmp.add(filename.substring(0, filename.indexOf(".unr")));
								break;
							}
						}
					}
					*/
					// other mod
					if (filename.startsWith(filteredMod)) {
						int index =  filename.indexOf(".unr");
						if ( index > 0 ){
							tmp.add(filename.substring(0, index));
						}
					}
				}
			}
			return (String[]) tmp.toArray(new String[0]);
		} else
			return null;
	}

	public String getConnectPostfix(ServerElement connectToMe) {
		// maybe get password
		String connectPostfix = "";
		//if( connectToMe.getPassword().intValue() == ServerElement.PASSWD_REQUIRED.intValue() ){
		//		connectPostfix +=( " +password \""+ new SearchtoolDialogs.PasswordDialog().showPasswordDialog() + "\" " );
		//}
		connectPostfix += (/*" unreal://" +*/
		connectToMe.getIp() + ":" + connectToMe.getGamePort());
		return connectPostfix;
	}
}
