/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
 * HlEditSettingsPanel.java
 *
 * Created on January 16, 2002, 7:21 PM
 */
package searchtool.ut2003.components;

import searchtool.components.*;
import searchtool.components.SearchtoolButtonBar;
import searchtool.util.IconLib;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import searchtool.*;
import searchtool.wrappers.*;
import searchtool.ut2003.wrappers.*;
import searchtool.interfaces.*;
import java.io.*;
import java.util.*;
import java.net.*;
// thirdparty
import ise.java.awt.*;

/**
 * this panel contains all the components that build up
 * the 'edit settings' dialog.
 *
 *
 * @author  zoidberg
 * @version
 */
public class Ut2003EditSettingsPanel extends JPanel {

	private String TABNAME = GameController.EDITSETTINGS_TABNAME;

	public Ut2003EditSettingsPanel(SearchtoolSettings settings, String HTML_HELPFILE_PATH) {

		LambdaLayout layout = new LambdaLayout();
		LambdaLayout.Constraints con = new LambdaLayout.Constraints();

		con.a = 8; // NORTH-WEST
		con.s = "wh";
		con.x = 0;
		setLayout(layout);

		// upper left
		con.w = 2;
		add(new SearchtoolSettingsPanel.FilterPanel(settings), con);

		// upper center
		con.x = 2;
		con.w = 6;
		add(new SearchtoolSettingsPanel.StartupPanel(settings), con);

		// upper right
		con.x = 8;
		con.w = 2;
		add(new Ut2003MiscPanel(settings), con);

		// lower left
		con.x = 0;
		con.w = 3;
		con.y = 1;
		add(new SearchtoolSettingsPanel.GlobalPanel(), con);

		// lower center
		con.x = 3;
		con.w = 7;
		add(new SearchtoolSettingsPanel.HelpBrowser(HTML_HELPFILE_PATH), con);

	}

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	////////	MISC SETTINGS
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	private static class Ut2003MiscPanel extends JPanel implements ActionListener {
		private SearchtoolSettings settings = null;
		// GUI STUFF
		private TitledBorder titleBorder = BorderFactory.createTitledBorder("Misc settings");
		private Border emptyBorder = BorderFactory.createEmptyBorder(5, 5, 5, 5);
		private LambdaLayout layout = new LambdaLayout();
		private LambdaLayout.Constraints con = new LambdaLayout.Constraints();
		private JTextField masterPort = new JTextField();
		private JTextField masterIp = new JTextField();
		private JButton applyButton = new JButton("apply");

		public Ut2003MiscPanel(SearchtoolSettings settings) {
			this.settings = settings;
			// GUI
			setPreferredSize(new Dimension(200, 150));

			masterPort.setPreferredSize(new Dimension(60, 20));
			masterIp.setPreferredSize(new Dimension(60, 20));

			setLayout(layout);
			setBorder(new CompoundBorder(titleBorder, emptyBorder));
			con.s = "wh";
			con.a = 8; // NORTH-WEST
			applyButton.addActionListener(this);

			// init
			masterPort.setText(settings.getValue(Ut2003Settings.MASTERSERVER_HTML_PAGE));
			masterIp.setText(settings.getValue(Ut2003Settings.MASTERSERVER_IP));

			con.w = 6;
			add(new JLabel("Masterserver Address: "), con);
			con.y = 1;
			add(masterIp, con);
			con.y = 2;
			add(layout.createVerticalStrut(10), con);
			con.y = 3;
			add(new JLabel("Server File: "), con);
			con.y = 4;
			add(masterPort, con);
			con.y = 5;
			add(layout.createVerticalStrut(10), con);
			con.y = 6;
			con.x = 4;
			con.w = 2;
			add(applyButton, con);
		}

		public void actionPerformed(ActionEvent e) {
			try {
				settings.setValue(SearchtoolSettings.MASTERSERVER_IP, masterIp.getText().trim());
				settings.setValue(Ut2003Settings.MASTERSERVER_HTML_PAGE, masterPort.getText().trim());
			} catch (Exception f) {
				JOptionPane optionPane = new JOptionPane("No valid address", JOptionPane.ERROR_MESSAGE);
				JDialog dialog = optionPane.createDialog(Searchtool.getGui(), "Sorry");
				dialog.show();
			}
		}
	}
}