/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

package searchtool.util;

import java.io.*;
import java.util.*;
// thirdparty
import com.jclark.xml.parse.*;
import com.jclark.xml.output.*;
import com.jclark.xml.parse.base.*;
import com.jclark.xml.sax.*;
import gnu.trove.*;

/**
 * @author thorsten
 *  extend this class to be able to store objects in a file and to recreate them later.
 */
public class XMLStorable {
	
	private THashMap table = new THashMap();
	private File settingsFile = null;
	
	public XMLStorable(File settingsfile){
		settingsFile = settingsfile;
	}
	protected XMLStorable(){
	}

	/**
	 *  return an array of all stored keys
	 */
	public String[] getAllKeys(){
		Set keys = table.keySet();
		return (String[])keys.toArray(new String[0]);	
	}
	
	/**
	 *  store a value with a key.
	 */
	public void setValue(String key, String value){
		table.put(key,value);
	}
	public void setValue(String key, int value){
		table.put(key,value+"");
	}
	
	public void removeKey(String key){
		table.remove(key);
	}

	/**
	 *  get a stored value if existant.
	 */
	public String getValue(String key){
		// <COMPATIBILITY  TO VERSION 0.3.2 AND LOWER>
		if(key.equals("HALFLIFE_DIR"))
			return getValue(searchtool.wrappers.SearchtoolSettings.GAME_DIR);
		// </COMPATIBILITY>
		return (String)table.get(key);	
	}
	
	/**
	 *  get a value as int if possible
	 */
	 public int getValueAsInt(String key){
	 	try{
	 		return Integer.parseInt(getValue(key));
	 	}catch(Exception e){
	 		//e.printStackTrace();
	 		return -1;
	 	}	
	 }

	/**
	 *  load values from file.  
	 */
	public void load() {
		MyXMLParserCallback parser = new MyXMLParserCallback();
		try {
			EntityManagerImpl entityManagerImpl = new EntityManagerImpl();
			OpenEntity openEntity = entityManagerImpl.openFile(settingsFile.getAbsolutePath());
			ParserImpl parserImpl = new ParserImpl();
			parserImpl.setApplication(parser);
			parserImpl.setEntityManager(entityManagerImpl);
			parserImpl.setLocale(Locale.getDefault());
			parserImpl.parseDocument(openEntity);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	/**
	 *  save the values of the object to a given file.
	 */
	public void save() {
		try {
			MyXMLWriter writer = new MyXMLWriter(new FileOutputStream(settingsFile));
			writer.saveData(table);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 *  this class takes handles the loading of the values from file.
	 */
	private class MyXMLParserCallback extends ApplicationImpl {

		public void startElement(StartElementEvent e) {
			if (e.getName().equals("VALUES")) {
				int count = e.getAttributeCount();
				for (int i = 0; i < count; i++) {
					//table.put(e.getAttributeName(i),e.getAttributeValue(i));
					setValue(e.getAttributeName(i),e.getAttributeValue(i));
				}
			}
		}
	}

	/**
	 *  this class takes care of writing values to file
	 */
	private class MyXMLWriter extends UTF8XMLWriter {
		protected MyXMLWriter(FileOutputStream settingsfile) {
			super(settingsfile);
		}

		private void saveData(THashMap table) throws IOException {
			super.startElement("DATA");
			super.startElement("VALUES");
			Iterator iterator = table.keySet().iterator();
			Object key;
			while(iterator.hasNext()){
				key = iterator.next();
				super.attribute((String)key,(String)table.get(key));
			}
			super.write("\n");
			super.endElement("VALUES");
			super.endElement("DATA");
			super.close();
		}
	}
}
