/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

package searchtool.wrappers;

import searchtool.wrappers.*;
import searchtool.interfaces.*;
import java.util.*;
// thirdparty
import gnu.trove.*;

/**
 * @author thorsten
 *
 */
public class SearchtoolPlayer implements Player{

	// the name of the player this object represents
	private String playerName = "";
	// helps creating unique ids
	private static int idCounter = 0;
	// the id itself
	private long id = 0;
	// the server element
	private ServerElement server;

	public SearchtoolPlayer(String playerName, ServerElement server) {
		this.playerName = playerName;
		this.server = server;
		id = createId();
	}
	
	public void setServerElement(ServerElement element){
		this.server = element;
	}
	
	public ServerElement getServerElement(){
		return server;
	}

	public int compareTo(Object O) {
		int returnMe = server.compareTo(((SearchtoolPlayer)O).getServerElement());
		if( returnMe == 0 )
			return playerName.compareTo( ((SearchtoolPlayer)O).getPlayerName() );
		else
			return returnMe;
	}

	public boolean equals(Object O) {
		SearchtoolPlayer o = (SearchtoolPlayer) O;
		return (id == o.getId());
	}

	private synchronized static int createId() {
		idCounter++;
		return idCounter;
	}

	/**
	 * Returns the playerName.
	 * @return String
	 */
	public String getPlayerName() {
		return playerName;
	}

	/**
	 * Sets the playerName.
	 * @param playerName The playerName to set
	 */
	public void setPlayerName(String playerName) {
		this.playerName = playerName;
	}

	/**
	 * Returns the id.
	 * @return long
	 */
	public long getId() {
		return id;
	}

}
