/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
package searchtool.wrappers;

import searchtool.*;
import searchtool.interfaces.*;
import searchtool.wrappers.*;
import searchtool.util.*;
import java.io.*;
import java.lang.*;
import java.util.*;

/**
 * @author thorsten
 * 
 *
 */
public class SearchtoolSettings extends XMLStorable {
	// the value than can be bound to a doubleclick on a server.
	public static final int MAKE_FAVOURITE = 0, 
	MAKE_BLACKLIST = 1, 
	CONNECT = 2, 
	GET_DETAILS = 3;
	// possible values of the filter variables
	public static final int ENABLED=1,
	DISABLED=0;
	// a vector with registered listeners
	private Vector listenerVector = new Vector();
	
	public static String LOOK_AND_FEEL = "LOOK_AND_FEEL",
		MASTERSERVER_IP = "MASTERSERVER_IP",
		MASTERSERVER_PORT = "MASTERSERVER_PORT",
		CONNECT_COMMAND = "CONNECT_COMMAND",
		SEARCHSTRING = "SEARCHSTRING",
		GAME_DIR = "GAME_DIR",
		MASTERLEVEL_FILTER = "MASTERLEVEL_FILTER",
		FILTERED_MOD = "FILTERED_MOD",
		FILTERED_MAP = "FILTERED_MAP",
		FILTER_STRING = "FILTER_STRING",
		FILTER_EMPTY = "FILTER_EMPTY",
		FILTER_FULL = "FILTER_FULL",
		FILTER_PUBLIC_SERVERS = "FILTER_PUBLIC_SERVERS",
		FILTER_PRIVATE_SERVERS = "FILTER_PRIVATE_SERVERS",
		FILTER_NOTPINGED_SERVERS = "FILTER_NOTPINGED_SERVERS",
		SHUTDOWN_TIMEOUT = "SHUTDOWN_TIMEOUT",
		SORTANDFILTER_INTERVAL = "SORTANDFILTER_INTERVAL",
		SIMULTANEOUS_QUERYS = "SIMULTANEOUS_QUERYS",
		DOUBLECLICK_BIND = "DOUBLECLICK_BIND",
		SELECTED_COLUMN = "SELECTED_COLUMN",
		DEBUG_FILE = "DEBUG_FILE",
		SLEEPING = "SLEEPING",
		CURRENT_GAME = "CURRENT_GAME";
	
	public SearchtoolSettings(File settingsfile){
		super(settingsfile);
		this.initValues();
	}
		
	protected void initValues(){
		// global
		setValue(SORTANDFILTER_INTERVAL, "1500");
		setValue(SIMULTANEOUS_QUERYS, "40");
		setValue(DOUBLECLICK_BIND, "3");
		setValue(LOOK_AND_FEEL,"Kunststoff");
		setValue(CURRENT_GAME,Searchtool.Constants.HALFLIFE_PREFIX);
		// game specific
		setValue(FILTER_EMPTY,ENABLED);
		setValue(FILTER_FULL,ENABLED);
		setValue(FILTER_PRIVATE_SERVERS,ENABLED);
		setValue(FILTER_PUBLIC_SERVERS,ENABLED);
		setValue(FILTER_STRING,"");
		
	}
	
	public void addSettingsListener(SettingsListener listener){
		listenerVector.add(listener);
	}
	
	public void removeSettingsListener(SettingsListener listener){
		for(int i =0; i < listenerVector.size(); i++){
			if(listenerVector.get(i).equals(listener))
				listenerVector.remove(i);
		}
	}
	
	/**
	 *  overridden because of listeners
	 */
	public void setValue(String key, String value){
		super.setValue(key,value);
		notifyListeners(key);
	}
	
	/**
	 *  overridden because of listeners
	 */
	public void setValue(String key, int value){
		super.setValue(key,value);
		notifyListeners(key);
	}
	
	/**
	 *  notifys every registered listener of a change
	 */
	private void notifyListeners(String key){
		for(int i =0; i < listenerVector.size(); i++)
			((SettingsListener)listenerVector.get(i)).settingsChanged(key);	
	}
	
	public String[] getInstalledMaps(){
		return new String[0];
	}
	
	public String[] getInstalledMods(){
		return new String[0];
	}
	
	public String getConnectPostfix(ServerElement connectToMe){
		return "";
	}
	
}
