/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * ServerElement.java
 *
 * Created on December 16, 2001, 2:59 PM
 */

package searchtool.wrappers;

import javax.swing.*;
import javax.swing.event.*;
import searchtool.interfaces.*;
import searchtool.wrappers.*;
import searchtool.*;
import java.util.*;
import java.text.*;
import java.io.*;
// thirdparty
import com.jclark.xml.parse.*;
import com.jclark.xml.output.*;
import com.jclark.xml.parse.base.*;
import com.jclark.xml.sax.*;
import gnu.trove.*;

/**
 * @author  zoidberg
 * @version
 */
public class ServerElement implements Element {

	public final static Byte /*OLD,*/ RESPONSE_POSITIVE, RESPONSE_NEGATIVE, SPECIAL, PASSWD_REQUIRED, NO_PASSWD;
	public final static Boolean IS_MEMBER, NO_MEMBER;
	// some status bytes the server can have. (more than one at a time)
	static {
		//OLD = new Byte((byte) 0);
		RESPONSE_POSITIVE = new Byte((byte) 1);
		RESPONSE_NEGATIVE = new Byte((byte) 0) /*new Byte((byte) 2)*/;
		SPECIAL = new Byte((byte) 3);
		PASSWD_REQUIRED = new Byte((byte) 0);
		NO_PASSWD = new Byte((byte) 1);
		IS_MEMBER = new Boolean(true);
		NO_MEMBER = new Boolean(false);
	}

	// this object wraps the ip and the (query)port of this server.
	private IpWrapper ipWrapper = null;
	// the actual gameport
	private int gameport = 7777;
	// the ping
	private Integer ping = new Integer(9999);
	// the maximum number of players
	private Byte maxplayers = new Byte((byte) 0);
	// the number of currently playing people
	private Byte currentplayers = new Byte((byte) 0);
	// passworded server or not
	private Byte password = NO_PASSWD;
	// responsive server or not
	private Byte response = RESPONSE_NEGATIVE;
	// the map running on the server
	private String map = "-";
	// for better getValueAt performance in SearchtoolTableModel !
	private String ipAndPort = "";
	// the servername
	private String servername = "-";
	// whether the server is a member of the blacklist 
	private Boolean blackList = NO_MEMBER;
	// whether the server is a member of favouritelist
	private Boolean favouriteList = NO_MEMBER;
	// the the user connected to this server the last time
	private Date connectionDate = new Date(1); // init
	// this byte sets whether this element is part of the last connected list or not.
	private Boolean lastConnectedElement = NO_MEMBER;
	// the rules of this server
	private THashMap rulesMap = new THashMap();
	// the list of players on this server
	private String[] currentPlayerList = new String[0]; // init
	// the mod of this game.
	private String modName = "default"; // init
	// the server type
	private String serverType = "unknown"; // init
	// the operatingsystem
	private String operatingSystem = "unknown"; // init

	/** Creates new ServerElement */

	public ServerElement() {
		this(new IpWrapper(127, 0, 0, 1, 1));
	}
	public ServerElement(IpWrapper ipWrapper) {
		this.ipWrapper = ipWrapper;
		createIpAndPortValue();
	}

	public ServerElement(IpWrapper ipWrapper, String SERVERNAME, int PING, String MAP, int MAXPLAYERS, int CURRENTPLAYERS, Byte PASSWORD) {
		this(ipWrapper);
		servername = SERVERNAME;
		ping = new Integer(PING);
		map = MAP;
		maxplayers = new Byte((byte) MAXPLAYERS);
		currentplayers = new Byte((byte) CURRENTPLAYERS);
		password = PASSWORD;
	}

	public synchronized ServerElement getNewServerElement() {
		return new ServerElement();
	}

	public synchronized ServerElement getNewServerElement(IpWrapper ipWrapper) {
		return new ServerElement(ipWrapper);
	}

	public boolean equals(Object O) {
		return (ipWrapper.id == ((ServerElement) O).getIpWrapper().id);
	}

	/*
	 *  compare only ip and port.
	 */
	public int compareTo(Object O) {
		return ipWrapper.compareTo(((ServerElement) O).ipWrapper);
	}

	/*
	 * get
	 */
	public Date getDate() {
		return connectionDate;
	}
	public String getIpAndPort() {
		return ipAndPort;
	}
	public boolean isFavourite() {
		return (favouriteList.booleanValue());
	}
	public boolean isBlacklist() {
		return (blackList.booleanValue());
	}
	public boolean isResultlist() {
		return false; // fixme!
	}
	public boolean isLastConnectedlist() {
		return (lastConnectedElement.booleanValue());
	}
	public String getIp() {
		return ipWrapper.ip1 + "." + ipWrapper.ip2 + "." + ipWrapper.ip3 + "." + ipWrapper.ip4;
	}
	public Integer getPort() {
		return new Integer(ipWrapper.port);
	}
	public String getServername() {
		return servername;
	}
	public String getMap() {
		return map;
	}
	public Integer getPing() {
		return ping;
	}
	public Byte getMaxplayers() {
		return maxplayers;
	}
	public Byte getCurrentplayers() {
		return currentplayers;
	}
	public Byte getPassword() {
		return password;
	}
	public Byte getResponse() {
		return response;
	}
	public IpWrapper getIpWrapper() {
		return ipWrapper;
	}
	public long getId() {
		return ipWrapper.id;
	}
	public THashMap getRulesMap() {
		return rulesMap;
	}
	public String[] getCurrentPlayerList() {
		return currentPlayerList;
	}
	public String getMod() {
		return modName;
	}
	public String getServerType() {
		return serverType;
	}
	public String getOperatingSystem() {
		return operatingSystem;
	}
	public int getGamePort() {
		return gameport;
	}

	/*
	 * set
	 */
	public void setGamePort(int gameport) {
		this.gameport = gameport;
	}
	public void setDate(Date NEWDATE) {
		connectionDate = NEWDATE;
	}
	public void createIpAndPortValue() {
		ipAndPort = ipWrapper.ip1 + "." + ipWrapper.ip2 + "." + ipWrapper.ip3 + "." + ipWrapper.ip4 + ":" + ipWrapper.port;
		gameport = ipWrapper.port; // default
	}
	public void setFavouritelist(boolean value) {
		if (value) {
			blackList = NO_MEMBER;
			favouriteList = IS_MEMBER;
		} else
			favouriteList = NO_MEMBER;
	}
	public void setBlacklist(boolean value) {
		if (value) {
			favouriteList = NO_MEMBER;
			blackList = IS_MEMBER;
		} else
			blackList = NO_MEMBER;
	}
	public void setLastConnectedlist(boolean VALUE) {
		if (VALUE)
			lastConnectedElement = IS_MEMBER;
		else
			lastConnectedElement = NO_MEMBER;
	}
	public void setResultlist(boolean VALUE) {
		; // fixme!
	}
	public void setServername(String SERVERNAME) {
		// for some reason a couple of servernames (halfllife at least) include a carriage return which can really mess up
		// the parsing of this string once we load it from file.
		servername = removeWhitespacesAndAsciiControl(SERVERNAME);
	}
	private int tmpIndex = 0;
	protected String removeWhitespacesAndAsciiControl(String string) {
		string = string.trim(); // leading-trailing whitespaces
		for (int i = 0; i <= 31; i++) {
			// damn ascii control s*** .   WOULD YOU SERVER-ADMINS PLEASE NOT (NOT!) ADD ASCII CONTROL CHARACTERS TO YOUR SERVERNAMES!!!!!!
			tmpIndex = string.indexOf(i);
			if (tmpIndex >= 0) {
				string = string.substring(0, tmpIndex) + string.substring(tmpIndex + 2, string.length());
				i--;
			}
		}
		return string;
	}
	public void setMap(String MAP) {
		map = MAP.trim();
	}
	public void setPing(int PING) {
		ping = new Integer(PING);
	}
	public void setMaxplayers(int MAXPLAYERS) {
		maxplayers = new Byte((byte) MAXPLAYERS);
	}
	public void setCurrentplayers(int CURRENTPLAYERS) {
		currentplayers = new Byte((byte) CURRENTPLAYERS);
	}
	public void setPassword(byte PASSWORD) {
		if (PASSWORD == NO_PASSWD.byteValue())
			password = NO_PASSWD;
		else
			password = PASSWD_REQUIRED;
	}
	public void setPassword(Byte PASSWORD) {
		password = PASSWORD;
	}
	public void setResponse(Byte VALUE) {
		response = VALUE;
	}
	protected void setIpWrapper(IpWrapper ipWrapper) {
		this.ipWrapper = ipWrapper;
		createIpAndPortValue();
	}
	public void setRulesMap(THashMap rulesMap) {
		this.rulesMap = rulesMap;
	}
	public void setCurrentPlayerList(String[] VALUE) {
		currentPlayerList = VALUE;
	}
	public void setMod(String modName) {
		this.modName = modName;
	}
	public void setServerType(String serverType) {
		this.serverType = serverType;
	}
	public void setOperatingSystem(String operatingSystem) {
		this.operatingSystem = operatingSystem;
	}

	/**
	 *  this class knows how to handle saved data.  subclasses of ServerElement should also create
	 * a (inner) class that does extend this one.
	 */
	public static class ListDataElementParserCallback extends ApplicationImpl {

		private Vector tmpElementStorage = new Vector(0);
		protected ServerElement tmpDataElement = new ServerElement();
		private JProgressBar progressBar = null;

		public ListDataElementParserCallback() {
		}

		public Vector getLoadedList() {
			return tmpElementStorage;
		}

		public void registerProgressBar(JProgressBar proggie) {
			progressBar = proggie;
		}

		protected final void increaseStatusBar() {
			if (progressBar != null) {
				progressBar.setValue(progressBar.getValue() + 1);
				progressBar.validate();
			}
		}
		protected final void initializeStatusBar(int SIZE) {
			if (progressBar != null) {
				progressBar.setMaximum(SIZE);
				progressBar.setValue(0);
				progressBar.validate();
			}
		}
		protected final boolean isProgressBarRegistered() {
			if (progressBar == null)
				return false;
			else
				return true;
		}

		/**
		 *  called at start of element
		 *  should be overridden by extending classes.
		 */

		public void startElement(StartElementEvent e) {
			if (e.getName().equals(Searchtool.Constants.serverTag)) {
				ServerElement tmp = parseElement(e);
				storeElement(tmp);
			} else if (e.getName().equals(Searchtool.Constants.serverCountTag)) {
				if (isProgressBarRegistered()) {
					try {
						initializeStatusBar(Integer.parseInt(e.getAttributeValue(Searchtool.Constants.numberAtt)));
					} catch (Exception f) {
						f.printStackTrace();
					}
				}
			}
		}

		protected ServerElement parseElement(StartElementEvent e) {
			try {
				String value = null;
				int ip1 = 0, ip2 = 0, ip3 = 0, ip4 = 0, port = 0;
				ServerElement listDataElement = new ServerElement();

				value = e.getAttributeValue("IP1");
				if (value != null) {
					ip1 = Integer.parseInt(value);
					value = null;
				}
				value = e.getAttributeValue("IP2");
				if (value != null) {
					ip2 = Integer.parseInt(value);
					value = null;
				}
				value = e.getAttributeValue("IP3");
				if (value != null) {
					ip3 = Integer.parseInt(value);
					value = null;
				}
				value = e.getAttributeValue("IP4");
				if (value != null) {
					ip4 = Integer.parseInt(value);
					value = null;
				}
				value = e.getAttributeValue("PRT");
				if (value != null) {
					port = Integer.parseInt(value);
					value = null;
				}
				listDataElement.setIpWrapper(new IpWrapper(ip1, ip2, ip3, ip4, port));
				value = e.getAttributeValue("PNG");
				if (value != null) {
					listDataElement.setPing(Integer.parseInt(value));
					value = null;
				}
				value = e.getAttributeValue("MAP");
				if (value != null) {
					listDataElement.setMap(value);
					value = null;
				}
				value = e.getAttributeValue("NME");
				if (value != null) {
					listDataElement.setServername(value);
					value = null;
				}
				value = e.getAttributeValue("CRP");
				if (value != null) {
					listDataElement.setCurrentplayers(Integer.parseInt(value));
					value = null;
				}
				value = e.getAttributeValue("MXP");
				if (value != null) {
					listDataElement.setMaxplayers(Integer.parseInt(value));
					value = null;
				}
				value = e.getAttributeValue("PWD");
				if (value != null) {
					int tmp = Integer.parseInt(value);
					if (tmp == ServerElement.NO_PASSWD.byteValue())
						listDataElement.setPassword(ServerElement.NO_PASSWD);
					else if (tmp == ServerElement.PASSWD_REQUIRED.byteValue())
						listDataElement.setPassword(ServerElement.PASSWD_REQUIRED);
					value = null;
				}
				value = e.getAttributeValue("DTE");
				if (value != null) {
					listDataElement.setDate(new Date(Long.parseLong(value)));
					value = null;
				}
				value = e.getAttributeValue("HST");
				if (value != null) {
					listDataElement.setLastConnectedlist(Boolean.valueOf(value).booleanValue());
					value = null;
				}
				value = e.getAttributeValue("BLK");
				if (value != null) {
					listDataElement.setBlacklist(Boolean.valueOf(value).booleanValue());
					value = null;
				}
				value = e.getAttributeValue("FVR");
				if (value != null) {
					listDataElement.setFavouritelist(Boolean.valueOf(value).booleanValue());
					value = null;
				}
				
				// <since 0.4.2>
				value = e.getAttributeValue("GMP");
				if (value != null) {
					listDataElement.setGamePort(Integer.parseInt(value));
					value = null;
				}		
				// </since 0.4.2>
				
				// <since 0.4.3>
				value = e.getAttributeValue("MOD");
				if (value != null) {
					listDataElement.setMod( value );
					value = null;
				}		
				// </since 0.4.3>
				
				increaseStatusBar();
				return listDataElement;
			} catch (Exception f) {
				increaseStatusBar();
				System.out.println("could not parse element " + f.getMessage());
				return null;
			}
		}

		protected final void storeElement(ServerElement e) {
			tmpElementStorage.add(e);
		}

		/**
		 *  called at end of element.
		 *  should not be overridden by extending classes.
		 */
		public final void endElement(EndElementEvent e) {
			;
		}
	}

	/**
	 * this class stores the content of one ServerElement in a
	 * 'xml object' .  classes that extend ServerElement should also take
	 * care to extend this class to be able to store additional data.
	 */
	public static class ListDataElementXMLWriter {

		protected UTF8XMLWriter xmlWriter = null;

		public ListDataElementXMLWriter(File serverfile) {
			try {
				xmlWriter = new UTF8XMLWriter(new FileOutputStream(serverfile));
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		public final void startOutput(int LENGTH_OF_LIST) {
			try {
				xmlWriter.comment(Searchtool.Constants.commentString);
				xmlWriter.write("\n");
				xmlWriter.startElement(Searchtool.Constants.serverListTag);
				xmlWriter.write("\n");
				xmlWriter.startElement(Searchtool.Constants.versionTag);
				xmlWriter.attribute(Searchtool.Constants.nameAtt, Searchtool.Constants.serverListVersion);
				xmlWriter.endElement(Searchtool.Constants.versionTag);
				xmlWriter.write("\n");
				xmlWriter.startElement(Searchtool.Constants.elementTypeTag);
				xmlWriter.write(Searchtool.Constants.elementType);
				xmlWriter.endElement(Searchtool.Constants.elementTypeTag);
				xmlWriter.write("\n");
				xmlWriter.startElement(Searchtool.Constants.serverCountTag);
				xmlWriter.attribute(Searchtool.Constants.numberAtt, LENGTH_OF_LIST + "");
				xmlWriter.endElement(Searchtool.Constants.serverCountTag);
				xmlWriter.write("\n");
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		/**
		 *  extending classes have to override this method unless they do not save more data (which does not make sense).
		 */
		public void writeElement(ServerElement saveMe) {
			try {
				xmlWriter.attribute("IP1", saveMe.getIpWrapper().ip1 + "");
				xmlWriter.attribute("IP2", saveMe.getIpWrapper().ip2 + "");
				xmlWriter.attribute("IP3", saveMe.getIpWrapper().ip3 + "");
				xmlWriter.attribute("IP4", saveMe.getIpWrapper().ip4 + "");
				xmlWriter.attribute("PRT", saveMe.getPort() + "");
				xmlWriter.attribute("PNG", saveMe.getPing() + "");
				xmlWriter.attribute("NME", saveMe.getServername());
				xmlWriter.attribute("MAP", saveMe.getMap());
				xmlWriter.attribute("CRP", saveMe.getCurrentplayers() + "");
				xmlWriter.attribute("MXP", saveMe.getMaxplayers() + "");
				xmlWriter.attribute("PWD", saveMe.getPassword().byteValue() + "");
				xmlWriter.attribute("DTE", saveMe.getDate().getTime() + "");
				xmlWriter.attribute("HST", new Boolean(saveMe.isLastConnectedlist()).toString());
				xmlWriter.attribute("FVR", new Boolean(saveMe.isFavourite()).toString());
				xmlWriter.attribute("BLK", new Boolean(saveMe.isBlacklist()).toString());
				
				// <since 0.4.2>
				xmlWriter.attribute("GMP", saveMe.getGamePort()+"");
				// </since 0.4.2>
				
				// <since 0.4.3>
				xmlWriter.attribute("MOD", saveMe.getMod());
				// </since 0.4.3>
				
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		public final void startListElement() {
			try {
				xmlWriter.startElement(Searchtool.Constants.serverTag);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		public final void endListElement() {
			try {
				xmlWriter.endElement(Searchtool.Constants.serverTag);
				xmlWriter.write("\n");
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		public final void endOutput() {
			try {
				xmlWriter.endElement(Searchtool.Constants.serverListTag);
				xmlWriter.close();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
}
